; SpeechHub SomeTTS synthesizer Installer.nsi
; This is a template installer that can be modified to suit most synthesizers  
; Author Isaac Porat based on contribution from numerous sources on the NSIS website   
; This installer ensures that:
; SpeechHub is installed and it is above a minimum version required for compatibility
; SpeechHub is not running
; The synthesizer version is checked against previous versions and the user is informed accordingly
; Lines that might need modification are marked as:
; Modify the next line to suit - required or optional

; SomeTTS synthesizer  installer version
; Modify the next line to suit - required
!define VERSION "0.1.0.0"
; Minimum SH version that this synthesizer version is compatible with
; Modify the next line to suit - required
!define SH_MIN_VERSION "0.4.0.0"

; Execution level for the installer, adnin required  
RequestExecutionLevel admin
ShowInstDetails "nevershow"
ShowUninstDetails "nevershow"
; Compressor level commented to allow faster installer development
; For production for maximum compression /SOLID LZMA recommended
;SetCompressor /SOLID LZMA
CRCCheck On
XPStyle on
InstProgressFlags Smooth

!include "FileFunc.nsh"
!insertmacro GetFileVersion
!include "WordFunc.nsh"
!insertmacro VersionCompare

!define BRAND "SpeechHub"
; In case of single synthesizer Product name is usually the name of the synthesizer
; In case of multiple synthesizers installer the product name might be different 
; Modify the next line to suit - required
!define PRODUCT_NAME "SomeTTS"
; Engine refers to synthesizer in SH
; Engine directory in SH must be the lower case of the engine name
; together with a three parts  version number
; separated by a '-'
; In case of multiple synthesizers installer lines for each must be added
; here, and in the installer and uninstaller sections to add and remove these
; Modify the next line to suit - required
!define ENGINE_DIR "sometts-0.1.0"

!define PROD_SH "SpeechServer"
; Directory read write of SH used to verify that it is shut down before installation
!define RW_DIR "$APPDATA\${BRAND}\${PROD_SH}"
!define PROD_DIR "C:\${BRAND}_engine_installer_dir\Windows" 
; License
!define LICENSE_TEXT "License for ${PRODUCT_NAME}"
!define LICENSE_FILE "${PRODUCT_NAME}_License.txt" 

; The name of the installer
Name "${PRODUCT_NAME} ${VERSION}"
Caption "${PRODUCT_NAME} ${VERSION}"
; Icon "${PRODUCT_NAME}.ico"

; Installer file 
OutFile "SH-${PRODUCT_NAME}Install.exe"

; Installer file properties
VIAddVersionKey "ProductName" "${PRODUCT_NAME}"
; Modify the next line to suit - optional
VIAddVersionKey "Comments" "${BRAND} ${PRODUCT_NAME} installer"
; Modify the next line to suit - optional
VIAddVersionKey "CompanyName" "Not applicable"
; Modify the next line to suit - optional
VIAddVersionKey "LegalTrademarks" "Not applicable"
; Modify the next line to suit - optional
VIAddVersionKey "LegalCopyright" "See copyright with installed synthesizer"
VIAddVersionKey "FileDescription" "${PRODUCT_NAME} ${BRAND} Windows installer"
VIAddVersionKey "FileVersion" "${VERSION}"
VIProductVersion "${VERSION}"

; The default installation directory
; 32-bit
; C:Program Files
; 64-bit
; C:Program Files (x86)
InstallDir $PROGRAMFILES\${BRAND}

; Check and assign InstallDir if selected in previous installation
; This was disabled for simplicity - users have no choice of installation directory, always default 
;InstallDirRegKey HKLM "Software\${PRODUCT_NAME}" "Install_Dir"
; The text to prompt the user to enter a directory
; This was disabled for simplicity - users have no choice of directory installation, always C:Program Files
;DirText "This will install the ${PRODUCT_NAME} synthesizer on your computer. Choose an installation directory"

; Pages
 
 Page license
LicenseText "${LICENSE_TEXT}"
LicenseData "${PROD_DIR}\${BRAND}\${LICENSE_FILE}"

Page instfiles

UninstPage uninstConfirm
UninstPage instfiles

Section -CheckNotRunning
; Obtain Application data path and maintain context all users 
SetShellVarContext current
StrCpy $R7 ${RW_DIR} 
SetShellVarContext all
; Try to delete instance file, if not successeful it means instance is running  
ClearErrors
Delete "$R7\instance.lock"
 IfErrors 0 notrunning
MessageBox MB_OK "${BRAND} server is running, installation aborted. $\n$\n Make sure the server is shutdown and try again."
Abort
notrunning:
SectionEnd

Section -CheckVersion
SetDetailsPrint none
; Get the previous SH version uninstaller if exists
ReadRegStr $R0 HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${BRAND}" "UninstallString"
; Abort if not installed before
StrCmp $R0 "" 0 proceed
MessageBox MB_OK "SpeechHub is not installed, install SpeechHub first and then try to install this synthesizer again!"
Abort
proceed:
; Read previous SH version
ReadRegStr $R1 HKLM SOFTWARE\${BRAND} "Version"
${VersionCompare} ${SH_MIN_VERSION} $R1 $R2
; Result in $R2 0 - the same, 1 first version is newer, 2 - second version is newer 
; Check whether $R2!= "2" or "0"
StrCmp $R2 "2" enginecheck 0
StrCmp $R2 "0" enginecheck 0
MessageBox MB_ICONSTOP "The version of ${BRAND} installed on your computer is too old for this version of ${PRODUCT_NAME}. $\n$\n  Upgrade ${BRAND} to the latest version and try again."
Abort
enginecheck:
/*
* A developer reported that the following key needs to be checked in Windows 7 64 - needs to confirm:
* So far no reported problems
*HKLM  \SOFTWARE\Wow6432Node\Microsoft\Windows\CurrentVersion\Uninstall\${BRAND}" \
  *"UninstallString"
*/
 ; Get the previous engine version uninstaller  if exists
 ReadRegStr $R0 HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\SH-${PRODUCT_NAME}" "UninstallString"
 ; Continue if not installed before
 StrCmp $R0 "" done
; Read previous engine version
ReadRegStr $R1 HKLM SOFTWARE\SH-${PRODUCT_NAME} "Version"
${VersionCompare} $R1 ${VERSION} $R2
; Result in $R2 0 - the same, 1 first version is newer, 2 - second version is newer 
; Check whether $R2!= "2"
StrCmp $R2 "2" upgrade 0
MessageBox MB_OKCANCEL|MB_ICONEXCLAMATION "This ${PRODUCT_NAME} installer version is older or the same as the one installed already on your computer. $\n$\n Replace the existing version anyway?"  IDOK uninst
Abort
upgrade:
MessageBox MB_OKCANCEL|MB_ICONEXCLAMATION "${PRODUCT_NAME} is already installed. $\n$\nSelect 'OK' to remove the previous version or 'Cancel' to cancel this upgrade." IDOK uninst
Abort
;Run the uninstaller in silent mode as the user just confirmed
uninst:
ClearErrors
ExecWait '$R0 /S _?=$INSTDIR' ;Do not copy the uninstaller to a temp file - see uninstaller command line options
IfErrors +1 done
; Do nothing allow installation on top of the previous one
; Note that a check for server running was done in the previous section 
MessageBox MB_OK "Error detected during uninstall of the previous version, will attempt to install on top overriding the previous installation"
done:

SectionEnd

Section -SETTINGS
; Set output path to the installation directory.
SetOutPath "$INSTDIR"
SectionEnd

; Copy files and create start menus
Section -INSTALLATION

SetDetailsPrint none
;  All users
SetShellVarContext all

; Set output path to the engine directory
SetOutPath $INSTDIR\sh\engine
; Copy the engine
File /r "${PROD_DIR}\${BRAND}\sh\engine\${ENGINE_DIR}"
; If the installer includes more than one synthesizers copy them here

; Set output path back to the installation directory
SetOutPath $INSTDIR
File "${PROD_DIR}\${BRAND}\${LICENSE_FILE}"

; Create start menu shortcuts
CreateDirectory "$SMPROGRAMS\${BRAND}"
CreateShortCut "$SMPROGRAMS\${BRAND}\Uninstall ${PRODUCT_NAME}.lnk" "$INSTDIR\SH-${PRODUCT_NAME}Uninstall.exe"

; Registry 
; Write the installation path
WriteRegStr HKLM SOFTWARE\SH-${PRODUCT_NAME} "Install_Dir" "$INSTDIR"
; Write the version 
WriteRegStr HKLM SOFTWARE\SH-${PRODUCT_NAME} "Version" "${VERSION}"

; Write the uninstall keys for Windows
WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\SH-${PRODUCT_NAME}" "DisplayName" "SpeechHub ${PRODUCT_NAME}"
WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\SH-${PRODUCT_NAME}" "UninstallString" '"$INSTDIR\SH-${PRODUCT_NAME}Uninstall.exe"'
WriteRegDWORD HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\SH-${PRODUCT_NAME}" "NoModify" 1
WriteRegDWORD HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\SH-${PRODUCT_NAME}" "NoRepair" 1
  
; Tell the compiler to write an uninstaller and to look for a "Uninstall" section 
WriteUninstaller $INSTDIR\SH-${PRODUCT_NAME}Uninstall.exe
; Set output path back to the installation directory
SetOutPath $INSTDIR

SectionEnd 

; The uninstall section
Section "Uninstall"

SetDetailsPrint none
; All users, this helps start menus removal for Windows Vista and Windows 7
; Obtain Application data path and maintain context all users 
SetShellVarContext current
StrCpy $R7 ${RW_DIR} 
SetShellVarContext all
; Try to delete instance file, if not successeful it means instance is running  
ClearErrors
Delete "$R7\instance.lock"
 IfErrors 0 notrunning
MessageBox MB_OK "${BRAND} server is running, uninstall aborted. $\n$\n Make sure the server is shutdown and try again."
Abort
notrunning:

; Remove registry keys
DeleteRegKey HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\SH-${PRODUCT_NAME}"
DeleteRegKey HKLM SOFTWARE\SH-${PRODUCT_NAME}

; Delete Start Menu entry
Delete "$SMPROGRAMS\${BRAND}\Uninstall ${PRODUCT_NAME}.lnk"

; Delete installed files 
RmDir /r $INSTDIR\sh\engine\${ENGINE_DIR}
; If the installer includes more than one synthesizers delete them here

Delete $INSTDIR\${LICENSE_FILE}
Delete $INSTDIR\SH-${PRODUCT_NAME}Uninstall.exe

SectionEnd 

; MessageBox for debugging example
;MessageBox MB_OK "Hello this is a useful message"
