; JRE handler with automatic JRE installation
; Put together by Isaac Porat Based with some modifications on code published here:
; http://nsis.sourceforge.net/Java_Launcher_with_automatic_JRE_installation
; Thanks to all contributors

; JRE definitions
!define JRE_MINIMUM_VERSION "6.0"
!define JRE_DOWNLOAD_URL "http://www.speechhub.org/SpeechHub/install-w/jre-7-windows-i586-iftw.exe"

; Use javaw to avoid DosBox (STDOUT and STDERR suppressed)
!define JAVA_EXE "javaw.exe"

!include "FileFunc.nsh"
!insertmacro GetFileVersion
!include "WordFunc.nsh"
!insertmacro VersionCompare

;  returns the full path of a valid java.exe
;  looks in:
;  1 - .\jre directory (JRE Installed with application)
;  2 - JAVA_HOME environment variable
;  3 - the registry
;  4 - hopes it is in current dir or PATH
Function GetJRE 
    Push $R0
    Push $R1
    Push $2
    
    ; 1) Check local JRE
    ClearErrors
    StrCpy $R0 "$EXEDIR\jre\bin\${JAVA_EXE}"
    IfFileExists $R0 JreFound
    
    ; 2) Check for JAVA_HOME
    ClearErrors
    ReadEnvStr $R0 "JAVA_HOME"
    StrCpy $R0 "$R0\bin\${JAVA_EXE}"
    IfErrors CheckRegistry     
    IfFileExists $R0 0 CheckRegistry
    Call CheckJREVersion
    IfErrors CheckRegistry JreFound
    
    ; 3) Check for registry
  CheckRegistry:
    ClearErrors
    ReadRegStr $R1 HKLM "SOFTWARE\JavaSoft\Java Runtime Environment" "CurrentVersion"
    ReadRegStr $R0 HKLM "SOFTWARE\JavaSoft\Java Runtime Environment\$R1" "JavaHome"
    StrCpy $R0 "$R0\bin\${JAVA_EXE}"
    IfErrors JREDownload
    IfFileExists $R0 0 JREDownload
    Call CheckJREVersion
    IfErrors JREDownload JreFound
    
    JREDownload:
    
    ReadRegStr $R1 HKLM "SOFTWARE\JavaSoft\Java Runtime Environment" "CurrentVersion"
    ReadRegStr $R0 HKLM "SOFTWARE\JavaSoft\Java Runtime Environment\$R1" "JavaHome"
    StrCpy $R0 "$R0\bin\${JAVA_EXE}"
    IfFileExists $R0 0 JreNotFound
    Call CheckJREVersion
    IfErrors JreNotFound JreFound
    
    ; 4) Jre not found  
  JreNotFound:
    SetErrors
Return
    JreFound:
    Pop $2
    Pop $R1
    Exch $R0
FunctionEnd

; Pass the "javaw.exe" path by $R0
; Version needs to be the same or greater than version specified
Function CheckJREVersion
    Push $R1
    
    ; Get the file version of javaw.exe
    ${GetFileVersion} $R0 $R1
    ${VersionCompare} ${JRE_MINIMUM_VERSION} $R1 $R1
; Result in $R1 0 - the same, 1 first version is newer, 2 - second version is newer 
    ; Check whether $R1!= "1"
    ClearErrors
    StrCmp $R1 "1" 0 done
    SetErrors
done:
    Pop $R1

FunctionEnd

Function DownloadJRE
    StrCpy $2 "$TEMP\Java Runtime Environment.exe"
    nsisdl::download /TIMEOUT=30000 ${JRE_DOWNLOAD_URL} $2
    Pop $R0 ;Get the return value
    StrCmp $R0 "success" +3
      MessageBox MB_ICONSTOP "Download failed! $\n$\n  download Java from Oracle website, install and then try again."
      Abort
    ExecWait $2
    Delete $2
    
FunctionEnd

