; SpeechHubInstaller.nsi
; Author Isaac Porat based on contribution from numerous sources on the NSIS website   

; SpeechHub version
!define VERSION "0.4.0.0"

; Execution level for the installer, adnin required  
RequestExecutionLevel admin
ShowInstDetails "nevershow"
ShowUninstDetails "nevershow"
; Compressor level commented to allow faster installer development
; For production for maximum compression /SOLID LZMA recommended
;SetCompressor /SOLID LZMA
CRCCheck On
XPStyle on
InstProgressFlags Smooth

!include "JREHandler.nsh"

!define BRAND "SpeechHub"
!define PROD_SH "SpeechServer"
; Installation type
; TCF (Talking Computer Framework - OSGi)
;!define APP_DIR "tcf"
;!define SPEECH_SERVER_EXE "SpeechServer_t.exe"
; Independent 
!define APP_DIR "SpeechServer"
!define SPEECH_SERVER_EXE "SpeechServer_i.exe"
!define RW_DIR "$APPDATA\${BRAND}\${PROD_SH}"
!define PROD_DIR "C:\${BRAND}_installer_dir\Windows" 
; License
!define LICENSE_TEXT "License for ${BRAND}"
!define LICENSE_FILE "${BRAND}_License.txt" 

; The name of the installer
Name "${BRAND} ${VERSION}"
Caption "${BRAND} ${VERSION}"
; Icon "${BRAND}.ico"

; Installer file 
OutFile "${BRAND}Install.exe"

; Installer file properties
VIAddVersionKey "ProductName" "${BRAND}"
VIAddVersionKey "Comments" "${BRAND} installer"
VIAddVersionKey "CompanyName" "${BRAND} contributors"
VIAddVersionKey "LegalTrademarks" "${BRAND} is a brand name of its contributors"
VIAddVersionKey "LegalCopyright" "Copyright (c) Isaac Porat and Bill Cox 2011, All right reserved"
VIAddVersionKey "FileDescription" "${BRAND} Windows installer"
VIAddVersionKey "FileVersion" "${VERSION}"
VIProductVersion "${VERSION}"


; The default installation directory
InstallDir $PROGRAMFILES\${BRAND}

; Check and assign InstallDir if selected in previous installation
; This was disabled for simplicity - users have no choice of installation directory, always C:Program Files
;InstallDirRegKey HKLM "Software\${BRAND}" "Install_Dir"
; The text to prompt the user to enter a directory
; This was disabled for simplicity - users have no choice of directory installation, always C:Program Files
;DirText "This will install the ${BRAND} server Program on your computer. Choose an installation directory"

; Pages
 
Page license
LicenseText "${LICENSE_TEXT}"
LicenseData "${PROD_DIR}\${BRAND}\${LICENSE_FILE}"

Page instfiles

UninstPage uninstConfirm
UninstPage instfiles

; JRE check, download and install if required 
Section -JRE 
; Check if JRE exists
ClearErrors
Call GetJRE
IfErrors 0 jrefound
MessageBox MB_OKCANCEL|MB_ICONEXCLAMATION "${BRAND} required Java Runtime Environment will now be downloaded and installed. $\n$\n Do you want to continue?" IDOK download
Abort
download:
ClearErrors
Call DownloadJRE
IfErrors 0 jrefound
   Abort 
jrefound:
SectionEnd

Section -CheckNotRunning
; Obtain Application data path and maintain context all users 
SetShellVarContext current
StrCpy $R7 ${RW_DIR} 
SetShellVarContext all
; Try to delete instance file, if not successeful it means instance is running  
ClearErrors
Delete "$R7\instance.lock"
 IfErrors 0 notrunning
MessageBox MB_OK "${BRAND} server is running, installation aborted. $\n$\n Make sure the server is shutdown and try again."
Abort
notrunning:
SectionEnd

Section -CheckVersion

SetDetailsPrint none
; Get the previous version uninstaller if exists
ReadRegStr $R0 HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${BRAND}" "UninstallString"
; Continue if not installed before
StrCmp $R0 "" done
/*
* A developer reported that the following key needs to be checked in Windows 7 64 - needs to confirm:
*HKLM  \SOFTWARE\Wow6432Node\Microsoft\Windows\CurrentVersion\Uninstall\${BRAND}" \
  *"UninstallString"
*/
; Read previous version
ReadRegStr $R1 HKLM SOFTWARE\${BRAND} "Version"
${VersionCompare} $R1 ${VERSION} $R2
; Result in $R2 0 - the same, 1 first version is newer, 2 - second version is newer 
; Check whether $R2!= "2"
StrCmp $R2 "2" upgrade 0
MessageBox MB_ICONSTOP "This ${BRAND} installer version is older or the same as the one you have installed already. $\n$\n If you want to install this version, uninstall the existing version first."
Abort
upgrade:
MessageBox MB_OKCANCEL|MB_ICONEXCLAMATION "${BRAND} is already installed. $\n$\nSelect 'OK' to remove the previous version or 'Cancel' to cancel this upgrade." IDOK uninst
Abort
;Run the uninstaller in silent mode as the user just confirmed
uninst:
ClearErrors
ExecWait '$R0 /S _?=$INSTDIR' ;Do not copy the uninstaller to a temp file - see uninstaller command line options
IfErrors +1 done
; Do nothing allow installation on top of the previous one
; Note that a check for server running was done in the previous section 
MessageBox MB_OK "Error detected during uninstall of the previous version, will attempt to install on top overriding the previous installation"
done:
SectionEnd

Section -SETTINGS
; Set output path to the installation directory.
SetOutPath "$INSTDIR"
SectionEnd

; Copy files and create start menus and desktop shortcuts
Section -INSTALLATION

SetDetailsPrint none
;  All users
SetShellVarContext all

; copy directory and files there
; Copy the main installation directory
; Set output path to the application  directory
SetOutPath $INSTDIR\${APP_DIR}
File /r "${PROD_DIR}\${BRAND}\${APP_DIR}\*.*"
; Set output path to the sh directory
SetOutPath $INSTDIR\sh
; Copy the data directory
File /r "${PROD_DIR}\${BRAND}\sh\data"
; Set output path to the engine directory
SetOutPath $INSTDIR\sh\engine
; Copy the default engines
File /r "${PROD_DIR}\${BRAND}\sh\engine\espeak-0.1.0"
File /r "${PROD_DIR}\${BRAND}\sh\engine\sapi5-0.1.0"
File /r "${PROD_DIR}\${BRAND}\sh\engine\sapi5c-0.1.0"
File /r "${PROD_DIR}\${BRAND}\sh\engine\msp-0.1.0"
File /r "${PROD_DIR}\${BRAND}\sh\engine\marytts-0.1.0"
File /r "${PROD_DIR}\${BRAND}\sh\engine\picotts-0.1.0"
; Set output path to the NVDA addon  directory
SetOutPath $INSTDIR\NVDAAddOn
File /r "${PROD_DIR}\${BRAND}\NVDAAddOn\*.*"
; Set output path to the resources directory
SetOutPath $INSTDIR\resources
File /r "${PROD_DIR}\${BRAND}\resources\*.*"
; Set output path back to the installation directory
SetOutPath $INSTDIR
File "${PROD_DIR}\${BRAND}\${BRAND}_License.txt"
File "${PROD_DIR}\${BRAND}\${BRAND}_Acknowledgments.txt"
File "${PROD_DIR}\${BRAND}\readme.html"

;Create Desktop shortcut
; Desktop shortcut not installed
; Change output path to the desired current directory.
;SetOutPath $INSTDIR\${APP_DIR}
;CreateShortCut "$DESKTOP\${BRAND} Server.lnk" "$INSTDIR\${APP_DIR}\${SPEECH_SERVER_EXE}"
; Restore output path to the installation directory.
;SetOutPath $INSTDIR

; Create start menu shortcuts
CreateDirectory "$SMPROGRAMS\${BRAND}"
CreateShortCut "$SMPROGRAMS\${BRAND}\Uninstall ${BRAND}.lnk" "$INSTDIR\uninstall.exe"
; Change output path to the desired current directory.
SetOutPath $INSTDIR\${APP_DIR}
CreateShortCut "$SMPROGRAMS\${BRAND}\${BRAND} Server.lnk" "$INSTDIR\${APP_DIR}\${SPEECH_SERVER_EXE}"
; Restore output path to the installation directory.
SetOutPath $INSTDIR

; Registry 
; Write the installation path
WriteRegStr HKLM SOFTWARE\${BRAND} "Install_Dir" "$INSTDIR"
; Write the version 
WriteRegStr HKLM SOFTWARE\${BRAND} "Version" "${VERSION}"

; Write the uninstall keys for Windows
WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${BRAND}" "DisplayName" "${BRAND}"
WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${BRAND}" "UninstallString" '"$INSTDIR\uninstall.exe"'
WriteRegDWORD HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${BRAND}" "NoModify" 1
WriteRegDWORD HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${BRAND}" "NoRepair" 1
  
; Tell the compiler to write an uninstaller and to look for a "Uninstall" section 
WriteUninstaller $INSTDIR\Uninstall.exe
; Set output path back to the installation directory
SetOutPath $INSTDIR

SectionEnd 

; The uninstall section
Section "Uninstall"

SetDetailsPrint none
; All users, this helps start menus removal for Windows Vista and Windows 7
; Obtain Application data path and maintain context all users 
SetShellVarContext current
StrCpy $R7 ${RW_DIR} 
SetShellVarContext all
; Try to delete instance file, if not successeful it means instance is running  
ClearErrors
Delete "$R7\instance.lock"
 IfErrors 0 notrunning
MessageBox MB_OK "${BRAND} server is running, uninstall aborted. $\n$\n Make sure the server is shutdown and try again."
Abort
notrunning:

; Remove registry keys
DeleteRegKey HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${BRAND}"
DeleteRegKey HKLM SOFTWARE\${BRAND}

; Delete Desktop shortcut
; Desktop shortcut not installed
;Delete "$DESKTOP\${BRAND} Server.lnk"

; Delete Start Menu
Delete "$SMPROGRAMS\${BRAND}\${BRAND} Server.lnk"
Delete "$SMPROGRAMS\${BRAND}\Uninstall ${BRAND}.lnk"
RmDir "$SMPROGRAMS\${BRAND}"

; Delete installed files 
RmDir /r $INSTDIR\${APP_DIR}
RmDir /r $INSTDIR\sh\data
RmDir /r $INSTDIR\sh\engine\espeak-0.1.0
RmDir /r $INSTDIR\sh\engine\sapi5-0.1.0
RmDir /r $INSTDIR\sh\engine\sapi5c-0.1.0
RmDir /r $INSTDIR\sh\engine\msp-0.1.0
RmDir /r $INSTDIR\sh\engine\marytts-0.1.0
RmDir /r $INSTDIR\sh\engine\picotts-0.1.0
RmDir /r $INSTDIR\NVDAAddOn
RmDir /r $INSTDIR\resources
Delete $INSTDIR\${BRAND}_License.txt
Delete $INSTDIR\${BRAND}_Acknowledgments.txt
Delete $INSTDIR\readme.html
Delete $INSTDIR\Uninstall.exe
RmDir $INSTDIR
; RW directory - delete log and cache
; Leave user's settings files intact
; Delete log file
Delete "$R7\log.txt"
; Delete tcf cache if exists
RmDir /r "$R7\.cache"

SectionEnd 

; MessageBox for debugging example
;MessageBox MB_OK "Hello this is a useful message"
