
# This is appended to _speechhub.py to create the actual installTasks.py use
# in the plugin.  This is because I can't figure out how to load another module
# from installTasks.py

import logHandler

def log(level, text):
    """This function is passed to the speech-hub driver so it can interface to
       our logger."""
    if level == INFO_MESSAGE:
        logHandler.log.info(text)
    elif level == WARNING_MESSAGE:
        logHandler.log.debugWarning(text)
    elif level == IO_MESSAGE:
        logHandler.log.io(text)
    elif level == DEBUG_MESSAGE:
        logHandler.log.debug(text)
    else:
        logHandler.log.error("Invalid logging level:" + text)

log(DEBUG_MESSAGE, "Loading Speech-Hub post install script")

def readFile(fileName):
    """Read a file into an array of strings."""
    try:
        file = open(fileName, "r")
        lines = file.readlines()
        file.close()
    except IOError:
        raise Exception(None, "Unable to read " + fileName)
    log(DEBUG_MESSAGE, "Read " + fileName + " successfully")
    return lines

def writeFile(fileName, lines):
    """Write the lines to the file."""
    try:
        file = open(fileName, "w")
        file.writelines(lines)
        file.close()
    except IOError:
        raise Exception("Can\'t write to file " + fileName)
    log(DEBUG_MESSAGE, "Wrote " + fileName + " successfully")

def genScript(id, niceName, supportsVariants, supportsRateBoost):
    """This function generates the NVDA script to support a synth."""
    fileName = os.path.join(os.path.dirname(__file__), "sh_template")
    lines = readFile(fileName)
    newLines = []
    for line in lines:
        if supportsVariants or line.find('SynthDriver.VariantSetting') == -1:
            if supportsRateBoost or line.find('BooleanSynthSetting("rateBoost"') == -1:
                newLine = line.replace('MODULE_NAME', id)
                newLine = newLine.replace('SHOWN_NAME', niceName)
                newLines.append(newLine)
    outputFileName = "sh_" + id + ".py"
    outputFileName = os.path.join(os.path.dirname(__file__), "synthDrivers", outputFileName)
    log(IO_MESSAGE, "Generaing " + outputFileName)
    writeFile(outputFileName, newLines)

def getSynthSettings():
    log(WARNING_MESSAGE, "Checked that speechhub MODULE_NAME is up")
    sh = SpeechHub(log)
    sh.startServer()
    modules = sh.listOutputModules()
    synths = {}
    for module in modules:
        caption = sh.getModuleProperty(module, "engine.caption")
        if caption.upper() == "NONE":
            log(DEBUG_MESSAGE, "No support for module " + module)
        else:
            audioData = sh.getModuleProperty(module, "engine.supports.audio.data")
            useSonic = sh.getModuleProperty(module, "engine.use.sonic")
            supportsVariants = sh.getModuleProperty(module, "engine.supports.variants")
            synths[module] = [caption, supportsVariants.lower() == "true",
                audioData.lower() == "true" and useSonic.lower() == "false"]
    sh.quit()
    sh.stopServer()
    return synths

def onInstall():
    synths = getSynthSettings()
    for id in synths:
        niceName, supportsVariants, supportsRateBoost = synths[id]
        log(DEBUG_MESSAGE, "Name " + niceName + " Variants " + str(supportsVariants) +
            " rate boost " + str(supportsRateBoost))
        genScript(id, niceName, supportsVariants, supportsRateBoost)
