/*
* Copyright (c) Isaac Porat 2011 - 2013 all rights reserved.
* This file is released under the GNU General Public License, version 2, or (at your option) any later version.
* See the file SpeechHub_License.txt for more details.
*/

package core.utilities;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class RegexpUtilities
{

	public static boolean contains(String string, String patternString)
	{
		Pattern pattern = Pattern.compile(patternString);
		return (contains(string, pattern));
	}

	public static boolean contains(String string, Pattern pattern)
	{
		Matcher matcher = pattern.matcher(string);
		return (matcher.find());
	}

	public static String replaceAll(String string, Pattern pattern, String replacement)
	{
		Matcher matcher = pattern.matcher(string);
		return (matcher.replaceAll(replacement));
	}

	public static RegexpMatchGroupInterface[] findGroupMatches(StringBuilder stringBuilder, String patternString,
			int groupIndex)
	{
		Pattern pattern = Pattern.compile(patternString);
		return (findGroupMatches(stringBuilder, pattern, groupIndex));
	}

	public static RegexpMatchGroupInterface[] findGroupMatches(StringBuilder stringBuilder, Pattern pattern,
			int groupIndex)
	{
		List<RegexpMatchGroupInterface> groups = new ArrayList<RegexpMatchGroupInterface>();
		Matcher matcher = pattern.matcher(stringBuilder);
		while (matcher.find())
		{
			groups.add(new MatchGroup(matcher.group(groupIndex), matcher.start(groupIndex), matcher.end(groupIndex)));
		}
		RegexpMatchGroupInterface[] groupsArray = new MatchGroup[groups.size()];
		return (groups.toArray(groupsArray));
	}

	public static class MatchGroup implements RegexpMatchGroupInterface
	{
		private String fGroup;
		private int fGroupStart;
		private int fGroupEnd;

		public MatchGroup(String group, int groupStart, int groupEnd)
		{
			fGroup = group;
			fGroupStart = groupStart;
			fGroupEnd = groupEnd;
		}

		public String getGroup()
		{
			return (fGroup);
		}

		public int getGroupStart()
		{
			return (fGroupStart);
		}

		public int getGroupEnd()
		{
			return (fGroupEnd);
		}

	}

}
