/*
* Copyright (c) Isaac Porat 2011 - 2013 all rights reserved.
* This file is released under the GNU General Public License, version 2, or (at your option) any later version.
* See the file SpeechHub_License.txt for more details.
*/

package core.utilities;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringReader;
import java.net.URL;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

// The following import required for version 5
import javax.xml.xpath.*;

public class XMLUtils
{

	public static Document parseXmlFile(String filePath) throws ExceptionWrapper
	{
		DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
		DocumentBuilder documentBuilder = null;
		try
		{
			documentBuilder = documentBuilderFactory.newDocumentBuilder();
		} catch (ParserConfigurationException ex)
		{
			throw new ExceptionWrapper(ex);
		}
		Document document = null;
		try
		{
			document = documentBuilder.parse(new File(filePath));
		} catch (SAXException ex)
		{
			throw new ExceptionWrapper(ex);
		} catch (IOException ex)
		{
			throw new ExceptionWrapper(ex);
		}
		return (document);
	}

	public static Document parseXmlFile(URL url) throws ExceptionWrapper
	{
		try
		{
			return (parseXmlFile(url.openStream()));
		} catch (IOException ex)
		{
			throw new ExceptionWrapper(ex);
		}
	}

	public static Document parseXmlFile(InputStream inputStream) throws ExceptionWrapper
	{
		DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
		DocumentBuilder documentBuilder = null;
		try
		{
			documentBuilder = documentBuilderFactory.newDocumentBuilder();
		} catch (ParserConfigurationException ex)
		{
			throw new ExceptionWrapper(ex);
		}
		Document document = null;
		try
		{
			document = documentBuilder.parse(inputStream);
		} catch (SAXException ex)
		{
			throw new ExceptionWrapper(ex);
		} catch (IOException ex)
		{
			throw new ExceptionWrapper(ex);
		}
		return (document);
	}

	public static Document parseXMLString(String xmlString) throws ExceptionWrapper
	{
		Document document = null;
		try
		{
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			DocumentBuilder builder = factory.newDocumentBuilder();
			document = builder.parse(new InputSource(new StringReader(xmlString)));
		} catch (Exception ex)
		{
			throw new ExceptionWrapper(ex);
		}
		return document;
	}

	public static NodeList extractNodes(Node nd, String xpathString) throws ExceptionWrapper
	{
		NodeList nodeList = null;
		/*
		// Use with JRE version 1.4
		try
		{
		    nodeList = org.apache.xpath.XPathAPI.selectNodeList (nd,
		            xpathString, nd);
		}
		catch (javax.xml.transform.TransformerException ex)
		{
		    throw new SExceptionWrapper (ex);
		}
		*/
		// Use with JRE version 1.5 and above
		try
		{
			XPath xpath = XPathFactory.newInstance().newXPath();
			nodeList = (NodeList) xpath.evaluate(xpathString, nd, XPathConstants.NODESET);
		} catch (XPathExpressionException ex)
		{
			throw new ExceptionWrapper(ex);
		}
		return nodeList;
	}

	// as above, but first node only
	public static Node extractNode(Node node, String xpath) throws ExceptionWrapper
	{
		NodeList nodeList;
		Node firstNode = null;
		nodeList = extractNodes(node, xpath);
		if (nodeList.getLength() > 0)
			firstNode = nodeList.item(0);
		return (firstNode);
	}

	public static String getNodeText(Node node)
	{
		if (node == null)
			return (null);
		Node textNode = node.getFirstChild();
		if (textNode != null)
			return textNode.getNodeValue();
		return ("");
	}

	public static String getAttributeValue(Node node, String attribute)
	{
		NamedNodeMap attributes = node.getAttributes();
		if (attributes == null)
			return null;
		Node namedNode = attributes.getNamedItem(attribute);
		if (namedNode == null)
			return null;
		return namedNode.getNodeValue();
	}

	public static String getNodeName(Node node)
	{
		return (node.getNodeName());
	}

}
