/*
* Copyright (c) Isaac Porat 2011 - 2013 all rights reserved.
* This file is released under the GNU General Public License, version 2, or (at your option) any later version.
* See the file SpeechHub_License.txt for more details.
*/

package hrpc.handler;

import hrpc.common.HRTransportInterface;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.PrintStream;

import log.common.BasicLogger;
import log.common.CascadingLoggerInterface;

public abstract class HRMessageHandlerBase implements Runnable
{
	protected CascadingLoggerInterface log;
	protected boolean fDebugMode = false;
	private Thread fCommThread;
	protected HRTransportInterface fTransport;
	protected PrintStream fOut;
	private BufferedReader fIn;
	private String fInputLine;
	protected String lineSeparator = "\r\n";
	protected boolean fUnexpectedDroppedConnection = true;
	protected String fRemoteConnectorIndicator = "remote connector";
	protected boolean fClientClose = false;
	protected boolean fExitOnProcessClose = false;

	public void setTransport(HRTransportInterface transport)
	{
		fTransport = transport;
	}

	public void open() throws IOException
	{
		fClientClose = false;
		fTransport.open();
		fIn = fTransport.getStdin();
		fOut = fTransport.getStdout();
		fCommThread = new Thread(this);
                fCommThread.setName(HRMessageHandlerBase.class.getName());
		fCommThread.start();
	}

	protected void onBeforeTransportClose()
	{
		// Override if required
	}

	public void closeTransport() throws IOException
	{
		onBeforeTransportClose();
		fTransport.close();
		fCommThread = null;
		fIn = null;
		fOut = null;
	}

	public synchronized void dispatchMessage(String message)
	{
		fOut.print(message + lineSeparator);
	}

	public abstract boolean processMessage(String message);

	@Override
	public void run()
	{
		try
		{
			while ((fInputLine = fIn.readLine()) != null)
			{
				if (processMessage(fInputLine) == false)
				{
					fUnexpectedDroppedConnection = false;
					break;
				}
			}
			if (fUnexpectedDroppedConnection)
			{
				if (fClientClose == false)
					log.error("Connection unexpectedly dropped by " + fRemoteConnectorIndicator);
			} else
			{
				if (fDebugMode)
					BasicLogger.notifyPrintln("Connection terminated by " + fRemoteConnectorIndicator);
			}
		} catch (Exception ex)
		{
			if (fUnexpectedDroppedConnection) // Prevent Thread error on connection close when thread is blocked 
				log.error("Error in run method:\n" + core.utilities.StackTrace.stackToString(ex));
		} finally
		{
			try
			{
				closeTransport();
			} catch (Exception exx)
			{
			}
			if (fExitOnProcessClose)
				System.exit(0);
		}

	}
}
