/*
* Copyright (c) Isaac Porat 2011 - 2013 all rights reserved.
* This file is released under the GNU General Public License, version 2, or (at your option) any later version.
* See the file SpeechHub_License.txt for more details.
*/

package hrpc.handler;

import hrpc.common.HRPCConstants;
import hrpc.common.HRHandlerInterface;
import hrpc.common.HRMessageHandlerInterface;
import hrpc.common.HRMessageHandlerJSONInterface;
import hrpc.common.HRReceiverHandlerJSONInterface;
import hrpc.common.HRSenderHandlerJSONInterface;
import hrpc.common.HRTransportInterface;

import java.io.IOException;

import log.common.CascadingLoggerInterface;


public class HRMessageHandlerClientMultiProtocolBidirectional extends HRMessageHandlerBase implements
		HRMessageHandlerInterface, HRMessageHandlerJSONInterface
{
	private final static String REQUEST_REGEXP = "^" + HRPCConstants.HPR_REQUEST;
	private final static String JSON_REGEXP = "^" + HRPCConstants.HPR_JSON;
	private HRSenderHandlerJSONInterface fSenderHandlerJSON;
	private HRReceiverHandlerJSONInterface fReceiverHandlerJSON;
	private HRHandlerInterface fHandlerCustom;

	public HRMessageHandlerClientMultiProtocolBidirectional(HRTransportInterface client,
			HRSenderHandlerJSONInterface senderHandlerJSON, HRReceiverHandlerJSONInterface receiverHandlerJSON,
			HRHandlerInterface handlerCustom, CascadingLoggerInterface logger) throws IOException
	{
		log = logger.clone(HRMessageHandlerClientMultiProtocolBidirectional.class);
		fTransport = client;
		fSenderHandlerJSON = senderHandlerJSON;
		if (fSenderHandlerJSON != null)
			fSenderHandlerJSON.setMessageHandler(this);
		fReceiverHandlerJSON = receiverHandlerJSON;
		if (fReceiverHandlerJSON != null)
			fReceiverHandlerJSON.setMessageHandler(this);
		fHandlerCustom = handlerCustom;
		if (fHandlerCustom != null)
			fHandlerCustom.setMessageHandler(this);
	}

	@Override
	public void close() throws IOException
	{
		fClientClose = true; // Suppress unexpected close message from server 
		fSenderHandlerJSON.close();
		fUnexpectedDroppedConnection = false; // Prevent unnecessary log exception error 
		closeTransport();
	}

	@Override
	public void dispatchJSONRequestMessage(String requestMessage)
	{
		// No Request marker in client 
		dispatchMessage(HRPCConstants.HPR_JSON + requestMessage);
	}

	@Override
	public void dispatchJSONResponseMessage(String responseMessage)
	{
		// Insert response marker
		dispatchMessage(HRPCConstants.HPR_RESPONSE + HRPCConstants.HPR_JSON + responseMessage);
	}

	@Override
	public boolean processMessage(String message)
	{
		// Custom message does not support bi directional comm
		try
		{
			if (message.startsWith(HRPCConstants.HPR_REQUEST))
			{
				message = message.replaceAll(REQUEST_REGEXP, "");
				if (message.startsWith(HRPCConstants.HPR_JSON))
					message = message.replaceAll(JSON_REGEXP, "");
				return (fReceiverHandlerJSON.processRequestMessage(message));
			} else
			{
				// Route to JSON or custom
				if (message.startsWith(HRPCConstants.HPR_JSON))
				{
					message = message.replaceAll(JSON_REGEXP, "");
					return (fSenderHandlerJSON.processResponseMessage(message));
				} else
				{
					return (fHandlerCustom.processMessage(message));
				}
			}
		} catch (Exception ex)
		{
			log.error("Error in process method:\n" + core.utilities.StackTrace.stackToString(ex));
			return (true); // Do not close connection
		}
	}

}
