/*
* Copyright (c) Isaac Porat 2011 - 2013 all rights reserved.
* This file is released under the GNU General Public License, version 2, or (at your option) any later version.
* See the file SpeechHub_License.txt for more details.
*/

package hrpc.handler;

import hrpc.common.HRHandlerInterface;
import hrpc.common.HRMessageHandlerInterface;

import java.io.IOException;

import log.common.CascadingLoggerInterface;

public class HRMessageHandlerServer extends HRMessageHandlerBase implements HRMessageHandlerInterface
{
	private HRHandlerInterface fHandler;

	public HRMessageHandlerServer(HRHandlerInterface handler, boolean exitOnProcessClose, boolean debugMode,
			CascadingLoggerInterface logger) throws IOException
	{
		log = logger.clone(HRMessageHandlerServer.class);
		fHandler = handler;
		fExitOnProcessClose = exitOnProcessClose;
		fDebugMode = debugMode;
		fHandler.setMessageHandler(this);
	}

	@Override
	protected void onBeforeTransportClose()
	{
		try
		{
			fHandler.close();
		} catch (Exception ex)
		{
			log.error("Error in method onBeforeTransportClose:\n" + ex);
		}
	}

	@Override
	public void close() throws IOException
	{
		closeTransport();
	}

	@Override
	public boolean processMessage(String message)
	{
		try
		{
			return (fHandler.processMessage(message));
		} catch (Exception ex)
		{
			log.error("Error in processMessage method:\n" + core.utilities.StackTrace.stackToString(ex));
			return (true); // Do not drop the connection
		}
	}

}
