/*
* Copyright (c) Isaac Porat 2011 - 2013 all rights reserved.
* This file is released under the GNU General Public License, version 2, or (at your option) any later version.
* See the file SpeechHub_License.txt for more details.
*/

package hrpc.handler;

import hrpc.common.HRMessageHandlerInterface;
import hrpc.common.HRMessageHandlerJSONInterface;
import hrpc.common.HRReceiverHandlerJSONInterface;
import hrpc.common.HRSenderHandlerJSONInterface;
import hrpc.common.HRTransportInterface;

import java.io.IOException;

import log.common.CascadingLoggerInterface;

public class HRMessageHandlerServerJSONProtocolUnidirectional extends HRMessageHandlerBase implements
		HRMessageHandlerJSONInterface, HRMessageHandlerInterface
{
	private HRSenderHandlerJSONInterface fSenderHandlerJSON;
	private HRReceiverHandlerJSONInterface fReceiverHandlerJSON;

	public HRMessageHandlerServerJSONProtocolUnidirectional(HRTransportInterface server,
			HRSenderHandlerJSONInterface senderHandlerJSON, HRReceiverHandlerJSONInterface receiverHandlerJSON,
			CascadingLoggerInterface logger) throws IOException
	{
		log = logger.clone(HRMessageHandlerServerJSONProtocolUnidirectional.class);
		fTransport = server;
		fSenderHandlerJSON = senderHandlerJSON;
		if (fSenderHandlerJSON != null)
			fSenderHandlerJSON.setMessageHandler(this);
		fReceiverHandlerJSON = receiverHandlerJSON;
		if (fReceiverHandlerJSON != null)
			fReceiverHandlerJSON.setMessageHandler(this);
	}

	@Override
	public void close() throws IOException
	{
		closeTransport();
	}

	@Override
	public void dispatchJSONRequestMessage(String requestMessage)
	{
		log.error("Error in dispatchJSONRequestMessage method : Not applicable to server");
	}

	@Override
	public void dispatchJSONResponseMessage(String responseMessage)
	{
		dispatchMessage(responseMessage);
	}

	@Override
	public boolean processMessage(String message)
	{
		try
		{
			return (fReceiverHandlerJSON.processRequestMessage(message));
		} catch (Exception ex)
		{
			log.error("Error in process method:\n" + core.utilities.StackTrace.stackToString(ex));
			return (true); // Do not close connection
		}
	}

}
