/*
* Copyright (c) Isaac Porat 2011 - 2013 all rights reserved.
* This file is released under the GNU General Public License, version 2, or (at your option) any later version.
* See the file SpeechHub_License.txt for more details.
*/

package hrpc.handler;

import log.common.CascadingLoggerInterface;
import hrpc.common.HRPCConstants;
import hrpc.common.HRMessageHandlerJSONInterface;
import hrpc.common.HRReceiverHandlerJSONInterface;
import hrpc.common.HRReceiverRouterHandlerJSONInterface;

public class HRReceiverHandlerJSON implements HRReceiverHandlerJSONInterface
{
	@SuppressWarnings("unused")
	private CascadingLoggerInterface log;
	private HRMessageHandlerJSONInterface fMessageHandler;
	HRReceiverRouterHandlerJSONInterface fReceiverHandlerRouter;

	public HRReceiverHandlerJSON(HRReceiverRouterHandlerJSONInterface receiverHandlerRouter, CascadingLoggerInterface logger)
	{
		log = logger.clone(HRReceiverHandlerJSON.class);
		fReceiverHandlerRouter = receiverHandlerRouter;
		fReceiverHandlerRouter.setReceiverHandler(this);
	}

	@Override
	public void setMessageHandler(HRMessageHandlerJSONInterface messageHandler)
	{
		fMessageHandler = messageHandler;
	}

	@Override
	public boolean processRequestMessage(String requestMessage)
	{
		// Extract components 
		String[] parts = requestMessage.split(HRPCConstants.HPR_MESSAGE_SEP, 2);
		String handlerName = parts[0];
		String jsonRequestString = parts[1];
		return(fReceiverHandlerRouter.routeHandlers(handlerName, jsonRequestString));
	}

	public void dispatchResponseMessage(String responseMessage)
	{
		fMessageHandler.dispatchJSONResponseMessage(responseMessage);
	}

}
