/*
* Copyright (c) Isaac Porat 2011 - 2013 all rights reserved.
* This file is released under the GNU General Public License, version 2, or (at your option) any later version.
* See the file SpeechHub_License.txt for more details.
*/

package hrpc.handler;

import hrpc.common.HRPCConstants;
import hrpc.common.HRReceiverHandlerJSONInterface;

import java.util.Arrays;
import java.util.Map;

import log.common.CascadingLoggerInterface;

import com.thetransactioncompany.jsonrpc2.JSONRPC2Error;
import com.thetransactioncompany.jsonrpc2.JSONRPC2Request;
import com.thetransactioncompany.jsonrpc2.JSONRPC2Response;
import com.thetransactioncompany.jsonrpc2.util.NamedParamsRetriever;

public class HRReceiverMethodHandlerJSON
{
	@SuppressWarnings("unused")
	private CascadingLoggerInterface log;
	private HRReceiverHandlerJSONInterface fReceiverHandler;

	public HRReceiverMethodHandlerJSON(HRReceiverHandlerJSONInterface receiverHandler, CascadingLoggerInterface logger)
	{
		fReceiverHandler = receiverHandler;
		log = logger.clone(HRReceiverMethodHandlerJSON.class);
	}

	public NamedParamsRetriever getNamedParamsRetriever(JSONRPC2Request request)
	{
		@SuppressWarnings("unchecked")
		Map<String, Object> params = (Map<String, Object>) request.getParams();
		NamedParamsRetriever npr = new NamedParamsRetriever(params);
		return (npr);
	}

	public void dispatchResponseResult(Object result, String requestIdentifier)
	{
		JSONRPC2Response response = new JSONRPC2Response(result, requestIdentifier);
		String jsonResponseString = response.toString();
		fReceiverHandler.dispatchResponseMessage(jsonResponseString);
	}

	public void dispatchResponseError(int code, String message, String data, String requestIdentifier)
	{
		JSONRPC2Error error = new JSONRPC2Error(code, message, data);
		dispatchResponseError(error, requestIdentifier);
	}

	public void dispatchResponseError(JSONRPC2Error error, String requestIdentifier)
	{
		JSONRPC2Response response = new JSONRPC2Response(error, requestIdentifier);
		String jsonResponseString = response.toString();
		fReceiverHandler.dispatchResponseMessage(jsonResponseString);
	}

	public void dispatchResponseVoid(String requestIdentifier)
	{
		// This method not tested
		// Not sure if the following line is the best way to handle methods that return void
		dispatchResponseResult(true, requestIdentifier);
	}

	public void dispatchResponseBoolean(boolean status, String requestIdentifier)
	{
		if (status)
			dispatchResponseResult(true, requestIdentifier);
		else
			dispatchResponseError(HRPCConstants.METHOD_RETURNED_FALSE_CODE,
					HRPCConstants.METHOD_RETURNED_FALSE_MESSAGE, "none", requestIdentifier);
	}

	public void dispatchResponseBoolean(boolean status, String data, String requestIdentifier)
	{
		if (status)
			dispatchResponseResult(true, requestIdentifier);
		else
			dispatchResponseError(HRPCConstants.METHOD_RETURNED_FALSE_CODE,
					HRPCConstants.METHOD_RETURNED_FALSE_MESSAGE, data, requestIdentifier);
	}

	public void dispatchResponseObject(Object result, String requestIdentifier)
	{
		if (result != null)
			dispatchResponseResult(result, requestIdentifier);
		else
			dispatchResponseError(HRPCConstants.METHOD_RETURNED_NULL_CODE, HRPCConstants.METHOD_RETURNED_NULL_MESSAGE,
					"none", requestIdentifier);
	}

	public void dispatchResponseString(String result, String requestIdentifier)
	{
		dispatchResponseObject(result, requestIdentifier);
	}

	public void dispatchResponseStringArray(String[] result, String requestIdentifier)
	{
		dispatchResponseObject(Arrays.asList(result), requestIdentifier);
	}

}
