/*
* Copyright (c) Isaac Porat 2011 - 2013 all rights reserved.
* This file is released under the GNU General Public License, version 2, or (at your option) any later version.
* See the file SpeechHub_License.txt for more details.
*/

package hrpc.handler;

import hrpc.common.HRPCConstants;
import hrpc.common.HRMessageHandlerJSONInterface;
import hrpc.common.HRSenderHandlerJSONInterface;

import java.util.HashMap;
import java.util.Map;

import log.common.CascadingLoggerInterface;


import com.thetransactioncompany.jsonrpc2.JSONRPC2ParseException;
import com.thetransactioncompany.jsonrpc2.JSONRPC2Request;
import com.thetransactioncompany.jsonrpc2.JSONRPC2Response;

public class HRSenderHandlerJSON implements HRSenderHandlerJSONInterface
{
	private CascadingLoggerInterface log;
	private HRMessageHandlerJSONInterface fMessageHandler;
	private int fRequestIdentifierInt = 1;
	private Map<String, JSONRPC2Response> fResponseHolder;

	public HRSenderHandlerJSON(CascadingLoggerInterface logger)
	{
		log = logger.clone(HRSenderHandlerJSON.class);
		fResponseHolder = new HashMap<String, JSONRPC2Response>();
	}

	public Map<String, JSONRPC2Response> getResponseHolder()
	{
		return (fResponseHolder);
	}

	@Override
	public void setMessageHandler(HRMessageHandlerJSONInterface messageHandler)
	{
		fMessageHandler = messageHandler;
	}

	@Override
	public void close()
	{
		dispatchRequestMessage("quit", "");
	}

	public synchronized String getNewRequestIdentifier()
	{
		return (Integer.toString(fRequestIdentifierInt++));
	}

	@Override
	public boolean processResponseMessage(String responseMessage)
	{
		if (responseMessage.startsWith("{")) // No HPR header supplied
		{
			// Process immediatly
			postResponse(responseMessage);
		}
		return (true);
	}

	private void postResponse(String jsonResponseString)
	{
		JSONRPC2Response response = null;
		try
		{
			response = JSONRPC2Response.parse(jsonResponseString);
			String requestIdentifier = (String) response.getID();
			synchronized (fResponseHolder)
			{
				fResponseHolder.put(requestIdentifier, response);
				fResponseHolder.notifyAll();
			}
		} catch (JSONRPC2ParseException ex)
		{
			log.error("Error in postResponse method:\n" + ex);
			return;
		}
	}

	public void dispatchRequestMessage(String handler, String jsonString)
	{
		String requestMessage = handler + HRPCConstants.HPR_MESSAGE_SEP + jsonString;
		dispatchRequestMessage(requestMessage);
	}

	public void dispatchRequestMessage(String requestMessage)
	{
		fMessageHandler.dispatchJSONRequestMessage(requestMessage);
	}

	public void dispatchRequest(String handler, String method, String requestIdentifier)
	{
		JSONRPC2Request request = new JSONRPC2Request(method, requestIdentifier);
		String jsonRequestString = request.toString();
		dispatchRequestMessage(handler, jsonRequestString);
	}

	public void dispatchRequest(String handler, String method, Map<String, Object> params, String requestIdentifier)
	{
		JSONRPC2Request request = new JSONRPC2Request(method, params, requestIdentifier);
		String jsonRequestString = request.toString();
		dispatchRequestMessage(handler, jsonRequestString);
	}

}
