/*
* Copyright (c) Isaac Porat 2011 - 2013 all rights reserved.
* This file is released under the GNU General Public License, version 2, or (at your option) any later version.
* See the file SpeechHub_License.txt for more details.
*/

package hrpc.handler;

import hrpc.common.HRSenderHandlerJSONInterface;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import log.common.CascadingLoggerInterface;


import com.thetransactioncompany.jsonrpc2.JSONRPC2Error;
import com.thetransactioncompany.jsonrpc2.JSONRPC2Response;

public class HRSenderMethodHandlerJSON
{
	private CascadingLoggerInterface log;
	private HRSenderHandlerJSONInterface fSenderHandler;
	private String fHandlerName;
	private String fMethodName;
	private Map<String, JSONRPC2Response> fResponseHolder;

	public HRSenderMethodHandlerJSON(HRSenderHandlerJSONInterface senderHandler, String handlerName, String methodName,
			CascadingLoggerInterface logger)
	{
		fSenderHandler = senderHandler;
		fHandlerName = handlerName;
		fMethodName = methodName;
		log = logger.clone(HRSenderMethodHandlerJSON.class);
		fResponseHolder = fSenderHandler.getResponseHolder();
	}

	private void dispatchRequest(String requestIdentifier)
	{
		fSenderHandler.dispatchRequest(fHandlerName, fMethodName, requestIdentifier);
	}

	private void dispatchRequest(String requestIdentifier, Map<String, Object> params)
	{
		fSenderHandler.dispatchRequest(fHandlerName, fMethodName, params, requestIdentifier);
	}

	public JSONRPC2Response dispatchRequestWaitResponse(String requestIdentifier)
	{
		dispatchRequest(requestIdentifier);
		return (waitResponse(requestIdentifier));
	}

	public JSONRPC2Response dispatchRequestWaitResponse(String requestIdentifier, String param1Key, Object param1Value)
	{
		Map<String, Object> params = new HashMap<String, Object>();
		params.put(param1Key, param1Value);
		dispatchRequest(requestIdentifier, params);
		return (waitResponse(requestIdentifier));
	}

	public JSONRPC2Response dispatchRequestWaitResponse(String requestIdentifier, String param1Key, Object param1Value,
			String param2Key, Object param2Value)
	{
		Map<String, Object> params = new HashMap<String, Object>();
		params.put(param1Key, param1Value);
		params.put(param2Key, param2Value);
		dispatchRequest(requestIdentifier, params);
		return (waitResponse(requestIdentifier));
	}

	public JSONRPC2Response dispatchRequestWaitResponse(String requestIdentifier, String param1Key, Object param1Value,
			String param2Key, Object param2Value, String param3Key, Object param3Value)
	{
		Map<String, Object> params = new HashMap<String, Object>();
		params.put(param1Key, param1Value);
		params.put(param2Key, param2Value);
		params.put(param3Key, param3Value);
		dispatchRequest(requestIdentifier, params);
		return (waitResponse(requestIdentifier));
	}

	private JSONRPC2Response waitResponse(String requestIdentifier)
	{
		JSONRPC2Response response = null;
		synchronized (fResponseHolder)
		{
			while (fResponseHolder.containsKey(requestIdentifier) == false)
			{
				try
				{
					fResponseHolder.wait();
				} catch (InterruptedException ex)
				{
					log.error("Error in " + fMethodName + " method:\n" + ex);
				}
			}
			response = fResponseHolder.get(requestIdentifier);
			fResponseHolder.remove(requestIdentifier);
		}
		return (response);
	}

	public boolean getResponseBoolean(JSONRPC2Response response, String callingMethod)
	{
		if (response.indicatesSuccess())
		{
			return (true);
		} else
		{
			JSONRPC2Error error = response.getError();
			log.error("Error in getResponseBoolean method:\n" + "Calling method: " + callingMethod + "\n"
					+ "Error code: " + error.getCode() + "\n" + "Error message: " + error.getMessage() + "\n"
					+ "Error data: " + error.getData());
			return (false);
		}
	}

	public synchronized Object getResponseObject(JSONRPC2Response response, String callingMethod)
	{
		if (response.indicatesSuccess())
		{
			return (response.getResult());
		} else
		{
			JSONRPC2Error error = response.getError();
			log.error("Error in getResponseBoolean method:\n" + "Calling method: " + callingMethod + "\n"
					+ "Error code: " + error.getCode() + "\n" + "Error message: " + error.getMessage() + "\n"
					+ "Error data: " + error.getData());
			return (null);
		}
	}

	public String getResponseString(JSONRPC2Response response, String callingMethod)
	{
		Object responseObject = getResponseObject(response, callingMethod);
		if (responseObject == null)
			return (null);
		return ((String) responseObject);
	}

	public String[] getResponseStringArray(JSONRPC2Response response, String callingMethod)
	{
		Object responseObject = getResponseObject(response, callingMethod);
		if (responseObject == null)
			return (null);
		@SuppressWarnings("unchecked")
		List<String> resultList = (List<String>) responseObject;
		return (resultList.toArray(new String[0]));
	}

}
