/*
* Copyright (c) Isaac Porat 2011 - 2013 all rights reserved.
* This file is released under the GNU General Public License, version 2, or (at your option) any later version.
* See the file SpeechHub_License.txt for more details.
*/

package hrpc.sh.client;

import log.common.CascadingLoggerInterface;
import hrpc.common.HRMessageHandlerJSONInterface;
import hrpc.common.HRSenderHandlerJSONInterface;
import hrpc.handler.HRSenderMethodHandlerJSON;

import sh.connections.SHCustomeHandlersInterface;
import sh.connections.SHEngineConnectorInterface;
import sh.connections.SHAudioDataHandlerInterface;
import sh.connections.SHEventsListenerInterface;

import com.thetransactioncompany.jsonrpc2.JSONRPC2Response;

import core.utilities.StackTrace;

public abstract class SHEngineConnectorClientBase implements SHEngineConnectorInterface
{
	protected CascadingLoggerInterface log;
	protected HRMessageHandlerJSONInterface fMessageHandler;
	protected HRSenderHandlerJSONInterface fSenderHandler;

	@Override
	public boolean initialize()
	{
		String handlerName = "primary";
		String methodName = "initialize";
		try
		{
			fMessageHandler.open();
			String requestIdentifier = fSenderHandler.getNewRequestIdentifier();
			HRSenderMethodHandlerJSON smh = new HRSenderMethodHandlerJSON(fSenderHandler, handlerName, methodName, log);
			JSONRPC2Response response = smh.dispatchRequestWaitResponse(requestIdentifier);
			return (smh.getResponseBoolean(response, methodName));
		} catch (Exception ex)
		{
			log.error("Error in " + methodName + " method:\n" + StackTrace.stackToString(ex));
			return (false);
		}
	}

	@Override
	public boolean close()
	{
		String handlerName = "primary";
		String methodName = "close";
		try
		{
			String requestIdentifier = fSenderHandler.getNewRequestIdentifier();
			HRSenderMethodHandlerJSON smh = new HRSenderMethodHandlerJSON(fSenderHandler, handlerName, methodName, log);
			JSONRPC2Response response = smh.dispatchRequestWaitResponse(requestIdentifier);
			boolean status = smh.getResponseBoolean(response, methodName);
			fMessageHandler.close();
			return (status);
		} catch (Exception ex)
		{
			log.error("Error in " + methodName + " method:\n" + StackTrace.stackToString(ex));
			return (false);
		}
	}

	@Override
	public String[] getEngineIdentifiers()
	{
		String handlerName = "primary";
		String methodName = "getEngineIdentifiers";
		try
		{
			String requestIdentifier = fSenderHandler.getNewRequestIdentifier();
			HRSenderMethodHandlerJSON smh = new HRSenderMethodHandlerJSON(fSenderHandler, handlerName, methodName, log);
			JSONRPC2Response response = smh.dispatchRequestWaitResponse(requestIdentifier);
			return (smh.getResponseStringArray(response, methodName));
		} catch (Exception ex)
		{
			log.error("Error in " + methodName + " method:\n" + ex);
			return (null);
		}
	}

	@Override
	public boolean selectEngine(String engineIdentifier)
	{
		String handlerName = "primary";
		String methodName = "selectEngine";
		try
		{
			String requestIdentifier = fSenderHandler.getNewRequestIdentifier();
			HRSenderMethodHandlerJSON smh = new HRSenderMethodHandlerJSON(fSenderHandler, handlerName, methodName, log);
			JSONRPC2Response response = smh.dispatchRequestWaitResponse(requestIdentifier, "engineIdentifier",
					engineIdentifier);
			return (smh.getResponseBoolean(response, methodName));
		} catch (Exception ex)
		{
			log.error("Error in " + methodName + " method:\n" + ex);
			return (false);
		}
	}

	@Override
	public boolean selectLanguage(String languageCode)
	{
		String handlerName = "primary";
		String methodName = "selectLanguage";
		try
		{
			String requestIdentifier = fSenderHandler.getNewRequestIdentifier();
			HRSenderMethodHandlerJSON smh = new HRSenderMethodHandlerJSON(fSenderHandler, handlerName, methodName, log);
			JSONRPC2Response response = smh
					.dispatchRequestWaitResponse(requestIdentifier, "languageCode", languageCode);
			return (smh.getResponseBoolean(response, methodName));
		} catch (Exception ex)
		{
			log.error("Error in " + methodName + " method:\n" + ex);
			return (false);
		}
	}

	@Override
	public String[] getVoiceIdentifiers()
	{
		String handlerName = "primary";
		String methodName = "getVoiceIdentifiers";
		try
		{
			String requestIdentifier = fSenderHandler.getNewRequestIdentifier();
			HRSenderMethodHandlerJSON smh = new HRSenderMethodHandlerJSON(fSenderHandler, handlerName, methodName, log);
			JSONRPC2Response response = smh.dispatchRequestWaitResponse(requestIdentifier);
			return (smh.getResponseStringArray(response, methodName));
		} catch (Exception ex)
		{
			log.error("Error in " + methodName + " method:\n" + ex);
			return (null);
		}
	}

	@Override
	public boolean selectVoice(String voiceIdentifier)
	{
		String handlerName = "primary";
		String methodName = "selectVoice";
		try
		{
			String requestIdentifier = fSenderHandler.getNewRequestIdentifier();
			HRSenderMethodHandlerJSON smh = new HRSenderMethodHandlerJSON(fSenderHandler, handlerName, methodName, log);
			JSONRPC2Response response = smh.dispatchRequestWaitResponse(requestIdentifier, "voiceIdentifier",
					voiceIdentifier);
			return (smh.getResponseBoolean(response, methodName));
		} catch (Exception ex)
		{
			log.error("Error in " + methodName + " method:\n" + ex);
			return (false);
		}
	}

	@Override
	public String[] getVariantIdentifiers()
	{
		String handlerName = "primary";
		String methodName = "getVariantIdentifiers";
		try
		{
			String requestIdentifier = fSenderHandler.getNewRequestIdentifier();
			HRSenderMethodHandlerJSON smh = new HRSenderMethodHandlerJSON(fSenderHandler, handlerName, methodName, log);
			JSONRPC2Response response = smh.dispatchRequestWaitResponse(requestIdentifier);
			return (smh.getResponseStringArray(response, methodName));
		} catch (Exception ex)
		{
			log.error("Error in " + methodName + " method:\n" + ex);
			return (null);
		}
	}

	@Override
	public boolean selectVariant(String variantIdentifier)
	{
		String handlerName = "primary";
		String methodName = "selectVariant";
		try
		{
			String requestIdentifier = fSenderHandler.getNewRequestIdentifier();
			HRSenderMethodHandlerJSON smh = new HRSenderMethodHandlerJSON(fSenderHandler, handlerName, methodName, log);
			JSONRPC2Response response = smh.dispatchRequestWaitResponse(requestIdentifier, "variantIdentifier",
					variantIdentifier);
			return (smh.getResponseBoolean(response, methodName));
		} catch (Exception ex)
		{
			log.error("Error in " + methodName + " method:\n" + ex);
			return (false);
		}
	}

	@Override
	public boolean speak(int messageIdentifier, String message, boolean speakSync)
	{
		String handlerName = "primary";
		String methodName = "speak";
		try
		{
			String requestIdentifier = fSenderHandler.getNewRequestIdentifier();
			HRSenderMethodHandlerJSON smh = new HRSenderMethodHandlerJSON(fSenderHandler, handlerName, methodName, log);
			JSONRPC2Response response = smh.dispatchRequestWaitResponse(requestIdentifier, "messageIdentifier",
					messageIdentifier, "message", message, "speakSync", speakSync);

			return (smh.getResponseBoolean(response, methodName));
		} catch (Exception ex)
		{
			log.error("Error in " + methodName + " method:\n" + ex);
			return (false);
		}
	}

	@Override
	public boolean pause()
	{
		String handlerName = "secondary";
		String methodName = "pause";
		try
		{
			String requestIdentifier = fSenderHandler.getNewRequestIdentifier();
			HRSenderMethodHandlerJSON smh = new HRSenderMethodHandlerJSON(fSenderHandler, handlerName, methodName, log);
			JSONRPC2Response response = smh.dispatchRequestWaitResponse(requestIdentifier);
			return (smh.getResponseBoolean(response, methodName));
		} catch (Exception ex)
		{
			log.error("Error in " + methodName + " method:\n" + ex);
			return (false);
		}
	}

	@Override
	public boolean resume()
	{
		String handlerName = "secondary";
		String methodName = "resume";
		try
		{
			String requestIdentifier = fSenderHandler.getNewRequestIdentifier();
			HRSenderMethodHandlerJSON smh = new HRSenderMethodHandlerJSON(fSenderHandler, handlerName, methodName, log);
			JSONRPC2Response response = smh.dispatchRequestWaitResponse(requestIdentifier);
			return (smh.getResponseBoolean(response, methodName));
		} catch (Exception ex)
		{
			log.error("Error in " + methodName + " method:\n" + ex);
			return (false);
		}
	}

	@Override
	public boolean cancel()
	{
		String handlerName = "secondary";
		String methodName = "cancel";
		try
		{
			String requestIdentifier = fSenderHandler.getNewRequestIdentifier();
			HRSenderMethodHandlerJSON smh = new HRSenderMethodHandlerJSON(fSenderHandler, handlerName, methodName, log);
			JSONRPC2Response response = smh.dispatchRequestWaitResponse(requestIdentifier);
			return (smh.getResponseBoolean(response, methodName));
		} catch (Exception ex)
		{
			log.error("Error in " + methodName + " method:\n" + StackTrace.stackToString(ex));
			return (false);
		}
	}

	@Override
	public boolean setVolume(float volume)
	{
		String handlerName = "primary";
		String methodName = "setVolume";
		try
		{
			String requestIdentifier = fSenderHandler.getNewRequestIdentifier();
			HRSenderMethodHandlerJSON smh = new HRSenderMethodHandlerJSON(fSenderHandler, handlerName, methodName, log);
			JSONRPC2Response response = smh.dispatchRequestWaitResponse(requestIdentifier, "volume", volume);
			return (smh.getResponseBoolean(response, methodName));
		} catch (Exception ex)
		{
			log.error("Error in " + methodName + " method:\n" + ex);
			return (false);
		}
	}

	@Override
	public boolean setRate(float rate)
	{
		String handlerName = "primary";
		String methodName = "setRate";
		try
		{
			String requestIdentifier = fSenderHandler.getNewRequestIdentifier();
			HRSenderMethodHandlerJSON smh = new HRSenderMethodHandlerJSON(fSenderHandler, handlerName, methodName, log);
			JSONRPC2Response response = smh.dispatchRequestWaitResponse(requestIdentifier, "rate", rate);
			return (smh.getResponseBoolean(response, methodName));
		} catch (Exception ex)
		{
			log.error("Error in " + methodName + " method:\n" + ex);
			return (false);
		}
	}

	@Override
	public boolean setPitch(float pitch)
	{
		String handlerName = "primary";
		String methodName = "setPitch";
		try
		{
			String requestIdentifier = fSenderHandler.getNewRequestIdentifier();
			HRSenderMethodHandlerJSON smh = new HRSenderMethodHandlerJSON(fSenderHandler, handlerName, methodName, log);
			JSONRPC2Response response = smh.dispatchRequestWaitResponse(requestIdentifier, "pitch", pitch);
			return (smh.getResponseBoolean(response, methodName));
		} catch (Exception ex)
		{
			log.error("Error in " + methodName + " method:\n" + ex);
			return (false);
		}
	}

	@Override
	public String getProperty(String key)
	{
		if (key == null)
			return (null);
		String handlerName = "primary";
		String methodName = "getProperty";
		try
		{
			String requestIdentifier = fSenderHandler.getNewRequestIdentifier();
			HRSenderMethodHandlerJSON smh = new HRSenderMethodHandlerJSON(fSenderHandler, handlerName, methodName, log);
			JSONRPC2Response response = smh.dispatchRequestWaitResponse(requestIdentifier, "key", key);
			return (smh.getResponseString(response, methodName));
		} catch (Exception ex)
		{
			log.error("Error in " + methodName + " method:\n" + ex);
			return (null);
		}
	}

	@Override
	public boolean setProperty(String key, String value)
	{
		if ((key == null) || (value == null))
			return (false);
		String handlerName = "primary";
		String methodName = "setProperty";
		try
		{
			String requestIdentifier = fSenderHandler.getNewRequestIdentifier();
			HRSenderMethodHandlerJSON smh = new HRSenderMethodHandlerJSON(fSenderHandler, handlerName, methodName, log);
			JSONRPC2Response response = smh.dispatchRequestWaitResponse(requestIdentifier, "key", key, "value", value);

			return (smh.getResponseBoolean(response, methodName));
		} catch (Exception ex)
		{
			log.error("Error in " + methodName + " method:\n" + ex);
			return (false);
		}
	}

	@Override
	public boolean setEventsListener(SHEventsListenerInterface eventsListener)
	{
		try
		{
			log.error("setEventListener method not implemented");
			return (true); // true if successful false if not
		} catch (Exception ex)
		{
			log.error("Error in setEventsListener method:\n" + ex);
			return (false);
		}
	}

	@Override
	public boolean setAudioDataHandler(SHAudioDataHandlerInterface audioDataHandler)
	{
		try
		{
			log.error("setAudioDataListener method not implemented");
			return (true); // true if successful false if not
		} catch (Exception ex)
		{
			log.error("Error in setAudioDataListener method:\n" + ex);
			return (false);
		}
	}

	@Override
	public boolean setCustomeHandlers(SHCustomeHandlersInterface customeHandlers)
	{
		try
		{
			log.error("setCustomeHandlers method not implemented");
			return (true); // true if successful false if not
		} catch (Exception ex)
		{
			log.error("Error in setCustomeHandlers method:\n" + ex);
			return (false);
		}
	}

	public void checkEngine()
	{
		/* 
		* Usually this method is overridden on the client side To ensure 
		* engine initialization etc and not 
		* sent over the transport layer as it is called often and inefficient 
		* to implement this way 
		*/
	}

}
