/*
* Copyright (c) Isaac Porat 2011 - 2013 all rights reserved.
* This file is released under the GNU General Public License, version 2, or (at your option) any later version.
* See the file SpeechHub_License.txt for more details.
*/

package hrpc.sh.server;

import hrpc.common.HRReceiverHandlerJSONInterface;
import hrpc.handler.HRReceiverMethodHandlerJSON;

import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;

import log.common.CascadingLoggerInterface;

import sh.connections.SHEngineConnectorInterface;

import com.thetransactioncompany.jsonrpc2.JSONRPC2Error;
import com.thetransactioncompany.jsonrpc2.JSONRPC2ParseException;
import com.thetransactioncompany.jsonrpc2.JSONRPC2Request;

public class SHSecondaryServerHandler implements Runnable
{
	private CascadingLoggerInterface log;
	private BlockingQueue<String> fJSONRequestStringList = new LinkedBlockingQueue<String>();
	private HRReceiverMethodHandlerJSON fRMH;
	private SHEngineConnectorInterface fEngine;

	public SHSecondaryServerHandler(HRReceiverHandlerJSONInterface receiverHandler, SHEngineConnectorInterface engine,
			CascadingLoggerInterface logger)
	{
		fEngine = engine;
		log = logger.clone(SHSecondaryServerHandler.class);
		fRMH = new HRReceiverMethodHandlerJSON(receiverHandler, log);
		Thread methodsThread = new Thread(this);
		methodsThread.setName(SHSecondaryServerHandler.class.getName());
		methodsThread.start();
	}

	public void handleMethods(String jsonRequestString)
	{
		try
		{
			fJSONRequestStringList.put(jsonRequestString);
		} catch (InterruptedException ex)
		{
			log.error("Error in handleMethods method:\n" + ex);
		}
	}

	private void handleMethodsFromQueue()
	{
		String jsonRequestString = null;
		try
		{
			jsonRequestString = fJSONRequestStringList.take();
		} catch (InterruptedException ex)
		{
			log.error("Error in handleMethodsFromQueue method:\n" + ex);
		}
		JSONRPC2Request request = null;
		try
		{
			request = JSONRPC2Request.parse(jsonRequestString);
			String methodName = request.getMethod();
			// For efficiency methods order  is bbased according to their expected relative frequency
			if (methodName.equals("cancel"))
				handleCancel(request);
			else if (methodName.equals("pause"))
				handlePause(request);
			else if (methodName.equals("resume"))
				handleResume(request);
			else
			{
				log.error("Error in routeMethod method:\n Method " + methodName + " not found");
				fRMH.dispatchResponseError(JSONRPC2Error.METHOD_NOT_FOUND, (String) request.getID());
				return;
			}
		} catch (JSONRPC2ParseException ex)
		{
			log.error("Error in routeMethod method:\n" + ex);
			fRMH.dispatchResponseError(JSONRPC2Error.PARSE_ERROR, "0");
		} catch (JSONRPC2Error ex)
		{
			log.error("Error in routeMethod method:\n" + ex);
			fRMH.dispatchResponseError(ex, (String) request.getID());
		}
	}

	private void handleCancel(JSONRPC2Request request) throws JSONRPC2Error
	{
		boolean status = fEngine.cancel();
		fRMH.dispatchResponseBoolean(status, (String) request.getID());
	}

	private void handlePause(JSONRPC2Request request) throws JSONRPC2Error
	{
		boolean status = fEngine.pause();
		fRMH.dispatchResponseBoolean(status, (String) request.getID());
	}

	private void handleResume(JSONRPC2Request request) throws JSONRPC2Error
	{
		boolean status = fEngine.resume();
		fRMH.dispatchResponseBoolean(status, (String) request.getID());
	}

	@Override
	public void run()
	{
		for (;;)
		{
			handleMethodsFromQueue();
		}
	}

}
