/*
* Copyright (c) Isaac Porat 2011 - 2013 all rights reserved.
* This file is released under the GNU General Public License, version 2, or (at your option) any later version.
* See the file SpeechHub_License.txt for more details.
*/

package hrpc.sh.server;

import log.common.CascadingLoggerInterface;
import hrpc.common.HRReceiverHandlerJSONInterface;
import hrpc.common.HRReceiverRouterHandlerJSONInterface;
import sh.connections.SHEngineConnectorInterface;

public class SHServerHandlerRouter implements HRReceiverRouterHandlerJSONInterface
{
	private CascadingLoggerInterface log;
	private HRReceiverHandlerJSONInterface fReceiverHandler;
	private SHPrimaryServerHandler fPrimaryHandler;
	private SHSecondaryServerHandler fSecondaryHandler;
	private SHEngineConnectorInterface fEngine;

	public SHServerHandlerRouter(SHEngineConnectorInterface engine, CascadingLoggerInterface logger)
	{
		log = logger.clone(SHServerHandlerRouter.class);
		fEngine = engine;
	}

	public void setReceiverHandler(HRReceiverHandlerJSONInterface receiverHandler)
	{
		fReceiverHandler = receiverHandler;
		fPrimaryHandler = new SHPrimaryServerHandler(fReceiverHandler, fEngine, log);
		fSecondaryHandler = new SHSecondaryServerHandler(fReceiverHandler, fEngine, log);
	}

	public boolean routeHandlers(String handlerName, String jsonRequestString)
	{
		if (handlerName.equals("primary"))
			fPrimaryHandler.handleMethods(jsonRequestString);
		else if (handlerName.equals("secondary"))
			fSecondaryHandler.handleMethods(jsonRequestString);
		else if (handlerName.equals("quit"))
			return (false);
		else
			log.error("Error in routeHandlers methods:\n Handler " + handlerName + " not found");
		// Consider here to implement quit command
		return (true); // Do not close connection
	}

}
