/*
* Copyright (c) Isaac Porat 2011 - 2013 all rights reserved.
* This file is released under the GNU General Public License, version 2, or (at your option) any later version.
* See the file SpeechHub_License.txt for more details.
*/

package hrpc.transport;

import hrpc.common.HRHandlerFactoryInterface;
import hrpc.common.HRHandlerInterface;
import hrpc.common.HRMessageHandlerInterface;
import hrpc.common.HRTransportInterface;
import hrpc.handler.HRMessageHandlerServer;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;

import log.common.CascadingLoggerInterface;

public class HRServerMultiSocketInet
{
	private CascadingLoggerInterface log;
	private HRHandlerFactoryInterface fHandlerFactory;
	private int fServerPort = 0;
	private int fMaxConnections = 0; // 0 indicates no limit
	private boolean fDebugMode = false;

	public HRServerMultiSocketInet(int serverPort, int maxConnections, HRHandlerFactoryInterface handlerFactory,
			boolean debugMode, CascadingLoggerInterface logger)
	{
		log = logger.clone(HRServerMultiSocketInet.class);
		fServerPort = serverPort;
		fMaxConnections = maxConnections;
		fHandlerFactory = handlerFactory;
		fDebugMode = debugMode;

		HandleConnections handleConnections = new HandleConnections();
		Thread connectionThread = new Thread(handleConnections);
		connectionThread.setName(HRServerMultiSocketInet.class.getName());
		connectionThread.start();
	}

	private class HandleConnections implements Runnable
	{

		// Listen for incoming connections and handle them
		public void run()
		{
			int i = 0;
			try
			{
				ServerSocket serverSocket = new ServerSocket(fServerPort);
				Socket clientSocket;

				while ((i++ < fMaxConnections) || (fMaxConnections == 0))
				{
					clientSocket = serverSocket.accept();
					HRTransportInterface transportSocketServerClient = new HRTransportSocketInetServer(clientSocket,
							"UTF-8");
					HRHandlerInterface handler = fHandlerFactory.getNewInstance();
					HRMessageHandlerInterface messageHandler = new HRMessageHandlerServer(handler, false, fDebugMode,
							log);
					messageHandler.setTransport(transportSocketServerClient);
					messageHandler.open();
				}
			} catch (IOException ex)
			{
				log.error("IOException on server socket accept: " + ex);
			}
		}
	}

}
