/*
* Copyright (c) Isaac Porat 2011 - 2013 all rights reserved.
* This file is released under the GNU General Public License, version 2, or (at your option) any later version.
* See the file SpeechHub_License.txt for more details.
*/

package hrpc.transport;

import hrpc.common.HRMessageHandlerInterface;
import hrpc.common.HRTransportInterface;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;

import log.common.CascadingLoggerInterface;

public class HRServerSingleSocketInet
{
	private CascadingLoggerInterface log;
	HRMessageHandlerInterface fMessageHandler;
	private int fServerPort = 0;
	private int fMaxConnections = 1;

	public HRServerSingleSocketInet(int serverPort, HRMessageHandlerInterface messageHandler,
			CascadingLoggerInterface logger)
	{
		log = logger.clone(HRServerSingleSocketInet.class);
		fServerPort = serverPort;
		fMessageHandler = messageHandler;
		HandleConnections handleConnections = new HandleConnections();
		Thread connectionThread = new Thread(handleConnections);
		connectionThread.setName(HRServerSingleSocketInet.class.getName());
		connectionThread.start();
	}

	private class HandleConnections implements Runnable
	{

		// Listen for incoming connections and handle them
		public void run()
		{
			int i = 0;
			try
			{
				ServerSocket serverSocket = new ServerSocket(fServerPort);
				Socket clientSocket;

				while ((i++ < fMaxConnections) || (fMaxConnections == 0))
				{
					clientSocket = serverSocket.accept();
					HRTransportInterface transportSocketServerClient = new HRTransportSocketInetServer(clientSocket,
							"UTF-8");
					fMessageHandler.setTransport(transportSocketServerClient);
					fMessageHandler.open();
				}
			} catch (IOException ex)
			{
				log.error("IOException on server socket accept: " + ex);
			}
		}
	}

}
