/*
* Copyright (c) Isaac Porat 2011 - 2013 all rights reserved.
* This file is released under the GNU General Public License, version 2, or (at your option) any later version.
* See the file SpeechHub_License.txt for more details.
*/

package hrpc.transport;

import hrpc.common.HRTransportInterface;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.PrintStream;

import log.common.CascadingLoggerInterface;

public class HRTransportPipedClient implements HRTransportInterface
{
	@SuppressWarnings("unused")
	private CascadingLoggerInterface log;
	private BufferedReader fIn;
	private InputStream fInRaw;
	private PrintStream fOut;
	private String fFormat;

	public HRTransportPipedClient(String format, CascadingLoggerInterface logger) throws IOException
	{
		log = logger.clone(HRTransportPipedClient.class);
		fFormat = format;
	}

	@Override
	public BufferedReader getStdin()
	{
		return (fIn);
	}

	@Override
	public InputStream getStdinRaw()
	{
		return (fInRaw);
	}

	@Override
	public PrintStream getStdout()
	{
		return (fOut);
	}

	@Override
	public void open() throws IOException
	{
		HRPipeBridge pipeBridge = HRPipeBridge.getInstance();
		fInRaw = pipeBridge.getClientInputStream();
		fIn = new BufferedReader(new InputStreamReader(fInRaw, fFormat));
		OutputStream outputStream = pipeBridge.getClientOutputStream();
		fOut = new PrintStream(outputStream, true, fFormat);
	}

	@Override
	public void close() throws IOException
	{
		// Important to avoid IOException
		fOut.close();
		// Note fIn.close gets stuck probably due to the while loop
		//fIn.close();
		fOut = null;
		fIn = null;
	}

}
