/*
* Copyright (c) Isaac Porat 2011 - 2013 all rights reserved.
* This file is released under the GNU General Public License, version 2, or (at your option) any later version.
* See the file SpeechHub_License.txt for more details.
*/

package hrpc.transport;

// todo this class was not tested
import hrpc.common.HRTransportInterface;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintStream;

import org.newsclub.net.unix.AFUNIXSocket;
import org.newsclub.net.unix.AFUNIXSocketAddress;
import org.newsclub.net.unix.AFUNIXSocketException;

import core.utilities.StackTrace;

import log.common.BasicLogger;
import log.common.CascadingLoggerInterface;

public class HRTransportSocketUnixClient implements HRTransportInterface
{
	private CascadingLoggerInterface log;
	private AFUNIXSocket fSocket;
	private String fSocketPath;
	private PrintStream fOut;
	private BufferedReader fIn;
	private InputStream fInRaw;
	private String fFormat;
	private Process fProcess;
	private String[] fCommandLineArray;
	private String fCurrentDirectory;
	private String fProcessIdentifierMessage;
	private StderrStreamConsumer fErrorConsumer;
	private boolean fKeepServerProcessAliveOnClose = false;

	public HRTransportSocketUnixClient(AFUNIXSocket socket, String format) throws IOException
	{
		fSocket = socket;
		fFormat = format;
	}

	public HRTransportSocketUnixClient(String socketPath, String format) throws IOException
	{
		fSocketPath = socketPath;
		fFormat = format;
	}

	public HRTransportSocketUnixClient(String[] commandLineArray, String currentDirectory,
			String processIdentifierMessage, String socketPath, boolean keepServerProcessAliveOnClose, String format,
			CascadingLoggerInterface logger) throws IOException
	{
		log = logger.clone(HRTransportSocketUnixClient.class);
		fCommandLineArray = commandLineArray;
		fCurrentDirectory = currentDirectory;
		fProcessIdentifierMessage = processIdentifierMessage;
		fSocketPath = socketPath;
		fKeepServerProcessAliveOnClose = keepServerProcessAliveOnClose;
		fFormat = format;
	}

	public BufferedReader getStdin()
	{
		return (fIn);
	}

	public InputStream getStdinRaw()
	{
		return (fInRaw);
	}

	public PrintStream getStdout()
	{
		return (fOut);
	}

	public void open() throws IOException
	{
		if (fCommandLineArray != null)
			startProcess();
		if ((fSocket == null) || (fSocket.isClosed()))
			openSocket();
		fInRaw = fSocket.getInputStream();
		fIn = new BufferedReader(new InputStreamReader(fInRaw, fFormat));
		fOut = new PrintStream(fSocket.getOutputStream(), true, fFormat);
	}

	public void close() throws IOException
	{
		if ((fSocket != null) && (fSocket.isClosed() == false))
			fSocket.close();
		fSocket = null;
		if ((fCommandLineArray != null) && (fKeepServerProcessAliveOnClose == false))
			endProcess();
		fIn = null;
		fOut = null;
	}

	public void openSocket() throws IOException
	{
		if (fSocketPath == null)
		{
			throw new IOException("Cannot open socket, socketPath not specified");
		}
		fSocket = AFUNIXSocket.newInstance();
		try
		{
			fSocket.connect(new AFUNIXSocketAddress(new File(fSocketPath)));
		} catch (AFUNIXSocketException ex)
		{
			throw new IOException("Cannot connect to server");
		}

	}

	private void startProcess() throws IOException
	{
		if (fCurrentDirectory != null)
			fProcess = Runtime.getRuntime().exec(fCommandLineArray, null, new File(fCurrentDirectory));
		else
			fProcess = Runtime.getRuntime().exec(fCommandLineArray);
		// Start the process stderr consumer which transfers process error to SH stderr  
		fErrorConsumer = new StderrStreamConsumer(fProcessIdentifierMessage, fProcess.getErrorStream());
		fErrorConsumer.start();
	}

	public void endProcess() throws IOException
	{
		try
		{
			if (fProcess != null)
			{
				fProcess.destroy();
			}
		} finally
		{
			try
			{
				fProcess = null;
				// Trying to close stream does not return, probably because server still in run loop waiting for a connection? 
				//if (fErrorConsumer != null)
				//					fErrorConsumer.closeStream();
				fErrorConsumer = null;
			} catch (Exception ex)
			{
			}
		}
	}

	/**
	 * Consumer thread for capturing stderr from external process.
	 * 
	 */
	private class StderrStreamConsumer extends Thread
	{
		private InputStream fInputStream;
		private String fProcessIdentifierMessage;

		public StderrStreamConsumer(String processIdentifierMessage, InputStream stderrStream)
		{
			fProcessIdentifierMessage = processIdentifierMessage;
			fInputStream = stderrStream;
		}

		@SuppressWarnings("unused")
		public void closeStream() throws IOException
		{
			fInputStream.close();
		}

		@Override
		public void run()
		{
			try
			{
				InputStreamReader inputStreamReader = new InputStreamReader(fInputStream);
				BufferedReader bufferedReader = new BufferedReader(inputStreamReader);
				String line = null;
				while ((line = bufferedReader.readLine()) != null)
				{
					if (fProcessIdentifierMessage != null)
					{
						BasicLogger.notifyPrintln("From " + fProcessIdentifierMessage + " process (prefixed with p::");
						fProcessIdentifierMessage = null;
					}
					BasicLogger.notifyPrintln("p:: " + line);
				}
			} catch (Exception ex)
			{
				log.error("Error in StderrStreamConsumer run method: in " + fProcessIdentifierMessage + " process:\n"
						+ StackTrace.stackToString(ex));
			}
		}
	}

}
