/*
* Copyright (c) Isaac Porat 2011 - 2013 all rights reserved.
* This file is released under the GNU General Public License, version 2, or (at your option) any later version.
* See the file SpeechHub_License.txt for more details.
*/

package log.common;

/**
 * Simple logger implementing CascadingLoggerInterface
 * All printing directed through BasicLogger class to allow redirection flexibility
 * All printing using BasicLogger.errPrintln method to ensure correct sequencing of messages
 * 
 * @author Isaac Porat
 *
 */
public class SimpleLogger implements CascadingLoggerInterface
{
	private String fClassName = "";
	/*
	* Log levels 
	* Error - 1
	* Warning - 2
	* Info - 3
	* Debug - 4
	*/
	private int fLogLevel = 0;

	public SimpleLogger(Class<?> clazz, int logLevel)
	{
		fClassName = clazz.getName() + ":\n";
		fLogLevel = logLevel;
	}

	public CascadingLoggerInterface clone(Class<?> clazz)
	{
		return (new SimpleLogger(clazz, fLogLevel));
	}

	/**
	 * Notify prints always regardless of log levels and without class information
	 * @param message to print
	 */
	@Override
	public void notify(String message)
	{
		BasicLogger.errorPrintln(message);
	}

	@Override
	public void error(String message)
	{
		if (fLogLevel >= 1)
			println("Error", fClassName + message);
	}

	@Override
	public void error(Exception exception)
	{
		if (fLogLevel >= 1)
			println("Error", fClassName + exception.toString());
	}

	@Override
	public void warn(String message)
	{
		if (fLogLevel >= 2)
			println("Warn", fClassName + message);
	}

	@Override
	public void warn(Exception exception)
	{
		if (fLogLevel >= 2)
			println("Warn", fClassName + exception.toString());
	}

	@Override
	public void info(String message)
	{
		if (fLogLevel >= 3)
			println("Info", fClassName + message);
	}

	@Override
	public void info(Exception exception)
	{
		if (fLogLevel >= 3)
			println("Info", fClassName + exception.toString());
	}

	@Override
	public void debug(String message)
	{
		if (fLogLevel == 4)
			println("Debug", fClassName + message);
	}

	@Override
	public void debug(Exception exception)
	{
		if (fLogLevel == 4)
			println("Debug", fClassName + exception.toString());
	}

	private void println(String type, String message)
	{
		BasicLogger.errorPrintln(type + ": " + message);
	}
}
