/*
* Copyright (c) Bill Cox 2011 - 2013 all rights reserved.
* This file is released under the GNU General Public License, version 2, or (at your option) any later version.
* See the file SpeechHub_License.txt for more details.
*/

package sh.audio;

import log.common.CascadingLoggerInterface;
import sh.connections.SHMessageEnvelopeInterface;
import sh.connections.SHAudioDataHandlerInterface;

public class SHWaveListener implements SHAudioDataHandlerInterface
{
	private CascadingLoggerInterface log;
	private SHWave fWave;

	public SHWaveListener(String fileName, int sampleRate, CascadingLoggerInterface logger)
	{
		log = logger.clone(SHWaveListener.class);
		fWave = new SHWave(fileName, sampleRate, logger);
	}

	@Override
	public boolean processAudio(byte[] samples)
	{
		try
		{
			fWave.addData(samples);
			return (true);
		} catch (Exception ex)
		{
			log.error("Error in processAudio (bytes[]) method:\n" + ex);
			return (false);
		}
	}

	@Override
	public boolean processAudio(SHMessageEnvelopeInterface messageEnvelope)
	{
		try
		{
			for (;;)
			{
				byte[] bytes = messageEnvelope.takeAudio();
				processAudio(bytes);
				if (bytes.length == 0)
					return (true);
			}
		} catch (Exception ex)
		{
			log.error("Error in processAudio method:\n" + ex);
			return (false);
		}
	}

	public boolean save()
	{
		return fWave.save();
	}

}
