/*
* Copyright (c) Isaac Porat 2011 - 2013 all rights reserved.
* This file is released under the GNU General Public License, version 2, or (at your option) any later version.
* See the file SpeechHub_License.txt for more details.
*/

package sh.audio.javasound;

import java.util.*;
import javax.sound.sampled.*;

/**
 * This Javasound utility found on the internet 
 * It query the computer for ports and lists them
 * Note that it uses tabs to show structure
*
 */
public class QueryPortsUtility
{
	public QueryPortsUtility()
	{
	}

	public static void main(String[] args) throws Exception
	{
		probePort();
	}

	//The probePort() method first goes through the Mixers on the system and find the Port Mixer.

	public static void probePort() throws Exception
	{
		ArrayList<Mixer.Info> mixerInfos = new ArrayList<Mixer.Info>(Arrays.asList(AudioSystem.getMixerInfo()));
		Line.Info portInfo = new Line.Info(Port.class);
		for (Mixer.Info mixerInfo : mixerInfos)
		{
			Mixer mixer = AudioSystem.getMixer(mixerInfo);

			//Once the Port Mixer is located, it is probed for all the Ports that it has. Recall that a Port is a subclass of Line. Ports can be located using the getSourceLineInfo()
			//call (on the Mixer interface). Note that Ports are separated into source (for recording) and target (for playback). The source/target sense here is from
			//the perspective of the Mixer. For every Line (Port) that is located, the showControls() method is called.

			if (mixer.isLineSupported(portInfo))
			{
				// found a Port Mixer
				disp("Found mixer: " + mixerInfo.getName());
				disp("\t" + mixerInfo.getDescription());
				disp("Source Line Supported:");
				ArrayList<Line.Info> srcInfos = new ArrayList<Line.Info>(Arrays.asList(mixer.getSourceLineInfo()));
				for (Line.Info srcInfo : srcInfos)
				{
					Port.Info pi = (Port.Info) srcInfo;
					disp("\t" + pi.getName() + ", " + (pi.isSource() ? "source" : "target"));
					showControls(mixer.getLine(srcInfo));
				} // of for Line.Info
				disp("Target Line Supported:");
				ArrayList<Line.Info> targetInfos = new ArrayList<Line.Info>(Arrays.asList(mixer.getTargetLineInfo()));
				for (Line.Info targetInfo : targetInfos)
				{
					Port.Info pi = (Port.Info) targetInfo;
					disp("\t" + pi.getName() + ", " + (pi.isSource() ? "source" : "target"));
					showControls(mixer.getLine(targetInfo));
				}
			} // of if
				// (mixer.isLineSupported)
		} // of for (Mixer.Info)
	}

	//The showControls() method first opens the Line. Note that you have to open the line in order to access controls on the line. It then iterates through the
	//controls on the line and print information. Many controls are of the CompoundControl type. This means that it in turn contains other controls.

	private static void showControls(Line inLine) throws Exception
	{
		// must open the line to get
		// at controls
		inLine.open();
		disp("\t\tAvailable controls:");
		ArrayList<Control> ctrls = new ArrayList<Control>(Arrays.asList(inLine.getControls()));
		for (Control ctrl : ctrls)
		{
			disp("\t\t\t" + ctrl.toString());
			if (ctrl instanceof CompoundControl)
			{
				CompoundControl cc = ((CompoundControl) ctrl);
				ArrayList<Control> ictrls = new ArrayList<Control>(Arrays.asList(cc.getMemberControls()));
				for (Control ictrl : ictrls)
					disp("\t\t\t\t" + ictrl.toString());
			} // of if (ctrl instanceof)
		} // of for(Control ctrl)
		inLine.close();
	}

	private static void disp(String msg)
	{
		System.out.println(msg);
	}

}
