/*
* Copyright (c) Isaac Porat 2011 - 2013 all rights reserved.
* This file is released under the GNU General Public License, version 2, or (at your option) any later version.
* See the file SpeechHub_License.txt for more details.
*/

package sh.audio.windows;

import java.util.Arrays;
import java.util.List;

import sh.audio.windows.JNATime.Kernel32.SYSTEMTIME;

import com.sun.jna.*;
import com.sun.jna.win32.StdCallLibrary;

public class JNATime
{

	// kernel32.dll uses the __stdcall calling convention (check the function
	// declaration for "WINAPI" or "PASCAL"), so extend StdCallLibrary
	// Most C libraries will just extend com.sun.jna.Library,
	public interface Kernel32 extends StdCallLibrary
	{
		// Method declarations, constant and structure definitions go here

		Kernel32 INSTANCE = (Kernel32) Native.loadLibrary("kernel32", Kernel32.class);
		// Optional: wraps every call to the native library in a
		// synchronized block, limiting native calls to one at a time
		Kernel32 SYNC_INSTANCE = (Kernel32) Native.synchronizedLibrary(INSTANCE);

		/*
		The INSTANCE variable is for convenient reuse of a single instance of the library. Alternatively, you can load the library into a local variable so that
		it will be available for garbage collection when it goes out of scope. A Map of options may be provided as the third argument to loadLibrary to customize
		the library behavior; some of these options are explained in more detail below. The SYNC_INSTANCE is also optional; use it if you need to ensure that your
		native library has only one call to it at a time.

		Declare methods that mirror the functions in the target library by defining Java methods with the same name and argument types as the native function (refer
		to the basic mappings below or the detailed table of type mappings). You may also need to declare native structures to pass to your native functions. To
		do this, create a class within the interface definition that extends Structure and add public fields (which may include arrays or nested structures). 
		*/

		public static class SYSTEMTIME extends Structure
		{
			public short wYear;
			public short wMonth;
			public short wDayOfWeek;
			public short wDay;
			public short wHour;
			public short wMinute;
			public short wSecond;
			public short wMilliseconds;

			protected List<String> getFieldOrder()
			{
				return Arrays.asList(new String[]
				{ "wYear", "wMonth", "wDayOfWeek", "wDay", "wHour", "wMinute", "wSecond", "wMilliseconds", });
}
		
		}

		void GetSystemTime(SYSTEMTIME result);
}

	public static void timeTest()
	{
		//You can now invoke methods on the library instance just like any other Java class.
Kernel32 lib = Kernel32.INSTANCE;
		SYSTEMTIME time = new SYSTEMTIME();
		lib.GetSystemTime(time);

		System.out.println("Today's integer value is " + time.wDay);

	}
	
	public static void main(String[] args)
	{
timeTest();		
	}

}
