/*
* Copyright (c) Isaac Porat 2011 - 2013 all rights reserved.
* This file is released under the GNU General Public License, version 2, or (at your option) any later version.
* See the file SpeechHub_License.txt for more details.
*/

package sh.audio.windows;

import java.util.Arrays;

import java.util.List;

import com.sun.jna.Library;
import com.sun.jna.Native;
import com.sun.jna.Pointer;
import com.sun.jna.Structure;
import com.sun.jna.ptr.IntByReference;
/*
* This class attempted to use the waveOut interface to access Windows multi media directly
 *A problem encountered with the prepareHeader function see the end of this file for details
 *This happens only when the byte array contains non zero values (real case scenario) hence the initialization of the byte array in main
 *This avenue was not persuede for the time being
 *Se the main method for some experimentation with defining the header variable global which did not solve anything
 * Se also the alternativePrepareHeader method used in the experimentation
 *attempts to align the structures (see commented code) did not solve the problem either
 */
public class SHWaveOutJNAWrapper
{
	private static SHWaveOutJNAWrapper fWOW;
	//Constants mostly defined in mmsystem.h
	// WAVEFORMATEX structure values
	private final static int WAVE_FORMAT_PCM = 1;
	// waveOutOpen function arguments 
	private final static int WAVE_MAPPER = -1;
	private final static int CALLBACK_NULL = 0;
	// functions return values
	private final static int MMSYSERR_NOERROR = 0; // success

	public SHWaveOutJNAWrapper()
	{
fWaveOut = WaveOutInterface.waveOutInstance;
		}
	
	private WaveOutInterface fWaveOut; // JNA multi media waveOut library 
	private int fHWO; // Handle to waveOut device 

	public static void main(String[] args)
	{
		fWOW = new SHWaveOutJNAWrapper();
if (fWOW.open(22050, (short) 16, (short) 1) == false)
			return;
		System.err.println("open success");
		byte[] samples = new byte[64];
		for(int n = 0; n < samples.length; n++)
		{
			samples[n] = 1;
		}
		fWOW.fHeader = new WaveOutInterface.WAVEHDR(); 
		if(fWOW.alternativePrepareHeader(samples) == false)
				return;
		System.exit(0);
		// Continuation of the original code below
		WaveOutInterface.WAVEHDR header = new WaveOutInterface.WAVEHDR(); 
if(fWOW.prepareHeader(header, samples) == false)
		return;
		System.err.println("prepareHeader success");
		if(fWOW.write(header) == false)
			return;
			System.err.println("write success");
		try
		{
			Thread.sleep(1000);
		} catch (InterruptedException e)
		{
			}
		if(fWOW.unprepareHeader(header) == false)
			return;
		System.err.println("unprepareHeader success");
		if (fWOW.close())
			System.err.println("close method success");
	}

	public interface WaveOutInterface extends Library
	//public interface WaveOutInterface extends StdCallLibrary
	{

		WaveOutInterface waveOutInstance = (WaveOutInterface) Native.loadLibrary("winmm", WaveOutInterface.class);

		// Information at:
		// http://msdn.microsoft.com/en-gb/library/windows/hardware/ff538799%28v=vs.85%29.aspx

		public static class WAVEFORMATEX extends Structure
		{
			
			/*
			public WAVEFORMATEX () 
			{
				setAlignType(Structure.ALIGN_MSVC);
				}
			*/
			public short wFormatTag;
			public short nChannels;
			public int nSamplesPerSec;
			public int nAvgBytesPerSec;
			public short nBlockAlign;
			public short cbSize;
			public short wBitsPerSample;

			protected List<String> getFieldOrder()
			{
				return Arrays.asList(new String[]
				{ "wFormatTag", "nChannels", "nSamplesPerSec", "nAvgBytesPerSec", "nBlockAlign", "wBitsPerSample",
						"cbSize", });
			}
		}

		public static class WAVEHDR extends Structure
		{

			/*

			// WAVEHDR structure information in:
			// http://msdn.microsoft.com/en-us/library/aa909814.aspx
			  
			  LPSTR lpData;
			DWORD dwBufferLength;
			DWORD dwBytesRecorded;
			DWORD dwUser;
			DWORD dwFlags;
			DWORD dwLoops;
			struct wavehdr_tag* lpNext;
			DWORD reserved;
			*/

			
			/*
			public WAVEHDR () 
			{
				setAlignType(Structure.ALIGN_MSVC);
				}
			*/
			
			
			public byte[] samples;
			public int dwBufferLength;
			public int dwBytesRecorded;
			public int dwUser;
			public int dwFlags;
			public int dwLoops;
			public Pointer lpNext;
			public int reserved;

			protected List<String> getFieldOrder()
			{
				return Arrays
						.asList(new String[]
						{ "samples", "dwBufferLength", "dwBytesRecorded", "dwUser", "dwFlags", "dwLoops", "lpNext","reserved"});
			}

		}

		/*

				Function outWaveOpen signature 
				
				MMRESULT waveOutOpen(

				MMRESULT waveOutOpen(
		  LPHWAVEOUT phwo,
		  UINT uDeviceID,
		  LPWAVEFORMATEX pwfx,
		  DWORD dwCallback,
		  DWORD dwInstance,
		  DWORD fdwOpen
		);

		One of the values described in the following table is returned.

		MMSYSERR_NOERROR

		Success.

		MMSYSERR_INVALHANDLE

		Specified device handle is invalid.

		MMSYSERR_BADDEVICEID

		Specified device identifier is out of range.

		MMSYSERR_NODRIVER

		No device driver is present.

		MMSYSERR_NOMEM

		Unable to allocate or lock memory.

		WAVERR_BADFORMAT

		Attempted to open with an unsupported waveform-audio format.

		WAVERR_SYNC

		Device is synchronous but waveOutOpen was called without using the WAVE_ALLOWSYNC flag.

		*/

		public int waveOutOpen(IntByReference phwo, int uDeviceID, WAVEFORMATEX wfx, int dwCallback, int dwInstance,
				int fdwOpen);

		/*
		
		Function waveOutClose signature
		
		MMRESULT waveOutClose(
		HWAVEOUT hwo 
		); 
				list end

		Return Value--------------------------------------------------------------------------------

		One of the values described in the following table is returned.

		MMSYSERR_NOERROR

		Success.

		MMSYSERR_INVALHANDLE

		Specified device handle is invalid.

		MMSYSERR_NODRIVER

		No device driver is present.

		MMSYSERR_NOMEM

		Unable to allocate or lock memory.

		WAVERR_STILLPLAYING

		There are still buffers in the queue.

		*/

		public int waveOutClose(int hwo);

		/*

		waveOutPrepareHeader function signature
		MMRESULT waveOutPrepareHeader(
  HWAVEOUT hwo, 
  LPWAVEHDR pwh, 
  UINT cbwh 

Return Value--------------------------------------------------------------------------------

MMSYSERR_NOERROR

Success.

MMSYSERR_INVALHANDLE

Specified device handle is invalid.
//##
MMSYSERR_INVALPARAM

The buffer's base address is not aligned with the sample size.

MMSYSERR_NODRIVER

No device driver is present.

MMSYSERR_NOMEM

Unable to allocate or lock memory.

*/
		
public int 		waveOutPrepareHeader(int  hwo, WAVEHDR pwh, int cbwh); 

				/*
	
	MMRESULT waveOutWrite(
  HWAVEOUT hwo, 
  LPWAVEHDR pwh, 
  UINT cbwh) 

Return Value--------------------------------------------------------------------------------

MMSYSERR_NOERROR

Success.

MMSYSERR_INVALHANDLE

Specified device handle is invalid.

MMSYSERR_NODRIVER

No device driver is present.

MMSYSERR_NOMEM

Unable to allocate or lock memory.

WAVERR_UNPREPARED

Data block pointed to by the pwh parameter has not been prepared.

WAVERR_BADFORMAT

Unsupported wave format.

WAVERR_STILLPLAYING

The waveform output device is still playing some other data.

WAVERR_UNPREPARED

The wave header is not prepared.

WAVERR_SYNC

The waveform output device is synchronous.

	*/
	
public int 		waveOutWrite(int  hwo, WAVEHDR pwh, int cbwh); 

/*

MMRESULT waveOutUnprepareHeader(
  HWAVEOUT hwo, 
  LPWAVEHDR pwh, 
  UINT cbwh 
); 

Return Value--------------------------------------------------------------------------------

MMSYSERR_NOERROR

Success.

MMSYSERR_INVALHANDLE

Specified device handle is invalid.

MMSYSERR_NODRIVER

No device driver is present.

MMSYSERR_NOMEM

Unable to allocate or lock memory.

WAVERR_STILLPLAYING

Data block pointed to by the pwh parameter is still in the queue.
*/

public int 		waveOutUnprepareHeader(int  hwo, WAVEHDR pwh, int cbwh); 

}

	public boolean open(int samplesPerSec, short bitsPerSample, short channels)
	{
		WaveOutInterface.WAVEFORMATEX wfx = new WaveOutInterface.WAVEFORMATEX();
		wfx.wFormatTag = WAVE_FORMAT_PCM;
		wfx.nChannels = channels;
		wfx.nSamplesPerSec = samplesPerSec;
		wfx.wBitsPerSample = bitsPerSample;
		wfx.nBlockAlign = (short) (bitsPerSample / 8 * channels);
		wfx.nAvgBytesPerSec = samplesPerSec * wfx.nBlockAlign;
		wfx.cbSize = 0; // 0 indicates no extra format information  

		/*
		*/

		/*
		public int waveOutOpen(
		  IntByReference   phwo,
		  int uDeviceID,
		  WAVEFORMATEX wfx,
		  int dwCallback,
		  int dwInstance,
		  int fdwOpen
		);

		*/

		IntByReference phwo = new IntByReference();
		int mmresult = fWaveOut.waveOutOpen(phwo, WAVE_MAPPER, wfx, 0, 0, CALLBACK_NULL);
		if (mmresult == MMSYSERR_NOERROR)
		{
			fHWO = phwo.getValue();
			return (true);
		}
		System.err.println("Error in waveOutOpen method,code returned: " + mmresult);
		return (false);

		/*
		// From NVDA
		winmm.waveOutOpen(byref(waveout), self.outputDeviceID, LPWAVEFORMATEX(wfx), self._waveout_event, 0, CALLBACK_EVENT)
					self._waveout = waveout.value
					self._prev_whdr = None

		*/
	}

	public boolean close()
	{
		int mmresult = fWaveOut.waveOutClose(fHWO);
		if (mmresult == MMSYSERR_NOERROR)
			return (true);
		System.err.println("Error in waveOutClose method,code returned: " + mmresult);
		return (false);
}

	public boolean prepareHeader (WaveOutInterface.WAVEHDR header, byte[] samples)
	{
header.samples = samples;
	header.dwBufferLength = samples.length;
int mmresult = fWaveOut.waveOutPrepareHeader(fHWO, header, header.size());
	if (mmresult == MMSYSERR_NOERROR)
	return(true);
	System.err.println("Error in waveOutPrepareHeader method,code returned: " + mmresult);
	return (false);
	}

	public boolean write(WaveOutInterface.WAVEHDR header)
	{
int mmresult = fWaveOut.waveOutWrite(fHWO, header, header.size());
	if (mmresult == MMSYSERR_NOERROR)
		return (true);
	System.err.println("Error in waveOutWrite method,code returned: " + mmresult);
	return (false);
	}
	
	public boolean unprepareHeader(WaveOutInterface.WAVEHDR header)
	{
int mmresult = fWaveOut.waveOutUnprepareHeader(fHWO, header, header.size());
	if (mmresult == MMSYSERR_NOERROR)
		return (true);
	System.err.println("Error in waveOutUnprepareHeader method,code returned: " + mmresult);
	return (false);
	}

	public void writeAudioData(byte[] samples)
	{
System.err.println("hello");
		WaveOutInterface.WAVEHDR header = new WaveOutInterface.WAVEHDR(); 
System.err.println("1");
		if(prepareHeader(header, samples) == false)
		return;
		System.err.println("2");
System.err.println("prepareHeader success");
		if(write(header) == false)
			return;
			System.err.println("write success");
		try
		{
			Thread.sleep(1000);
		} catch (InterruptedException e)
		{
			}
		if(unprepareHeader(header) == false)
			return;
		System.err.println("unprepareHeader success");
		}

	private WaveOutInterface.WAVEHDR fHeader;
	
	// This method is only for experimentation see problem described at the top of the file
	public boolean alternativePrepareHeader (byte[] samples)
	{
		
		byte[] padded = new byte[samples.length + 1];
		System.arraycopy(samples, 0, padded, 0, samples.length);

		
		
		
		fHeader.samples = padded;
fHeader.dwBufferLength = padded.length;
int mmresult = fWaveOut.waveOutPrepareHeader(fHWO, fHeader, fHeader.size());
	if (mmresult == MMSYSERR_NOERROR)
	return(true);
	System.err.println("Error in waveOutPrepareHeader method,code returned: " + mmresult);
	return (false);
	}


	
	
	
	
	
	}


/*

The following is a solution to the problem
wen calling waveOutPrepareHeader

with the error

MMSYSERR_INVALPARAM which is code 11

The buffer's base address is not aligned with the sample size.

The problem ended up being not an issue with lpData (which was being allocated properly by LocalAlloc), but an issue with the way that the WAVEHDR structure
itself was being passed to the PInvoked waveOutPrepareHeader. 

Most material on waveOut (even for C#) has you passing in an object of the header's type. But in order to stop getting the error, I had to make a block
of data using Marshal.AllocHGlobal and copy the header into it with Marshal.StructureToPtr. The IntPtr from the AllocHGlobal then is passed into waveOutPrepareHeader
as such and that resolved the issue.

Here's some rough sample code for any who run up against a similar problem.

WAVEHDR header = new WAVEHDR();
//Read or setup header data here
IntPtr headerPtr = Marshal.AllocHGlobal(Marshal.SizeOf(header));
Marshal.StructureToPtr(header, headerPtr, true);
waveOutPrepareHeader(hWaveOut, headerPtr, (uint)Marshal.SizeOf(header));

This assumes that you set your PInvoke for everything that uses the header to an IntPtr type instead of a WAVEHDR one.

From there, use the headerPtr for all the places where you need to pass in that header. You'll have to handle the freeing of memory yourself at the appropriate
location as you would expect.

*/
