/*
* Copyright (c) Isaac Porat 2011 - 2013 all rights reserved.
* This file is released under the GNU General Public License, version 2, or (at your option) any later version.
* See the file SpeechHub_License.txt for more details.
*/

package sh.comm;

import java.io.IOException;

import log.common.CascadingLoggerInterface;
import sh.common.SHMessageEnvelope;
import sh.connections.SHCustomeHandlersInterface;
import sh.connections.SHEngineInterface;
import sh.connections.SHMessageEnvelopeInterface;
import sh.connections.SHModuleInterface;
import sh.connections.SHModuleTimestampInterface;
import sh.connections.SHAudioDataHandlerInterface;
import sh.connections.SHEventsListenerInterface;
import sh.connections.SHTTSInterface;
import sh.connections.SHUpdateParametersInterface;
import sun.misc.BASE64Decoder;
import sun.misc.BASE64Encoder;
import core.utilities.StackTrace;

/**
 * TTS coupled client server emulation adaptor 
 * emulation For development where both client and server are together in the class 
 * creating new messageEnvelope with encoding decoding  conversion
 * 
 * @author Isaac Porat
 *
 */
public class SHTTSCoupledClientServerCommEmulationAdaptor implements SHTTSInterface, SHModuleInterface,
		SHEngineInterface
{
	private CascadingLoggerInterface log;
	private SHTTSInterface fTTS;
	private SHAudioDataHandlerInterface fClientAudioDataHandler;
	private SHAudioDataHandlerInterface fServerAudioDataHandler;
	private BASE64Encoder fBASE64Encoder = new BASE64Encoder();
	private BASE64Decoder fBASE64Decoder = new BASE64Decoder();

	public SHTTSCoupledClientServerCommEmulationAdaptor(SHTTSInterface tts, CascadingLoggerInterface logger)
	{
		log = logger.clone(SHTTSCoupledClientServerCommEmulationAdaptor.class);
		fTTS = tts;
		fServerAudioDataHandler = new ServerAudioDataHandler(log);
		fTTS.setAudioDataHandler(fServerAudioDataHandler);
	}

	@Override
	public boolean configureTiming(SHModuleTimestampInterface moduleTimestamp, boolean masterCancel)
	{
		// Not applicable
		return (false);
	}

	@Override
	public boolean initialize()
	{
		return (fTTS.initialize());
	}

	@Override
	public boolean close()
	{
		return (fTTS.close());
	}

	@Override
	public boolean setPriority(int priorityLevel, boolean selfCancel)
	{
		// Not applicable
		return (false);
	}

	@Override
	public String[] getEngineIdentifiers()
	{
		try
		{
			return (fTTS.getEngineIdentifiers());
		} catch (Exception ex)
		{
			log.error("Error in getEngineIdentifiers method:\n" + StackTrace.stackToString(ex));
			return (null);
		}
	}

	@Override
	public boolean selectEngine(String engineIdentifier)
	{
		try
		{
			return (fTTS.selectEngine(engineIdentifier));
		} catch (Exception ex)
		{
			log.error("Error in selectEngine method:\n" + StackTrace.stackToString(ex));
			return (false);
		}
	}

	@Override
	public String getSelectedEngineIdentifier()
	{
		return (fTTS.getSelectedEngineIdentifier());
	}

	@Override
	public boolean selectLanguage(String languageCode)
	{
		return (fTTS.selectLanguage(languageCode));
	}

	@Override
	public String getSelectedLanguageCode()
	{
		return (fTTS.getSelectedLanguageCode());
	}

	@Override
	public String[] getVoiceIdentifiers()
	{
		return (fTTS.getVoiceIdentifiers());
	}

	@Override
	public boolean selectVoice(String voiceIdentifier)
	{
		return (fTTS.selectVoice(voiceIdentifier));
	}

	@Override
	public String getSelectedVoiceIdentifier()
	{
		return (fTTS.getSelectedVoiceIdentifier());
	}

	@Override
	public String[] getVariantIdentifiers()
	{
		return (fTTS.getVariantIdentifiers());
	}

	@Override
	public boolean selectVariant(String variantIdentifier)
	{
		return (fTTS.selectVariant(variantIdentifier));
	}

	@Override
	public String getSelectedVariantIdentifier()
	{
		return (fTTS.getSelectedVariantIdentifier());
	}

	@Override
	public boolean speak(SHMessageEnvelopeInterface messageEnvelope)
	{
		// Not applicable
		return (false);
	}

	@Override
	public boolean speak(int messageIdentifier, String message, boolean speakSync)
	{
		return (fTTS.speak(messageIdentifier, message, speakSync));
	}

	@Override
	public boolean speakChar(int messageIdentifier, String message, boolean speakSync)
	{
		return (fTTS.speakChar(messageIdentifier, message, speakSync));
	}

	@Override
	public boolean speakKey(int messageIdentifier, String message, boolean speakSync)
	{
		return (fTTS.speakKey(messageIdentifier, message, speakSync));
	}

	@Override
	public boolean speakSpell(int messageIdentifier, String message, boolean speakSync)
	{
		return (fTTS.speakSpell(messageIdentifier, message, speakSync));
	}

	@Override
	public boolean pause()
	{
		return (fTTS.pause());
	}

	@Override
	public boolean resume()
	{
		return (fTTS.resume());
	}

	@Override
	public boolean stop()
	{
		return (fTTS.stop());
	}

	@Override
	public boolean cancel()
	{
		return (fTTS.cancel());
	}

	@Override
	public boolean setVolume(float volume)
	{
		return (fTTS.setVolume(volume));
	}

	@Override
	public float getVolume()
	{
		return (fTTS.getVolume());
	}

	@Override
	public boolean setRate(float rate)
	{
		return (fTTS.setRate(rate));
	}

	@Override
	public float getRate()
	{
		return (fTTS.getRate());
	}

	@Override
	public boolean setRateBoostMode(boolean mode)
	{
		return (fTTS.setRateBoostMode(mode));
	}

	@Override
	public boolean setPitch(float pitch)
	{
		return (fTTS.setPitch(pitch));
	}

	@Override
	public float getPitch()
	{
		return (fTTS.getPitch());
	}

	@Override
	public boolean setPunctuationLevel(String level)
	{
		return (fTTS.setPunctuationLevel(level));
	}

	@Override
	public boolean setCharCapitalIndicator(String charCapitalIndicator, String arg)
	{
		return (fTTS.setCharCapitalIndicator(charCapitalIndicator, arg));
	}

	@Override
	public boolean setSilenceBeforeMessage(int silenceMilliseconds)
	{
		return (fTTS.setSilenceBeforeMessage(silenceMilliseconds));
	}

	@Override
	public boolean setProperty(String key, String value)
	{
		return (fTTS.setProperty(key, value));
	}

	@Override
	public String getProperty(String key)
	{
		return (fTTS.getProperty(key));
	}

	@Override
	public boolean setEventsListener(SHEventsListenerInterface eventsListener)
	{
		return (fTTS.setEventsListener(eventsListener));
	}

	@Override
	public boolean setAudioDataHandler(SHAudioDataHandlerInterface audioDataHandler)
	{
		fClientAudioDataHandler = audioDataHandler;
		return (true);
	}

	@Override
	public boolean setCustomeHandlers(SHCustomeHandlersInterface customeHandlers)
	{
		// Not applicable
		return (false);
	}

	@Override
	public boolean updateParametersDirectly(SHUpdateParametersInterface updateParameters)
	{
		// Not applicable
		return (false);
	}

	@Override
	public boolean setUpdateIdentifier(int updateIdentifier)
	{
		// Not applicable
		return (false);
	}

	@Override
	public boolean isUpdateAccepted(int updateIdentifier)
	{
		// Not applicable
		return (false);
	}

	private SHMessageEnvelopeInterface fClientMessageEnvelope;

	private void createClientMessageEnvelope(int messageIdentifier, String message, int sampleRate)
	{
		fClientMessageEnvelope = new SHMessageEnvelope(messageIdentifier, message);
		fClientMessageEnvelope.setAudioSampleRate(sampleRate);
		fClientMessageEnvelope.initializeAudio();
	}

	private byte[] base64EncodeDecodeTest(byte[] bytes) throws IOException
	{
		String encodedString = fBASE64Encoder.encode(bytes);
		//System.err.println(encodedString.length() + "\n");
		//System.err.println(encodedString);
		return (fBASE64Decoder.decodeBuffer(encodedString));
	}

	private class ServerAudioDataHandler implements SHAudioDataHandlerInterface
	{
		private CascadingLoggerInterface log;

		public ServerAudioDataHandler(CascadingLoggerInterface logger)
		{
			log = logger.clone(ServerAudioDataHandler.class);
		}

		@Override
		public boolean processAudio(byte[] samples)
		{
			try
			{
				// Unused
				return (true);
			} catch (Exception ex)
			{
				log.error("Error in processAudio (bytes[]) method:\n" + ex);
				return (false);
			}
		}

		@Override
		public boolean processAudio(final SHMessageEnvelopeInterface messageEnvelope)
		{
			try
			{
				//##
				// Create a new messageEnvelope with the minimal parameters required
				createClientMessageEnvelope(messageEnvelope.getMessageIdentifier(), messageEnvelope.getMessage(),
						messageEnvelope.getAudioSampleRate());
				// process the client message envelope in another thread
				new Thread(new Runnable()
				{
					public void run()
					{
						try
						{
							for (;;)
							{
								byte[] bytes = messageEnvelope.takeAudio();
								// Make sure if a 0 length byte is sent to terminate player playAudioData method
								fClientMessageEnvelope.putAudio(base64EncodeDecodeTest(bytes));
								if (bytes.length == 0)
									return;
							}
						} catch (Exception ex)
						{
							log.error("Error in run method:\n" + ex);
							return;
						}
					}
				}).start();
				return (fClientAudioDataHandler.processAudio(fClientMessageEnvelope));
			} catch (Exception ex)
			{
				log.error("Error in processAudio method:\n" + ex);
				return (false);
			}
		}

	}

}
