/*
* Copyright (c) Isaac Porat 2011 - 2013 all rights reserved.
* This file is released under the GNU General Public License, version 2, or (at your option) any later version.
* See the file SpeechHub_License.txt for more details.
*/

package sh.common;

import java.util.HashMap;
import java.util.Map;

import sh.connections.SHClientParametersInterface;
import sh.connections.SHModuleTimestampInterface;
import sh.connections.SHAudioDataHandlerInterface;
import sh.connections.SHEventsListenerInterface;
import sh.connections.SHUpdateParametersInterface;

/**
 * Parameters associated with every server client
 * @author Isaac Porat
 *
 */
public class SHClientParameters implements SHClientParametersInterface
{
	private SHUpdateParametersInterface fUpdateParameters;
	private SHModuleTimestampInterface fModuleTimestamp;
	private SHEventsListenerInterface fEventsListener;
	private SHAudioDataHandlerInterface fAudioDataHandler;
	private Map<String, String> fProperties;
	private String fClientIdentifierCaption = "unidentified";
	private boolean fPropertiesUpdateRequired = false;

	public SHClientParameters(String clientIdentifier)
	{
		fUpdateParameters = new SHUpdateParameters(clientIdentifier);
		fProperties = new HashMap<String, String>();
	}

	/**
	 * Get the new clone of SHUpdateParameters if it has been updated
	 * otherwise get its last clone 
	 * This means that all SHMessageEnvelope objects share an cloned object of SHUpdateParameters as long as it has not been modified   
	 * @return clone of SHUpdateParameters object as described above 
	 */
	@Override
	public SHUpdateParametersInterface getClonedUpdateParameters()
	{
		return (fUpdateParameters.getClone());
	}

	@Override
	public String getClientIdentifier()
	{
		return (fUpdateParameters.getClientIdentifier());
	}

	@Override
	public String getClientIdentifierCaption()
	{
		return (fClientIdentifierCaption);
	}

	@Override
	public void setClientIdentifierCaption(String clientIdentifierCaption)
	{
		fClientIdentifierCaption = clientIdentifierCaption;
	}

	@Override
	public SHModuleTimestampInterface getModuleTimestamp()
	{
		return (fModuleTimestamp);
	}

	@Override
	public void setModuleTimestamp(SHModuleTimestampInterface moduleTimestamp)
	{
		fModuleTimestamp = moduleTimestamp;
	}

	@Override
	public SHEventsListenerInterface getEventsListener()
	{
		return (fEventsListener);
	}

	@Override
	public void setEventsListener(SHEventsListenerInterface eventsListener)
	{
		fEventsListener = eventsListener;
	}

	@Override
	public SHAudioDataHandlerInterface getAudioDataHandler()
	{
		return (fAudioDataHandler);
	}

	@Override
	public void setAudioDataHandler(SHAudioDataHandlerInterface audioDataHandler)
	{
		fAudioDataHandler = audioDataHandler;
	}

	// Convenient methods to set fUpdateParameters

	@Override
	public void setEngineIdentifier(String engineIdentifier)
	{
		fUpdateParameters.setEngineIdentifier(engineIdentifier);
		fUpdateParameters.markNewUpdate();
	}

	@Override
	public String getEngineIdentifier()
	{
		return (fUpdateParameters.getEngineIdentifier());
	}

	@Override
	public void setLanguageCode(String languageCode)
	{
		fUpdateParameters.setLanguageCode(languageCode);
		fUpdateParameters.markNewUpdate();
	}

	@Override
	public String getLanguageCode()
	{
		return (fUpdateParameters.getLanguageCode());
	}

	@Override
	public void setVoiceIdentifier(String voiceIdentifier)
	{
		fUpdateParameters.setVoiceIdentifier(voiceIdentifier);
		fUpdateParameters.markNewUpdate();
	}

	@Override
	public String getVoiceIdentifier()
	{
		return (fUpdateParameters.getVoiceIdentifier());
	}

	@Override
	public void setVariantIdentifier(String variantIdentifier)
	{
		fUpdateParameters.setVariantIdentifier(variantIdentifier);
		fUpdateParameters.markNewUpdate();
	}

	@Override
	public String getVariantIdentifier()
	{
		return (fUpdateParameters.getVariantIdentifier());
	}

	@Override
	public void setVolume(float volume)
	{
		fUpdateParameters.setVolume(volume);
		fUpdateParameters.markNewUpdate();
	}

	@Override
	public float getVolume()
	{
		return (fUpdateParameters.getVolume());
	}

	@Override
	public void setRate(float rate)
	{
		fUpdateParameters.setRate(rate);
		fUpdateParameters.markNewUpdate();
	}

	@Override
	public float getRate()
	{
		return (fUpdateParameters.getRate());
	}

	@Override
	public void setPitch(float pitch)
	{
		fUpdateParameters.setPitch(pitch);
		fUpdateParameters.markNewUpdate();
	}

	@Override
	public float getPitch()
	{
		return (fUpdateParameters.getPitch());
	}

	@Override
	public void setProperty(String key, String value)
	{
		fProperties.put(key, value);
		fPropertiesUpdateRequired = true;
	}

	@Override
	public String getProperty(String key)
	{
		return (fProperties.get(key));
	}

	@Override
	public boolean isPropertiesUpdateRequired()
	{
		return (fPropertiesUpdateRequired);
	}

	@Override
	public void resetPropertiesUpdateRequired()
	{
		fPropertiesUpdateRequired = false;
	}

}
