/*
* Copyright (c) Isaac Porat 2011 - 2013 all rights reserved.
* This file is released under the GNU General Public License, version 2, or (at your option) any later version.
* See the file SpeechHub_License.txt for more details.
*/

package sh.common;

import java.io.File;

import sh.services.SHTTSModulesControllerServiceInterface;
import sh.settings.SHEnhancedPropertiesList;

public class SHConfig
{
	private static String fSettingsBaseDirectory = "";
	private static String fEnginesBaseDirectory = "";
	private static String fSystemBaseDirectory = "";
	private static SHEnhancedPropertiesList fPropertiesList;
	private static SHTTSModulesControllerServiceInterface fTTSModulesControllerService;

	public static String getSettingsBaseDirectory()
	{
		return (fSettingsBaseDirectory);
	}

	private static void configureSettingsBaseDirectory(String settingsParentDirectory, String settingsDirectoryName)
	{
		if (settingsParentDirectory.equals("."))
			settingsParentDirectory = System.getProperty("user.home") + "/.";
		else if (settingsParentDirectory.matches(".+[\\\\\\./]$") == false)
			settingsParentDirectory += "/";
		settingsParentDirectory += settingsDirectoryName;
		settingsParentDirectory = settingsParentDirectory.replaceAll("[\\\\/]$", ""); // Remove trailing slash left without directory name
		// Only absolute path is acceptable
		if (settingsParentDirectory.startsWith("."))
			settingsParentDirectory = getAbsolutePathFromRelativeToCurrentDirectory(settingsParentDirectory);
		fSettingsBaseDirectory = settingsParentDirectory;
	}

	public static String getEnginesBaseDirectory()
	{
		return (fEnginesBaseDirectory);
	}

	private static void configureEnginesBaseDirectory(String enginesParentDirectory, String enginesDirectoryName)
	{
		if (enginesParentDirectory.equals("."))
			enginesParentDirectory = System.getProperty("user.home") + "/.";
		else if (enginesParentDirectory.matches(".+[\\\\\\./]$") == false)
			enginesParentDirectory += "/";
		enginesParentDirectory += enginesDirectoryName;
		enginesParentDirectory = enginesParentDirectory.replaceAll("[\\\\/]$", ""); // Remove trailing slash left without directory name
		// Only absolute path is acceptable
		if (enginesParentDirectory.startsWith("."))
			enginesParentDirectory = getAbsolutePathFromRelativeToCurrentDirectory(enginesParentDirectory);
		fEnginesBaseDirectory = enginesParentDirectory;
	}

	public static String getSystemBaseDirectory()
	{
		return (fSystemBaseDirectory);
	}

	private static void configureSystemBaseDirectory(String systemParentDirectory, String systemDirectoryName)
	{
		if (systemParentDirectory.equals("."))
			systemParentDirectory = System.getProperty("user.home") + "/.";
		else if (systemParentDirectory.matches(".+[\\\\\\./]$") == false)
			systemParentDirectory += "/";
		systemParentDirectory += systemDirectoryName;
		systemParentDirectory = systemParentDirectory.replaceAll("[\\\\/]$", ""); // Remove trailing slash left without directory name
		// Only absolute path is acceptable
		if (systemParentDirectory.startsWith("."))
			systemParentDirectory = getAbsolutePathFromRelativeToCurrentDirectory(systemParentDirectory);
		fSystemBaseDirectory = systemParentDirectory;
	}

	private static String getAbsolutePathFromRelativeToCurrentDirectory(String relativeCurrentDirectory)
	{
		/* 
		*Java with path relative to current directory "./something/something
		*file.getAbsolutePath method produces
		*path which can create interpretation  problems, 
		*hence this method
		*/
		String absolutePath = (new File(relativeCurrentDirectory)).getAbsolutePath();
		absolutePath = absolutePath.replace("\\", "/");
		absolutePath = absolutePath.replaceAll("\\/\\.\\/", "/");
		return (absolutePath);
	}

	public static String getServerProperty(String key)
	{
		return (fPropertiesList.get(key));
	}

	public static void setServerProperties(SHEnhancedPropertiesList propertiesList)
	{
		fPropertiesList = propertiesList;
	}

	public static SHTTSModulesControllerServiceInterface getTTSModulesControllerService()
	{
		return (fTTSModulesControllerService);
	}

	public static void setTTSModulesControllerService(SHTTSModulesControllerServiceInterface ttsModulesControllerService)
	{
		fTTSModulesControllerService = ttsModulesControllerService;
	}

	public static void configureSettingsLocations(String settingsParentDirectory, String settingsDirectory)
	{
		configureSettingsBaseDirectory(settingsParentDirectory, settingsDirectory);
	}

	public static void configureEnginesLocations(String enginesParentDirectory, String enginesDirectory)
	{
		configureEnginesBaseDirectory(enginesParentDirectory, enginesDirectory);
	}

	public static void configureSystemLocations(String systemParentDirectory, String systemDirectory)
	{
		configureSystemBaseDirectory(systemParentDirectory, systemDirectory);
	}

}
