/*
* Copyright (c) Isaac Porat 2011 - 2013 all rights reserved.
* This file is released under the GNU General Public License, version 2, or (at your option) any later version.
* See the file SpeechHub_License.txt for more details.
*/

package sh.common;

import java.util.Date;
import java.util.concurrent.LinkedBlockingQueue;

import sh.connections.SHConstants.MessageAudioFeedState;
import sh.connections.SHConstants.MessageTextToAudioConversionState;
import sh.connections.SHMessageEnvelopeInterface;
import sh.connections.SHUpdateParametersInterface;
import sh.connections.SHConstants.CapitalIndicator;
import sh.connections.SHConstants.MessageState;
import sh.connections.SHConstants.MessageType;

public class SHMessageEnvelope extends SHMethodEnvelope implements SHMessageEnvelopeInterface
{
	private String fClientIdentifier;
	private int fMessageIdentifier = 0;
	private int fMessageFragmentIdentifier = 0;
	private Date fTimestamp;
	private boolean fMessageSynchronized = true;
	private MessageState fMessageState = MessageState.SINGLE;
	private MessageType fMessageType = MessageType.STANDARD;
	private MessageTextToAudioConversionState fMessageTextToAudioConversionState = MessageTextToAudioConversionState.UNKNOWN;
	private MessageAudioFeedState fMessageAudioFeedState = MessageAudioFeedState.NONE;
	private CapitalIndicator fCapitalIndicator = CapitalIndicator.NONE;
	private int fPriorityLevel = 4;
	private boolean fSelfCancel = false;
	private SHUpdateParametersInterface fUpdateParameters;
	private String fMessage;
	private LinkedBlockingQueue<byte[]> fAudioBuffer;
	private int fAudioSampleRate = -1;
	private float fTrailingSilenceTrimRatio = 1f; 
	private int fTotalAudioWritten = 0;
	private boolean fAudioCacheUsed = false;
	private int fSilenceBeforeMessage = 0; // In milliseconds

	public SHMessageEnvelope(int messageIdentifier, String message)
	{
		fMessageIdentifier = messageIdentifier;
		fMessage = message;
	}

	@Override
	public String getClientIdentifier()
	{
		return (fClientIdentifier);
	}

	@Override
	public void setClientIdentifier(String clientIdentifier)
	{
		fClientIdentifier = clientIdentifier;
	}

	@Override
	public int getMessageIdentifier()
	{
		return (fMessageIdentifier);
	}

	@Override
	public int getMessageFragmentIdentifier()
	{
		return (fMessageFragmentIdentifier);
	}

	@Override
	public void setMessageFragmentIdentifier(int messageFragmentIdentifier)
	{
		fMessageFragmentIdentifier = messageFragmentIdentifier;
	}

	public Date getTimestamp()
	{
		return (fTimestamp);
	}

	public void setTimestamp(Date timestamp)
	{
		fTimestamp = timestamp;
	}

	public boolean getMessageSynchronized()
	{
		return (fMessageSynchronized);
	}

	public void setMessageSynchronized(boolean messageSynchronized)
	{
		fMessageSynchronized = messageSynchronized;
	}

	public MessageState getMessageState()
	{
		return (fMessageState);
	}

	public void setMessageState(MessageState messageState)
	{
		fMessageState = messageState;
	}

	public MessageType getMessageType()
	{
		return (fMessageType);
	}

	public void setMessageType(MessageType messageType)
	{
		fMessageType = messageType;
	}

	public CapitalIndicator getCapitalIndicator()
	{
		return (fCapitalIndicator);
	}

	public void setCapitalIndicator(CapitalIndicator capitalIndicator)
	{
		fCapitalIndicator = capitalIndicator;
	}

	@Override
	public int getPriorityLevel()
	{
		return (fPriorityLevel);
	}

	@Override
	public void setPriorityLevel(int priorityLevel)
	{
		fPriorityLevel = priorityLevel;
	}

	@Override
	public boolean getSelfCancel()
	{
		return (fSelfCancel);
	}

	@Override
	public void setSelfCancel(boolean selfCancel)
	{
		fSelfCancel = selfCancel;
	}

	@Override
	public SHUpdateParametersInterface getUpdateParameters()
	{
		return (fUpdateParameters);
	}

	@Override
	public void setUpdateParameters(SHUpdateParametersInterface updateParameters)
	{
		fUpdateParameters = updateParameters;
	}

	@Override
	public String getMessage()
	{
		return (fMessage);
	}

	@Override
	public void setMessage(String message)
	{
		fMessage = message;
	}

	@Override
	public void setSilenceBeforeMessage(int silenceMilliseconds)
	{
		fSilenceBeforeMessage = silenceMilliseconds;
	}

	@Override
	public int getSilenceBeforeMessage()
	{
		return (fSilenceBeforeMessage);
	}

	@Override
	public String toString()
	{
		return (fClientIdentifier + " " + Integer.toString(fMessageIdentifier) + "\n" + fMessage);
	}

	// Audio

	@Override
	public void initializeAudio()
	{
		// Buffer not bound but sentences are limited and not more than two message envelopes containing audio exists at the same time
		fAudioBuffer = new LinkedBlockingQueue<byte[]>();
	}

	@Override
	public int getAudioSampleRate()
	{
		return (fAudioSampleRate);
	}

	@Override
	public void setAudioSampleRate(int audioSampleRate)
	{
		fAudioSampleRate = audioSampleRate;
	}

	@Override
	public float getTrailingSilenceTrimRatio()
	{
		return (fTrailingSilenceTrimRatio);
	}

	@Override
	public void setTrailingSilenceTrimRatio(float trailingSilenceTrimRatio)
	{
	fTrailingSilenceTrimRatio = trailingSilenceTrimRatio;
	}
	
	@Override
	public void setTotalAudioWritten(int totalWritten)
	{
		fTotalAudioWritten = totalWritten;
	}

	@Override
	public int getTotalAudioWritten()
	{
		return (fTotalAudioWritten);
	}

	@Override
	public void setAudioCacheUsed(boolean audioCacheUsed)
	{
		fAudioCacheUsed = audioCacheUsed;
	}

	@Override
	public boolean getAudioCacheUsed()
	{
		return (fAudioCacheUsed);
	}

	@Override
	public void setMessageTextToAudioConversionState(MessageTextToAudioConversionState messageTextToAudioConversionState)
	{
		fMessageTextToAudioConversionState = messageTextToAudioConversionState;
	}

	@Override
	public MessageTextToAudioConversionState getMessageTextToAudioConversionState()
	{
		return (fMessageTextToAudioConversionState);
	}

	@Override
	public MessageAudioFeedState getMessageAudioFeedState()
	{
		return (fMessageAudioFeedState);
	}

	@Override
	public void putAudio(byte[] bytes) throws InterruptedException
	{
		if (bytes.length > 0)
			fMessageAudioFeedState = MessageAudioFeedState.STARTED;
		else
			fMessageAudioFeedState = MessageAudioFeedState.ENDED;
		fAudioBuffer.put(bytes);
		synchronized (this)
		{
			notifyAll();
		}
	}

	@Override
	public byte[] takeAudio() throws InterruptedException
	{
		return (fAudioBuffer.take());
	}

	@Override
	public void clearAudio()
	{
		fAudioBuffer.clear();
	}

	@Override
	public boolean isEmptyAudio()
	{
		return ((fAudioBuffer == null) || (fAudioBuffer.isEmpty()));
	}

	public boolean waitForMessageAudio() throws InterruptedException
	{
		synchronized (this)
		{
			while (fMessageAudioFeedState.equals(MessageAudioFeedState.NONE))
			{
				wait(50);
			}
			if (fMessageAudioFeedState.equals(MessageAudioFeedState.NONE))
				return (false);
		}
		return (true);
	}

}
