/*
* Copyright (c) Isaac Porat 2011 - 2013 all rights reserved.
* This file is released under the GNU General Public License, version 2, or (at your option) any later version.
* See the file SpeechHub_License.txt for more details.
*/

package sh.connections;

public class SHConstants
{

	// Enumeration

	public enum MethodName
	{
		INITIALIZE, CLOSE, GET_ENGINE_IDENTIFIERS, SELECT_ENGINE, SPEAK, SELECT_LANGUAGE, GET_VOICE_IDENTIFIERS, SELECT_VOICE, GET_VARIANT_IDENTIFIERS, SELECT_VARIANT, SET_VOLUME, SET_RATE, SET_PITCH, GET_PROPERTY, SET_PROPERTY, UPDATE_PARAMETERS_DIRECTLY, SET_AUDIO_DATA_HANDLER
	}

	public enum MessageType
	{
		STANDARD, CHAR, KEY, SPELL, SSML
	}

	public enum MessageState
	{
		BEGIN, MIDDLE, END, SINGLE
	}

	public enum MessageTextToAudioConversionState
	{
		UNKNOWN, SUCCESS, FAILURE
	}

	public enum MessageAudioFeedState
	{
		NONE, STARTED, ENDED
	}

	public enum CapitalIndicator
	{
		SPELL, PITCH, ICON, NONE
	}

	public enum PunctuationLevel
	{
		NONE, SOME, MOST, ALL
	}

	// General

	public final static String STATUS_NONE = "NONE"; // Mainly used by the server to return a string in situation where null is returned by a method

	// Parameters - client, message envelope etc 

	public final static String STRING_PARAMETER_ERROR = "ERROR";
	public final static float FLOAT_PARAMETER_ERROR = -999;
	public final static String STRING_PARAMETER_DISABLED = "";
	public final static float FLOAT_PARAMETER_DISABLED = -998;
	public final static String STRING_PARAMETER_UNDEFINED = "undefined";
	public final static float FLOAT_PARAMETER_UNDEFINED = -997;

	// Compound properties

	public final static String COMPOUND_PROP_SEPARATOR = "::";
	public final static String COMPOUND_PROP_MARKER_ENGINE_FACTORY = "engine.factory";

	// Server identifiers

	public final static String SERVER_IDENTIFIER = "speechhub";
	public final static String SERVER_MARKER = SERVER_IDENTIFIER + "-";
	public final static String SERVER_CAPTION = "Speech Hub";

	// Server properties

	public final static String PROP_SERVER_DEBUG_MODE = "server.debug.mode";
	public final static String PROP_SERVER_SOCKET_INET_PORT = "server.socket.inet.port";
	public final static String PROP_SERVER_SOCKET_UNIX_PATH = "server.socket.unix.path";
	public final static String PROP_SERVER_ENGINE_IDENTIFIER_CLIENT_DEFAULT = "server.engine.identifier.client.default";
	public final static String PROP_SERVER_AUTO_SHUTDOWN = "server.auto.shutdown";

	// Server transport types

	// Multi client socket inet
	public final static String SERVER_TRANSPORT_MULTI_SOCKET_INET = "MultiClientSocketInet";
	// Multi client socket unix
	public final static String SERVER_TRANSPORT_MULTI_SOCKET_UNIX = "MultiClientSocketUnix";
	// Single client socket inet (restarted and used individually by client applications
	public final static String SERVER_TRANSPORT_SINGLE_SOCKET_INET = "SingleClientSocketInet";
	// Stdio (always single client, started and used individually by client applications
	public final static String SERVER_TRANSPORT_STDIO = "Stdio";

	// Module properties

	public final static String PROP_MODULE_ENGINE_IDENTIFIER = "module.engine.identifier";
	public final static String PROP_MODULE_CLIENT_IDENTIFIER = "module.client.identifier";
	public final static String PROP_MODULE_HANDLES_SPLIT = "module.handles.split";
	public final static String PROP_MODULE_HANDLES_REPLACEMENT = "module.handles.replacement";
	public final static String PROP_MODULE_HANDLES_PUNCTUATION = "module.handles.punctuation";
	public final static String PROP_MODULE_PUNCTUATION_LEVEL = "module.punctuation.level";
	public final static String PROP_MODULE_CHAR_CAPITAL_INDICATOR = "module.char.capital.indicator";
	public final static String PROP_MODULE_CHAR_CAPITAL_INDICATOR_PITCH_LEVEL = "module.char.capital.indicator.pitch.level";
	public final static String PROP_MODULE_ROUTE_MESSAGE_TO_CHAR = "module.route.message.to.char";
	public final static String PROP_MODULE_SSML_MODE = "module.ssml.mode";
	public final static String PROP_MODULE_SPELL_MODE = "module.spelling.mode";
	public final static String PROP_MODULE_SILENCE_BEFORE_MESSAGE = "module.silence.before.message";
	public final static String PROP_MODULE_HANDLES_PAUSE_RESUME = "module.handles.pause.resume";

	// Module parameters

	public final static String MODULE_CHAR_CAPITAL_INDICATOR_NONE = "none";
	public final static String MODULE_CHAR_CAPITAL_INDICATOR_PITCH = "pitch";
	public final static String MARKER_MODULE_CHAR_CAPITAL_INDICATOR_PITCH_LEVEL = "pitchLevel=";
	public final static float MODULE_CHAR_CAPITAL_INDICATOR_PITCH_LEVEL_DEFAULT = 30;
	public final static String MODULE_CHAR_CAPITAL_INDICATOR_ICON = "icon";
	public final static String MODULE_CHAR_CAPITAL_INDICATOR_SPELL = "spell";

	// Engine properties

	public final static String PROP_ENGINE_CLIENT_IDENTIFIER = "client.identifier";
	public final static String PROP_ENGINE_SUPPORTS_SPEAK_SYNC = "engine.supports.speak.sync";
	public final static String PROP_ENGINE_SUPPORTS_VARIANTS = "engine.supports.variants";
	public final static String PROP_ENGINE_SUPPORTS_PUNCTUATION = "engine.supports.punctuation";
	public final static String PROP_ENGINE_PUNCTUATION_LEVEL = "engine.punctuation.level";
	public final static String PROP_ENGINE_SUPPORTS_PAUSE_RESUME = "engine.supports.pause.resume";
	public final static String PROP_ENGINE_SUPPORTS_SSML = "engine.supports.ssml";
	public final static String PROP_ENGINE_SSML_MODE = "engine.ssml.mode";
	public final static String PROP_ENGINE_SUPPORTS_AUDIO_DATA = "engine.supports.audio.data";
	public final static String PROP_ENGINE_REQUIRES_ISOLATION_RESTART = "engine.requires.isolation.restart";
	public final static String PROP_ENGINE_IDENTIFIER_DEFAULT = "engine.identifier.default";
	public final static String PROP_ENGINE_CONNECTOR_SUPPORTS_MULTI_ENGINES = "engine.connector.supports.multi.engines";
	public final static String PROP_ENGINE_DEBUG_MODE_CONNECTOR_IDENTIFIER = "engine.debug.mode.connector.identifier";
	public final static String PROP_ENGINE_CAPTION = "engine.caption";
	public final static String PROP_ENGINE_PROTOCOL = "engine.protocol";
	public final static String PROP_ENGINE_PROTOCOL_VERSION = "engine.protocol.version";
	public final static String PROP_ENGINE_TEXT_FORMAT = "engine.text.format";
	public final static String PROP_ENGINE_USE_SONIC = "engine.use.sonic";
	public final static String PROP_ENGINE_USE_SONIC_PITCH = "engine.use.sonic.pitch";
	public final static String PROP_ENGINE_SAMPLE_RATE = "engine.sample.rate";
	public final static String PROP_ENGINE_TRAILING_SILENCE_TRIM_RATIO = "engine.trailing.silence.trim.ratio";
	public final static String PROP_ENGINE_FORCE_SLASH_PATHSEP = "engine.force.slash.pathsep";
	public final static String PROP_ENGINE_KEEP_ALIVE = "engine.keep.alive";
	public final static String PROP_ENGINE_CHAR_VOLUME = "engine.char.volume";
	public final static String PROP_ENGINE_COMMAND_STDIO_WINDOWS = "engine.command.stdio.windows";
	public final static String PROP_ENGINE_COMMAND_STDIO_LINUX = "engine.command.stdio.linux";
	public final static String PROP_ENGINE_SERVER_SOCKET_PORT = "engine.server.socket.port";
	public final static String PROP_ENGINE_COMMAND_SOCKET_INET_WINDOWS = "engine.command.socket.inet.windows";
	public final static String PROP_ENGINE_COMMAND_SOCKET_INET_LINUX = "engine.command.socket.inet.linux";

	// Engine parameters

	public final static String ENGINE_IDENTIFIER_DEFAULT = "default";
	public final static String ENGINE_LANGUAGE_CODE_DEFAULT = "en";
	public final static String ENGINE_VOICE_IDENTIFIER_DEFAULT = "default";
	public final static String ENGINE_VARIANT_IDENTIFIER_DEFAULT = "default";
	public final static float ENGINE_VOLUME_MAX = 100.0f;
	public final static float ENGINE_VOLUME_MIN = 0.0f;
	public final static float ENGINE_VOLUME_DEFAULT = 100.0f;
	public final static float ENGINE_RATE_MAX = 100.0f;
	public final static float ENGINE_RATE_MIN = -100.0f;
	public final static float ENGINE_RATE_DEFAULT = 0.0f;
	public final static float ENGINE_PITCH_MAX = 100.0f;
	public final static float ENGINE_PITCH_MIN = -100.0f;
	public final static float ENGINE_PITCH_DEFAULT = 0.0f;
	public final static String ENGINE_PUNCTUATION_LEVEL_NONE = "none";
	public final static String ENGINE_PUNCTUATION_LEVEL_SOME = "some";
	public final static String ENGINE_PUNCTUATION_LEVEL_MOST = "most";
	public final static String ENGINE_PUNCTUATION_LEVEL_ALL = "all";

	// Engine connector and engine identifiers

	public final static String ENGINE_CONNECTOR_IDENTIFIER_TC_SSIP_CLIENT = "tc-ssip-client";
	public final static String ENGINE_CONNECTOR_IDENTIFIER_MULTI_NATIVE = "multi-native";
	public final static String ENGINE_IDENTIFIER_SAPI5 = "sapi5"; // SAPI5 audio plugin
	public final static String ENGINE_IDENTIFIER_SAPI5_COMMON = "sapi5c"; // SAPI5 self audio 
	public final static String ENGINE_IDENTIFIER_MSP = "msp";
	public final static String ENGINE_IDENTIFIER_FREETTS = "freetts";
	public final static String ENGINE_IDENTIFIER_ESPEAK = "espeak";
	public final static String ENGINE_IDENTIFIER_IBMTTS = "ibmtts";
	public final static String ENGINE_IDENTIFIER_MARYTTS = "marytts";

	// Engine protocol identifiers

	public final static String ENGINE_PROTOCOL_NATIVE = "stdio.native";
	public final static String ENGINE_PROTOCOL_SAPI5 = "com.sapi5"; // SAPI5 audio plugin
	public final static String ENGINE_PROTOCOL_SAPI5_COMMON = "com.sapi5c"; // SAPI5 self audio
	public final static String ENGINE_PROTOCOL_MSP = "com.msp";
	public final static String ENGINE_PROTOCOL_FREETTS = "stdio.connector.json";
	public final static String ENGINE_PROTOCOL_MARYTTS = "stdio.marytts";

	// Service TTS properties

	public final static String PROP_SERVICE_TTS_USE = "service.tts.use";
	public final static String PROP_SERVICE_TTS_CLIENT_MODULE_TYPE = "service.tts.client.module.type";

	public final static String SERVICE_TTS_USE_UNIVERSAL = "universal"; // Can be used as a client of server-engine applications
	public final static String SERVICE_TTS_USE_CLIENT = "client";
	public final static String SERVICE_TTS_USE_SERVER = "server";

	public final static String SERVICE_TTS_CLIENT_MODULE_TYPE_BYPASS = "bypass"; // Module bypass effectively directo connection to engine
	public final static String SERVICE_TTS_CLIENT_MODULE_TYPE_COMPLETE = "complete"; // Complete standalone stack
	public final static String SERVICE_TTS_CLIENT_MODULE_TYPE_SERVER_ENGINE = "server-engine"; // Server engine stack

	// Components wrapping configuration properties

	public final static String PROP_COMPONENTS_CONFIGURATION = "components.configuration";
	public final static String COMPONENTS_CONFIGURATION_SERVER = "server";
	public final static String COMPONENTS_CONFIGURATION_TTS = "tts";

	/*
	* SH Comm Message Data protocol (CMDP)
	* This comm protocol provides message envelope parameters
	* Protocol use both metadata and audio data in BASE64.
	* In SH a text message to be spoken is stored in message envelope object which contains the message itself
	* together with parameters associated with it and audio data.
	* The protocol contains relevant parameters and audio data obtained from the message envelope
	* SH has its own audio player based on Javasound but a client can use its own player if desired.
	* This protocol is mainly used by a client to obtain audio data from the SH server  for a spoken message which is then passed to a audio player
	* The server get an audioMessage events which signals the start of a message
	* The client can just read the audio data, request metadata stored in the message envelope
	* or most probably use a combination of both
	* In a typical operation, after receiving the audioMessage event
	* The client request the header metadata which contains parameters required for processing the message, the header metadata depends on the type of messages might not be required for each message 
	* Audio data encoded in BASE64 is always terminated with zero length byte array of audio followed by 
	* a line of status metadata.
	* 
	*    Metadata line format:
	* A metadata line comprise of data separated by separator characters
	* Referring to the constants below where relevant
	* the metadata separator is '::' separating the metadata line into fields
	* Each metadata line starts with a metadata indicator 'm' field.
	* The second field is a metadata group indicator which are currently 'header', 'status' and 'message'
	* other fields are optional and depend on the group
	* header group fields:
	* messageIdentifier, audioSampleRate and messageState
	* 
	* status group fields:
	* 'end' indicates end of message (transmitted even if the message is cancelled)
	* 'unavailable' - indicates that part of the message are still missing the client should wait
	* 'cancel' - indicates a cancelled message
	*  
	* 	 Message group contains only one field the text message
	* 
	* Note on field separator, all definitions are known and do not contain '::'.
	* In a message group, a message can contain any characters and therefore the
	* split method is limited to the fields expected to ensure correct operation
	*/

	public final static String CMDP_MESSAGE_AUDIODATA = "audiodata";
	public final static String CMDP_MESSAGE_METADATA = "metadata";
	public final static String CMDP_METADATA_SEPARATOR = "::";
	public final static String CMDP_METADATA_IDENTIFIER = "m";
	public final static String CMDP_METADATA_PREFIX = CMDP_METADATA_IDENTIFIER + CMDP_METADATA_SEPARATOR;
	public final static String CMDP_GROUP_HEADER = "header";
	public final static String CMDP_GROUP_HEADER_MESSAGESTATE_SINGLE = "single";
	public final static String CMDP_GROUP_HEADER_MESSAGESTATE_BEGIN = "begin";
	public final static String CMDP_GROUP_HEADER_MESSAGESTATE_MIDDLE = "middle";
	public final static String CMDP_GROUP_HEADER_MESSAGESTATE_END = "end";
	public final static String CMDP_GROUP_STATUS = "status";
	public final static String CMDP_GROUP_STATUS_UNAVAILABLE = "unavailable";
	public final static String CMDP_GROUP_STATUS_CANCEL = "cancel";
	public final static String CMDP_GROUP_STATUS_END = "end";
	public final static String CMDP_GROUP_STATUS_UNAVAILABLE_MARKER = CMDP_METADATA_PREFIX + CMDP_GROUP_STATUS
			+ CMDP_METADATA_SEPARATOR + CMDP_GROUP_STATUS_UNAVAILABLE;
	public final static String CMDP_GROUP_STATUS_CANCEL_MARKER = CMDP_METADATA_PREFIX + CMDP_GROUP_STATUS
			+ CMDP_METADATA_SEPARATOR + CMDP_GROUP_STATUS_CANCEL;
	public final static String CMDP_GROUP_STATUS_END_MARKER = CMDP_METADATA_PREFIX + CMDP_GROUP_STATUS
			+ CMDP_METADATA_SEPARATOR + CMDP_GROUP_STATUS_END;

	public final static String CMDP_GROUP_MESSAGE = "message";
	public final static String CMDP_GROUP_MESSAGE_BLANK = "blank";
	public final static String CMDP_GROUP_MESSAGE_OMITTED = "omitted";

}
