/*
* Copyright (c) Isaac Porat 2011 - 2013 all rights reserved.
* This file is released under the GNU General Public License, version 2, or (at your option) any later version.
* See the file SpeechHub_License.txt for more details.
*/

package sh.connections;

/**
 * Update parameters associated with every message spoken
 * These parameters always form part of the SHClientParameters 
 * and optionally of SHMessageEnvelope and thus queued together with a message  
 * 
 * @author Isaac Porat
 *
 */
public interface SHUpdateParametersInterface
{

	public String getClientIdentifier();

	public String getEngineIdentifier();

	public void setEngineIdentifier(String engineIdentifier);

	public String getLanguageCode();

	public void setLanguageCode(String languageCode);

	public String getVoiceIdentifier();

	public void setVoiceIdentifier(String voiceIdentifier);

	public String getVariantIdentifier();

	public void setVariantIdentifier(String voiceIdentifier);

	public float getVolume();

	public void setVolume(float volume);

	public float getRate();

	public void setRate(float rate);

	public float getPitch();

	public void setPitch(float pitch);

	public int getUpdateIdentifier();

	/**
	 * new updateParameters are always transfered to the engine 
	 * provided the updateIdentifier is greater than the previous one
	 * As the updateIdentifier is obtained globally this applies to all clients but in some cases reset update needs to be apply to ensure update when clients are switched and might still be on a lower
	 * updateIdentifier which is still the most updated one for that client. 
	 */
	public void markNewUpdate();

	/**
	 * UpdateParameters are transfered to the engine without affecting
	 * any updateParameters objects behind in the queue
	 * This means that until an updateIdentifier number is increased update behind in the queue will not be accepted	 
	  */
	public void markInsertUpdate();

	/**
	 * This update is always accepted and the update counter of the engine is reset
	 * This is used when client changed; a client might be on a lower update parameter which is still the recent update  for that client 
	 */
	public void markResetUpdate();

	public SHUpdateParametersInterface getClone();

	public boolean equals(SHUpdateParametersInterface updateParameters);

}
