/*
* Copyright (c) Isaac Porat 2011 - 2013 all rights reserved.
* This file is released under the GNU General Public License, version 2, or (at your option) any later version.
* See the file SpeechHub_License.txt for more details.
*/

package sh.connector.freeTTS;

import hrpc.common.HRMessageHandlerJSONInterface;
import hrpc.common.HRSenderHandlerJSONInterface;
import hrpc.common.HRTransportInterface;
import hrpc.handler.HRMessageHandlerClientJSONProtocolUnidirectional;
import hrpc.handler.HRSenderHandlerJSON;
import hrpc.transport.HRTransportSocketInetClient;

import core.utilities.StackTrace;

import log.common.CascadingLoggerInterface;
import log.common.SimpleLogger;
import sh.connections.SHConstants;
import sh.connections.SHEngineConnectorInterface;
import sh.engine.SHEngineCommand;

/**
 * hrpc socket transport json protocol setup 
 * @author Isaac Porat
 *
 */
public class SHFreeTTSSocketInetClientSetup
{
	private CascadingLoggerInterface log = null;
	private SHEngineCommand fEngineCommand;

	public static void main(String[] args)
	{
		int logLevel = 2;
		CascadingLoggerInterface logger = new SimpleLogger(SHFreeTTSSocketInetClientSetup.class, logLevel);
		try
		{
			/*
			* to run independently:
			* Uncomment location line as indicated below
			* Configure locations - settings and  engines
			* Comment before OSGi compilation to avoid package not found error
			*/
			//SHConfigDev.configureDevLocations(logger);
			SHFreeTTSSocketInetClientSetup setup = new SHFreeTTSSocketInetClientSetup(logger);
			SHEngineConnectorInterface engine = setup.getEngineConnector();
			engine.initialize();
			engine.selectVoice("kevin16");
			engine.speak(1, "Hello it works!", true);
			engine.close();
		} catch (Exception ex)
		{
			logger.error("Error in test method:\n" + StackTrace.stackToString(ex));
		}
	}

	public SHFreeTTSSocketInetClientSetup(CascadingLoggerInterface logger)
	{
		log = logger.clone(SHFreeTTSSocketInetClientSetup.class);
		fEngineCommand = new SHEngineCommand(logger);
	}

	public SHEngineConnectorInterface getEngineConnector()
	{
		try
		{
			String serverPort = fEngineCommand.getSocketInetServerPort(SHConstants.ENGINE_IDENTIFIER_FREETTS);
			String[] commandArray = fEngineCommand.getStartSocketInetCommand(SHConstants.ENGINE_IDENTIFIER_FREETTS,
					serverPort);
			HRTransportInterface client = new HRTransportSocketInetClient(commandArray, null, "freetts server",
					Integer.parseInt(serverPort), false, "UTF-8", log);
			HRSenderHandlerJSONInterface senderHandlerClient = new HRSenderHandlerJSON(log);
			HRMessageHandlerJSONInterface messageHandlerClient = new HRMessageHandlerClientJSONProtocolUnidirectional(
					client, senderHandlerClient, null, log);
			return (new SHConnectorFreeTTSClientAdaptor(messageHandlerClient, senderHandlerClient, log));
		} catch (Exception ex)
		{
			log.error("Error in getEngineConnector method:\n" + StackTrace.stackToString(ex));
			return (null);
		}
	}

}
