/*
* Copyright (c) Isaac Porat 2011 - 2013 all rights reserved.
* This file is released under the GNU General Public License, version 2, or (at your option) any later version.
* See the file SpeechHub_License.txt for more details.
*/

package sh.connector.freeTTS;

import hrpc.common.HRMessageHandlerJSONInterface;
import hrpc.common.HRSenderHandlerJSONInterface;
import hrpc.common.HRTransportInterface;
import hrpc.handler.HRMessageHandlerClientJSONProtocolUnidirectional;
import hrpc.handler.HRSenderHandlerJSON;
import hrpc.transport.HRTransportStdioClient;

import log.common.CascadingLoggerInterface;
import log.common.SimpleLogger;
import sh.connections.SHConstants;
import sh.connections.SHEngineConnectorInterface;
import sh.engine.SHEngineCommand;
import core.utilities.StackTrace;

/**
 * hrpc stdio streams  transport json protocol setup 
 * @author Isaac Porat
 *
 */
public class SHFreeTTSStdioClientSetup
{
	private CascadingLoggerInterface log;
	SHEngineCommand fEngineCommand;

	public static void main(String[] args)
	{
		int logLevel = 2;
		CascadingLoggerInterface logger = new SimpleLogger(SHFreeTTSStdioClientSetup.class, logLevel);
		try
		{
			/*
			* to run independently:
			* uncomment location line as indicated below
			* Configure locations - settings and  engines
			* Comment before OSGi compilation to avoid package not found error
			*/
			//SHConfigDev.configureDevLocations(logger);

			SHFreeTTSStdioClientSetup setup = new SHFreeTTSStdioClientSetup(logger);
			SHEngineConnectorInterface engine = setup.getEngineConnector();
			engine.initialize();
			engine.selectVoice("kevin16");
			engine.speak(1, "Hello it works!", true);
			engine.close();
		} catch (Exception ex)
		{
			logger.error("Error in test method:\n" + StackTrace.stackToString(ex));
		}
	}

	public SHFreeTTSStdioClientSetup(CascadingLoggerInterface logger)
	{
		log = logger.clone(SHFreeTTSStdioClientSetup.class);
		fEngineCommand = new SHEngineCommand(logger);
	}

	public SHEngineConnectorInterface getEngineConnector()
	{
		try
		{
			String[] commandArray = fEngineCommand.getStartStdioCommand(SHConstants.ENGINE_IDENTIFIER_FREETTS);
			if (commandArray == null)
			{
				log.error("Error in method: Command not found in engine properties file");
				return (null);
			}
			HRTransportInterface client = new HRTransportStdioClient(commandArray, null, "freetts server", "UTF-8", log);
			HRSenderHandlerJSONInterface senderHandlerClient = new HRSenderHandlerJSON(log);
			HRMessageHandlerJSONInterface messageHandlerClient = new HRMessageHandlerClientJSONProtocolUnidirectional(
					client, senderHandlerClient, null, log);
			return (new SHConnectorFreeTTSClientAdaptor(messageHandlerClient, senderHandlerClient, log));
		} catch (Exception ex)
		{
			log.error("Error in getEngineConnector method:\n" + StackTrace.stackToString(ex));
			return (null);
		}
	}

}
