/*
* Copyright (c) Isaac Porat 2011 - 2013 all rights reserved.
* This file is released under the GNU General Public License, version 2, or (at your option) any later version.
* See the file SpeechHub_License.txt for more details.
*/

package sh.connector.sapi5Common;

import java.io.IOException;

import log.common.CascadingLoggerInterface;
import sh.common.SHConfig;
import sh.settings.SHEngineParameters;
import sh.settings.SHEnhancedPropertiesList;
import sh.settings.SHModuleConfiguration;

/**
 * 
 *Read 'Factory set' engine language code properties not intended for user settings
 *Translate language and language country codes from Microsoft to RFC standards
* @author Isaac Porat
 */
public class SHEngineLanguageFactoryProperties
{
	private CascadingLoggerInterface log;
	private SHEngineParameters fEngineParameters;
	private String fEngineIdentifier;
	private SHEnhancedPropertiesList fLanguageCodesPropertiesList;
	private SHEnhancedPropertiesList fLanguageCountryCodesPropertiesList;
	
	public SHEngineLanguageFactoryProperties(String engineIdentifier, CascadingLoggerInterface logger)
	{
		log = logger.clone(SHEngineLanguageFactoryProperties.class);
		fEngineIdentifier = engineIdentifier;
		fEngineParameters = SHEngineParameters.getInstance(logger);
		loadEngineLanguageCodesProperties();
		loadEngineLanguageCountryCodesProperties();
	}

	private void loadEngineLanguageCodesProperties()
	{
		try
		{
			String languageCodesPropertiesFilePath = SHConfig.getEnginesBaseDirectory() + "/engine/"
					+ fEngineParameters.getEngineDirectory(fEngineIdentifier) + "/sh/language-codes.properties";
			SHModuleConfiguration configuration = new SHModuleConfiguration();
			fLanguageCodesPropertiesList = new SHEnhancedPropertiesList();
			configuration.loadProperties(fLanguageCodesPropertiesList, languageCodesPropertiesFilePath);
		} catch (Exception ex)
		{
			log.error("Error in loadEngineLanguageCodesProperties method:\n" + core.utilities.StackTrace.stackToString(ex));
		}
	}

	public String getLanguageCodeProperty(String key) throws IOException
	{
return (fLanguageCodesPropertiesList.get(key));
	}

	private void loadEngineLanguageCountryCodesProperties()
	{
		try
		{
			String languageCountryCodesPropertiesFilePath = SHConfig.getEnginesBaseDirectory() + "/engine/"
					+ fEngineParameters.getEngineDirectory(fEngineIdentifier) + "/sh/language-country-codes.properties";
			SHModuleConfiguration configuration = new SHModuleConfiguration();
			fLanguageCountryCodesPropertiesList = new SHEnhancedPropertiesList();
			configuration.loadProperties(fLanguageCountryCodesPropertiesList, languageCountryCodesPropertiesFilePath);
		} catch (Exception ex)
		{
			log.error("Error in loadEngineLanguageCountryCodesProperties method:\n" + core.utilities.StackTrace.stackToString(ex));
		}
	}

	public String getLanguageCountryCodeProperty(String key) throws IOException
	{
		return(fLanguageCountryCodesPropertiesList.get(key));
	}

}
