/*
* Copyright (c) Isaac Porat 2011 - 2013 all rights reserved.
* This file is released under the GNU General Public License, version 2, or (at your option) any later version.
* See the file SpeechHub_License.txt for more details.
*/

package sh.dev;

import log.common.CascadingLoggerInterface;
import sh.connections.SHEngineInterface;
import sh.connections.SHMessageEnvelopeInterface;
import sh.connections.SHModuleInterface;
import sh.module.SHModuleBase;
import sh.module.SHModuleEngineAdaptor;
import sh.module.SHModuleEventsAdaptor;
import sh.module.SHModuleSpeakAsyncAdaptor;
import sh.module.SHModuleIsolationRestartAdaptor;
import sh.module.SHModuleSpeakSyncAdaptor;
import sh.settings.SHModuleConfiguration;
import sh.settings.SHPropertiesFileLocations;
import sh.settings.SHEnhancedPropertiesList;

/**
 * Server  TTS emulation engine module wrapper 
 * This class is used to wrap all adapttors required in a server module
 * This class can emulate both speakSync and speakAsync based on settings of flags before compiling 
 * @author Isaac Porat
 */
public class SHModuleEmulationServerEngineWrapper extends SHModuleBase implements SHModuleInterface
{
	private SHEngineInterface fEngine;
	@SuppressWarnings("unused")
	private String fEngineIdentifier;

	public SHModuleEmulationServerEngineWrapper(SHEngineInterface engine, String engineIdentifier,
			String settingsBaseDirectory, CascadingLoggerInterface logger)
	{
		fEngine = engine;
		fEngineIdentifier = engineIdentifier;
		log = logger.clone(SHModuleEmulationServerEngineWrapper.class);
		wrap();
		initializeSettings();
	}

	private void initializeSettings()
	{
		try
		{
			SHModuleConfiguration configuration = new SHModuleConfiguration();
			SHPropertiesFileLocations propertiesFileLocations = new SHPropertiesFileLocations(log);
			SHEnhancedPropertiesList propertiesList = new SHEnhancedPropertiesList();
			String[] propertiesFilePathArray = propertiesFileLocations
					.getConfigurationModuleCommonPropertiesFilesPath();
			for (int n = 0; n < propertiesFilePathArray.length; n++)
			{
				configuration.loadProperties(propertiesList, propertiesFilePathArray[n]);
			}
			configuration.transferPropertiesToModule(this, propertiesList);
		} catch (Exception ex)
		{
			log.error("Error in initializeSettings method:\n" + ex);
		}
	}

	private void wrap()
	{
		// Engine wrapper 
		SHModuleInterface moduleAdaptor = new SHModuleEngineAdaptor(fEngine, log);

		boolean speakAsyncRequired = false; // Used for emulation of speakAsync with engine supporting speakSync for development
		if (speakAsyncRequired)
		{
			// speakAsync with basic message events capabilities for engine implementation with speakSync only
			// This class in this location in the chain is only required for simulation with SAPI engine
			moduleAdaptor = new SHModuleSpeakAsyncAdaptor(moduleAdaptor, log);
		}

		// Note - speakSync support requires the engine to provide speakAsync with messageEnd event
		boolean speakSyncRequired = false;
		boolean singleThreadRequired = true;

		if (singleThreadRequired == false)
		{
			if (speakSyncRequired)
			{
				moduleAdaptor = new SHModuleSpeakSyncAdaptor(moduleAdaptor, log);
			}
		} else
		// Single thread
		{
			moduleAdaptor = new SHModuleIsolationRestartAdaptor(moduleAdaptor, speakSyncRequired, log);
		}

		// Above this line speakSync must be provided
		// Events support
		fModule = new SHModuleEventsAdaptor(moduleAdaptor, log);
	}

	@Override
	public boolean speak(SHMessageEnvelopeInterface messageEnvelope)
	{
		try
		{
			return (fModule.speak(messageEnvelope));
		} catch (Exception ex)
		{
			log.error("Error in speak method:\n" + ex);
			return (false);
		}
	}

}
