/*
* Copyright (c) Isaac Porat 2011 - 2013 all rights reserved.
* This file is released under the GNU General Public License, version 2, or (at your option) any later version.
* See the file SpeechHub_License.txt for more details.
*/

package sh.dev.test;

import core.utilities.StackTrace;
import core.utilities.Stopwatch;
import log.common.CascadingLoggerInterface;
import log.common.SimpleLogger;
import sh.connections.SHConstants;
import sh.connections.SHEngineConnectorInterface;
import sh.connections.SHEngineInterface;
import sh.connector.multiNative.SHConnectorMultiNativeAdaptor;
import sh.engine.SHEngineFilterAdaptor;
import sh.independent.SHConfigIndependent;

/**
 * Direct multiNative type engine implementation test
 * Note - To work, this test must include SHConnectorAudioPlayer in the connector stack 
 * (not engaged by default)
* @author Isaac Porat
 */
public class SHMultiEngineTest
{

	public static void main(String[] args)
	{

		/*
		* Log levels 
		* Error - 1
		* Warning - 2
		* Info - 3
		* Debug - 4
		*/
		int logLevel = 2;
		CascadingLoggerInterface log = new SimpleLogger(SHMultiEngineTest.class, logLevel);

		SHEngineInterface engine;
		try
		{
			// Configure locations - settings and  engines
			SHConfigIndependent.configureLocations(log);

			SHEngineConnectorInterface engineConnector = new SHConnectorMultiNativeAdaptor(log);
			engine = new SHEngineFilterAdaptor(engineConnector, log);
		} catch (Exception ex)
		{
			log.error("Error in main method:\n" + StackTrace.stackToString(ex));
			return;
		}

		engine.initialize();
		String[] engineArray = engine.getEngineIdentifiers();
		log.notify("Available engine identifiers:");
		for (int n = 0; n < engineArray.length; n++)
		{
			log.notify(engineArray[n]);
		}

		engine.selectEngine(SHConstants.ENGINE_IDENTIFIER_IBMTTS);

		String[] voiceIdentifiers = engine.getVoiceIdentifiers();
		log.notify("\nVoice Identifiers:\n");
		for (int n = 0; n < voiceIdentifiers.length; n++)
		{
			log.notify(voiceIdentifiers[n]);
		}

		// Variants only applicable for Formant synthesizers 
		String[] variantIdentifiers = engine.getVariantIdentifiers();
		log.notify("\nVariant Identifiers:\n");
		for (int n = 0; n < variantIdentifiers.length; n++)
		{
			log.notify(variantIdentifiers[n]);
		}

		// Initialize a stopwatch and start it

		// Initialize a stopwatch and start it
		Stopwatch stopwatch = new Stopwatch();
		stopwatch.start();

		engine.selectLanguage("en");

		engine.speak(1, "hello", false);
		engine.speak(2, "It works!", true);

		// Stop stopwatch and print result
		stopwatch.stop();
		log.notify(stopwatch.toString());
	}

}
