/*
* Copyright (c) Isaac Porat 2011 - 2013 all rights reserved.
* This file is released under the GNU General Public License, version 2, or (at your option) any later version.
* See the file SpeechHub_License.txt for more details.
*/

package sh.dev.test;

import log.common.CascadingLoggerInterface;
import log.common.SimpleLogger;
import sh.common.SHConfig;
import sh.connections.SHConstants;
import sh.connections.SHTTSInterface;
import sh.independent.SHConfigIndependent;
import sh.independent.SHIndependentModulesController;
import sh.services.SHTTSModulesControllerServiceInterface;
import sh.tts.SHTTS;
import core.utilities.StackTrace;
import core.utilities.Stopwatch;

/**
 * Independent (no server) multi full stack modules TTS test
 * Demoonstrating enumerating engines, selecting a module, getting the module 
 * and calling TTS methods
 *
 *  @author Isaac Porat
 */
public class SHTTSTest
{
	// Ensures controller returns module with full TTS stack 
	private final static boolean SERVER_MODE = false;
	private final static String CLIENT_IDENTIFIER = "DefaultClient";

	public static void main(String[] args)
	{

		/*
		* Log levels 
		* Error - 1
		* Warning - 2
		* Info - 3
		* Debug - 4
		*/
		int logLevel = 2;
		CascadingLoggerInterface log = new SimpleLogger(SHTTSTest.class, logLevel);

		SHTTSModulesControllerServiceInterface modulesController;
		SHTTSInterface tts = null;
		try
		{
			// Configure locations - settings and  engines
			SHConfigIndependent.configureLocations(log);

			modulesController = new SHIndependentModulesController(SERVER_MODE, log);
			// Register the modulesController globally
			SHConfig.setTTSModulesControllerService(modulesController);
			tts = SHTTS.getInstance(CLIENT_IDENTIFIER, log);
		} catch (Exception ex)
		{
			log.error("Error in main method:\n" + StackTrace.stackToString(ex));
			return;
		}

		String[] engineArray = tts.getEngineIdentifiers();
		log.notify("Engines available:");
		for (int n = 0; n < engineArray.length; n++)
		{
			log.notify(engineArray[n]);
		}

		if (tts.selectEngine(SHConstants.ENGINE_IDENTIFIER_ESPEAK) == false)
		{
			log.error("Module selection failed!");
			return;
		}

		// Initialize a stopwatch and start it
		Stopwatch stopwatch = new Stopwatch();

		tts.selectLanguage("en");

		String[] voiceArray = tts.getVoiceIdentifiers();
		log.notify("voices available:");
		for (int n = 0; n < voiceArray.length; n++)
		{
			log.notify(voiceArray[n]);
		}

		// Variants only applicable for Formant synthesizers 
		String[] variantIdentifiers = tts.getVariantIdentifiers();
		log.notify("\nVariant Identifiers:\n");
		for (int n = 0; n < variantIdentifiers.length; n++)
		{
			log.notify(variantIdentifiers[n]);
		}

		/*
		// Simulate closing engine and restarting again
		stopwatch.start();
		tts.close();
						tts.initialize();
						stopwatch.stop();
				log.notify(stopwatch.toString());
		*/
		// Select a voice
		tts.selectVoice("default");

		stopwatch.start();

		// Note - important - to work message identifier must be unique and in descending order

		tts.speak(1, "hello", true);
		tts.speak(2, "It works!", true);
		// Test of abbriviation handling (should be ignored when splitting into sentences)
		//tts.speak(3, "First sentence.  Second with abbriviation etc. sentence", true);
		// Test of two message with '{' to verify char expansion handling
		tts.speak(4, "{", true);
		tts.speak(5, "{", true);
		tts.cancel();

		// Stop stopwatch and print result
		stopwatch.stop();
		log.notify(stopwatch.toString());
	}

}
