/*
* Copyright (c) Isaac Porat 2011 - 2013 all rights reserved.
* This file is released under the GNU General Public License, version 2, or (at your option) any later version.
* See the file SpeechHub_License.txt for more details.
*/

package sh.dev.unused;

import log.common.CascadingLoggerInterface;
import sh.connections.SHConstants;
import sh.connections.SHMessageEnvelopeInterface;
import sh.connections.SHModuleInterface;
import sh.module.SHModuleBase;

/**
 * Implemenst pause and resume functionality
 * Note - not used anymore- replaced with a queue version 
 * Used for standalone TTS implementations
 * In multi client application this functionality is implemented in the multiplexer 
 * @author Isaac Porat
 *
 */
public class SHModulePauseResumeAdaptor extends SHModuleBase
{
	private boolean fMessagePaused = false;
	private SHMessageEnvelopeInterface fLastMessageEnvelope;
	private boolean fModuleHandlesPauseResume = true;

	public SHModulePauseResumeAdaptor(SHModuleInterface module, CascadingLoggerInterface logger)
	{
		log = logger.clone(SHModulePauseResumeAdaptor.class);
		fModule = module;
	}

	@Override
	public boolean speak(SHMessageEnvelopeInterface messageEnvelope)
	{
		// This method must be called with messageSynchronized true	
		return (processSync(messageEnvelope));
	}

	private boolean processSync(SHMessageEnvelopeInterface messageEnvelope)
	{
		synchronized (this)
		{
			try
			{
				while (fMessagePaused)
					wait();
			} catch (InterruptedException e)
			{
			}
			fLastMessageEnvelope = messageEnvelope;
			fModule.speak(messageEnvelope);
			return (true);
		}
	}

	@Override
	public boolean pause()
	{
		if (fModuleHandlesPauseResume == false)
			return (fModule.pause()); // Command to engine 
		if (fModule.pause()) // Required for event notification
		{
			fMessagePaused = true;
			fModule.cancel();
			return (true);
		}
		return (false);
	}

	@Override
	public boolean resume()
	{
		if (fModuleHandlesPauseResume == false)
			return (fModule.resume()); // Command to engine 
		resume(); // Required for event notification
		fModule.cancel(); // Flush synthesizer in case for some reason it is still talking
		if (fLastMessageEnvelope != null)
		{
			fModule.speak(fLastMessageEnvelope);
			fLastMessageEnvelope = null;
		}
		fMessagePaused = false;
		synchronized (this)
		{
			notifyAll();
		}
		return (true);
	}

	@Override
	public boolean setProperty(String key, String value)
	{
		if ((key == null) || (value == null))
			return (false);
		key = key.toLowerCase();
		if (key.equals(SHConstants.PROP_MODULE_HANDLES_PAUSE_RESUME))
			fModuleHandlesPauseResume = value.equalsIgnoreCase("true");
		return (fModule.setProperty(key, value));
	}

}
