/*
* Copyright (c) Isaac Porat 2011 - 2013 all rights reserved.
* This file is released under the GNU General Public License, version 2, or (at your option) any later version.
* See the file SpeechHub_License.txt for more details.
*/

package sh.engine;

import log.common.CascadingLoggerInterface;
import sh.connections.SHConstants;
import sh.connections.SHCustomeHandlersInterface;
import sh.connections.SHEngineConnectorInterface;
import sh.connections.SHEngineInterface;
import sh.connections.SHAudioDataHandlerInterface;
import sh.connections.SHEventsListenerInterface;

/**
 * Simple engine to engineConnector adapter
 * This class has no filtering or checkEngine functionality   
* Parameters get values are based purely on previous successfull select or set values
* There is no logic or any assumptions after a new engine is selected
* Upstream logic must implement any required functionality 
* It is assumed the the engineConnector catch all exceptions
*    
 * @author Isaac Porat
 */
public class SHEngineSimpleAdaptor implements SHEngineInterface
{
	@SuppressWarnings("unused")
	private CascadingLoggerInterface log;
	private SHEngineConnectorInterface fEngine;
	private int fUpdateIdentifier = 0;
	private String fSelectedEngineIdentifier = SHConstants.STRING_PARAMETER_UNDEFINED;
	private String fSetLanguageCode = SHConstants.STRING_PARAMETER_UNDEFINED;
	private String fSelectedVoiceIdentifier = SHConstants.STRING_PARAMETER_UNDEFINED;
	private String fSelectedVariantIdentifier = SHConstants.STRING_PARAMETER_UNDEFINED;
	private float fSetVolume = SHConstants.FLOAT_PARAMETER_UNDEFINED;
	private float fSetRate = SHConstants.FLOAT_PARAMETER_UNDEFINED;
	private float fSetPitch = SHConstants.FLOAT_PARAMETER_UNDEFINED;

	public SHEngineSimpleAdaptor(SHEngineConnectorInterface engine, CascadingLoggerInterface logger)
	{
		log = logger.clone(SHEngineFilterAdaptor.class);
		fEngine = engine;
	}

	@Override
	public boolean initialize()
	{
		return (fEngine.initialize());
	}

	@Override
	public boolean close()
	{
		return (fEngine.close());
	}

	@Override
	public String[] getEngineIdentifiers()
	{
		return (fEngine.getEngineIdentifiers());
	}

	@Override
	public boolean selectEngine(String engineIdentifier)
	{
		boolean status = fEngine.selectEngine(engineIdentifier);
		if (status)
			fSelectedEngineIdentifier = engineIdentifier;
		return (status);
	}

	@Override
	public String getSelectedEngineIdentifier()
	{
		return (fSelectedEngineIdentifier);
	}

	@Override
	public String[] getVoiceIdentifiers()
	{
		return (fEngine.getVoiceIdentifiers());
	}

	@Override
	public boolean selectVoice(String voiceIdentifier)
	{
		boolean status = fEngine.selectVoice(voiceIdentifier);
		if (status)
			fSelectedVoiceIdentifier = voiceIdentifier;
		return (status);
	}

	@Override
	public String getSelectedVoiceIdentifier()
	{
		return (fSelectedVoiceIdentifier);
	}

	@Override
	public String[] getVariantIdentifiers()
	{
		return (fEngine.getVariantIdentifiers());
	}

	@Override
	public boolean selectVariant(String variantIdentifier)
	{
		boolean status = fEngine.selectVariant(variantIdentifier);
		if (status)
			fSelectedVariantIdentifier = variantIdentifier;
		return (status);
	}

	@Override
	public String getSelectedVariantIdentifier()
	{
		return (fSelectedVariantIdentifier);
	}

	@Override
	public boolean speak(int messageIdentifier, String message, boolean speakSync)
	{
		return (fEngine.speak(messageIdentifier, message, speakSync));
	}

	@Override
	public boolean pause()
	{
		return (fEngine.pause());
	}

	@Override
	public boolean resume()
	{
		return (fEngine.resume());
	}

	@Override
	public boolean cancel()
	{
		return (fEngine.cancel());
	}

	@Override
	public String getSelectedLanguageCode()
	{
		return (fSetLanguageCode);
	}

	@Override
	public boolean selectLanguage(String languageCode)
	{
		boolean status = fEngine.selectLanguage(languageCode);
		if (status)
			fSetLanguageCode = languageCode;
		return (status);
	}

	@Override
	public boolean setVolume(float volume)
	{
		boolean status = fEngine.setVolume(volume);
		if (status)
			fSetVolume = volume;
		return (status);
	}

	@Override
	public float getVolume()
	{
		return (fSetVolume);
	}

	@Override
	public boolean setRate(float rate)
	{
		boolean status = fEngine.setRate(rate);
		if (status)
			fSetRate = rate;
		return (status);
	}

	@Override
	public float getRate()
	{
		return (fSetRate);
	}

	@Override
	public boolean setPitch(float pitch)
	{
		boolean status = fEngine.setPitch(pitch);
		if (status)
			fSetPitch = pitch;
		return (status);
	}

	@Override
	public float getPitch()
	{
		return (fSetPitch);
	}

	@Override
	public String getProperty(String key)
	{
		return (fEngine.getProperty(key));
	}

	@Override
	public boolean setProperty(String key, String value)
	{
		return (fEngine.setProperty(key, value));
	}

	@Override
	public boolean setEventsListener(SHEventsListenerInterface eventsListener)
	{
		return (fEngine.setEventsListener(eventsListener));
	}

	@Override
	public boolean setAudioDataHandler(SHAudioDataHandlerInterface audioDataHandler)
	{
		return (fEngine.setAudioDataHandler(audioDataHandler));
	}

	@Override
	public boolean setCustomeHandlers(SHCustomeHandlersInterface customeHandlers)
	{
		return (fEngine.setCustomeHandlers(customeHandlers));
	}

	@Override
	public boolean setUpdateIdentifier(int updateIdentifier)
	{
		fUpdateIdentifier = updateIdentifier;
		return (true);
	}

	@Override
	public boolean isUpdateAccepted(int updateIdentifier)
	{
		if (updateIdentifier == 0) // Insert update
			return (true);
		if (updateIdentifier == -1) // Reset update
		{
			fUpdateIdentifier = 0; // Ensure that after this update the engine updates again with the last before this update identifier
			return (true);
		}
		return (updateIdentifier > fUpdateIdentifier); // Standard update
	}

}
