/*
* Copyright (c) Isaac Porat 2011 - 2013 all rights reserved.
* This file is released under the GNU General Public License, version 2, or (at your option) any later version.
* See the file SpeechHub_License.txt for more details.
*/

package sh.independent;

import java.io.IOException;

import log.common.CascadingLoggerInterface;
import sh.common.SHConfig;
import sh.settings.SHEnhancedPropertiesList;
import sh.settings.SHModuleConfiguration;
import sh.settings.SHSettingsDirectoriesHandler;
import core.utilities.OS;
import core.utilities.OSDir;

/**
 * Handles location configuration for the independent version 
 * without TCF - OSGi support
 * @author Isaac Porat
 *
 */
public class SHConfigIndependent
{
	private static SHEnhancedPropertiesList fPropertiesList;

	private final static String PROP_SETTINGS_PARENT_DIR_R = "settings.parent.dir.r";
	private final static String PROP_SETTINGS_DIR_NAME_R = "settings.dir.name.r";
	private final static String PROP_SETTINGS_PARENT_DIR_RW = "settings.parent.dir.rw";
	private final static String PROP_SETTINGS_DIR_NAME_RW = "settings.dir.name.rw";
	private final static String PROP_ENGINES_PARENT_DIR_R = "engine.parent.dir.r";
	private final static String PROP_ENGINES_DIR_NAME_R = "engine.dir.name.r";
	private final static String PROP_SYSTEM_PARENT_DIR_RW = "system.parent.dir.rw";
	private final static String PROP_SYSTEM_DIR_NAME_RW = "system.dir.name.rw";

	private static void loadConfigProperties() throws IOException
	{
		// Config file always located in current directory
		// Default main installation
		String configPropertiesFilePath = "sh_config_i.properties";
		String installationType = System.getenv("SH_INSTALLATION");
		if ((installationType != null) && (installationType.toLowerCase().equals("portable")))
			configPropertiesFilePath = "sh_config_ip.properties";
		SHModuleConfiguration configuration = new SHModuleConfiguration();
		fPropertiesList = new SHEnhancedPropertiesList();
		configuration.loadProperties(fPropertiesList, configPropertiesFilePath);
	}

	private static String getProperty(String key)
	{
		if (key == null)
			return (null);
		String value = fPropertiesList.get(key);
		if (value == null)
			return (null);
		if (value.equals("none"))
			value = "";
		if (OS.isWindows()) // auto marking Currently works only in windows
		{
			if (value.startsWith("auto"))
			{
				String osSettingsDirectory = OSDir.getSettingsDirectory();
				osSettingsDirectory = osSettingsDirectory.replaceAll("\\\\", "/");
				value = value.replaceAll("^auto", osSettingsDirectory);
			}
		}
		return (value);
	}

	public static void configureLocations(CascadingLoggerInterface logger) throws IOException
	{
		if (fPropertiesList == null)
			loadConfigProperties();
		SHConfig.configureSettingsLocations(getProperty(PROP_SETTINGS_PARENT_DIR_R),
				getProperty(PROP_SETTINGS_DIR_NAME_R));
		SHConfig.configureEnginesLocations(getProperty(PROP_ENGINES_PARENT_DIR_R), getProperty(PROP_ENGINES_DIR_NAME_R));
		String settingsBaseDirectoryRead = SHConfig.getSettingsBaseDirectory();

		SHConfig.configureSettingsLocations(getProperty(PROP_SETTINGS_PARENT_DIR_RW),
				getProperty(PROP_SETTINGS_DIR_NAME_RW));
		String settingsBaseDirectoryReadWrite = SHConfig.getSettingsBaseDirectory();

		SHSettingsDirectoriesHandler.getInstance(logger).verifySettingsDirectories(settingsBaseDirectoryRead,
				settingsBaseDirectoryReadWrite);

		SHConfig.configureSystemLocations(getProperty(PROP_SYSTEM_PARENT_DIR_RW), getProperty(PROP_SYSTEM_DIR_NAME_RW));
	}

}
