/*
* Copyright (c) Isaac Porat 2011 - 2013 all rights reserved.
* This file is released under the GNU General Public License, version 2, or (at your option) any later version.
* See the file SpeechHub_License.txt for more details.
*/

package sh.module;

import log.common.CascadingLoggerInterface;
import sh.connections.SHMessageEnvelopeInterface;
import sh.connections.SHModuleInterface;
import sh.connections.SHEventsListenerInterface;

/*
 * Provides speakAsync method with basic message events
 * Used for Engines with only speakSync implementation
 * This class if included must be placed immediatly after the module engine adaptore   
 * This class can be used   also for speakAsync simulation for engine implementations
 * that provide speakSync and perhaps also speakAsync methods without generating message events
 */
public class SHModuleSpeakAsyncAdaptor extends SHModuleBase implements SHModuleInterface, Runnable
{
	private SHEventsListenerInterface fEventsListener;
	private SHMessageEnvelopeInterface fMessageEnvelope;
	private boolean fMessageArrived = false;

	public SHModuleSpeakAsyncAdaptor(SHModuleInterface module, CascadingLoggerInterface logger)
	{
		fModule = module;
		log = logger.clone(SHModuleSpeakAsyncAdaptor.class);
		Thread messageThread = new Thread(this);
		messageThread.setName(SHModuleSpeakAsyncAdaptor.class.getName());
		messageThread.start();
	}

	@Override
	public boolean setEventsListener(SHEventsListenerInterface eventsListener)
	{
		fEventsListener = eventsListener;
		return (true);
	}

	@Override
	public boolean speak(SHMessageEnvelopeInterface messageEnvelope)
	{
		// This method must be called with messageSynchronized false	
		return (processAsync(messageEnvelope));
	}

	private boolean processAsync(SHMessageEnvelopeInterface messageEnvelope)
	{
		synchronized (this)
		{
			fMessageEnvelope = messageEnvelope;
			fMessageArrived = true;
			notifyAll();
			return (true);
		}
	}

	@Override
	public void run()
	{
		for (;;)
		{
			speakSynchronized();
		}
	}

	private synchronized void speakSynchronized()
	{
		try
		{
			while (fMessageArrived == false)
				wait();
		} catch (InterruptedException e)
		{
		}
		fMessageArrived = false;
		speakSyncWithEvents();
	}

	private void speakSyncWithEvents()
	{
		fMessageEnvelope.setMessageSynchronized(true);
		int messageIdentifier = fMessageEnvelope.getMessageIdentifier();
		String clientIdentifier = fMessageEnvelope.getClientIdentifier();
		if (fEventsListener != null)
			fEventsListener.messageBegin(clientIdentifier, messageIdentifier);
		fModule.speak(fMessageEnvelope);
		if (fEventsListener != null)
			fEventsListener.messageEnd(clientIdentifier, messageIdentifier);
	}

}
