/*
* Copyright (c) Isaac Porat 2011 - 2013 all rights reserved.
* This file is released under the GNU General Public License, version 2, or (at your option) any later version.
* See the file SpeechHub_License.txt for more details.
*/

package sh.module;

import log.common.CascadingLoggerInterface;
import sh.connections.SHMessageEnvelopeInterface;
import sh.connections.SHModuleInterface;
import sh.connections.SHEventsListenerInterface;

/*
 * 
 *Module adaptor providing SpeakSync method 
 *for engine implementations with only speakAsync method with messageEnd event
 */
public class SHModuleSpeakSyncAdaptor extends SHModuleBase
{
	private SHEventsListenerInterface fEvents;
	private boolean fMessageEnded = false;

	public SHModuleSpeakSyncAdaptor(SHModuleInterface module, CascadingLoggerInterface logger)
	{
		log = logger.clone(SHModuleSpeakSyncAdaptor.class);
		fModule = module;
		fEvents = new Events();
		fModule.setEventsListener(fEvents);
	}

	@Override
	public boolean setEventsListener(SHEventsListenerInterface eventsListener)
	{
		// Do nothing, disable this method  
		return (false);
	}

	@Override
	public boolean speak(SHMessageEnvelopeInterface messageEnvelope)
	{
		// This method must be called with messageSynchronized true	
		return (processSync(messageEnvelope));
	}

	private boolean processSync(SHMessageEnvelopeInterface messageEnvelope)
	{
		synchronized (this)
		{
			fMessageEnded = false;
			messageEnvelope.setMessageSynchronized(false);
			fModule.speak(messageEnvelope);
			try
			{
				while (fMessageEnded == false)
					wait();
			} catch (InterruptedException e)
			{
			}
			return (true);
		}
	}

	private synchronized void messageEndSynchronized()
	{
		fMessageEnded = true;
		notifyAll();
	}

	private class Events implements SHEventsListenerInterface
	{

		@Override
		public void messageBegin(String clientIdentifier, int messageIdentifier)
		{
			// Unused - not required	
		}

		@Override
		public void messageEnd(String clientIdentifier, int messageIdentifier)
		{
			messageEndSynchronized();
		}

		@Override
		public void messagePause(String clientIdentifier, int messageIdentifier)
		{
			// Unused - not generated by engine		
		}

		@Override
		public void messageResume(String clientIdentifier, int messageIdentifier)
		{
			// Unused - not generated by engine		
		}

		@Override
		public void messageCancel(String clientIdentifier, int messageIdentifier)
		{
			// Unused - not generated by engine		
		}

		@Override
		public void messageAudioAvailable(String clientIdentifier, int messageIdentifier)
		{
			// Unused not generated by engine
		}

	}

}
