/*
* Copyright (c) Isaac Porat 2011 - 2013 all rights reserved.
* This file is released under the GNU General Public License, version 2, or (at your option) any later version.
* See the file SpeechHub_License.txt for more details.
*/

package sh.module;

import java.util.Calendar;
import java.util.Date;

import sh.connections.SHModuleTimestampInterface;

/**
 * Timing support for module operations
 * @author Isaac Porat
 *
 */
public class SHModuleTimestamp implements SHModuleTimestampInterface
{
	private Date fCancelTimestamp;

	public SHModuleTimestamp()
	{
	}

	public Date getTimestamp()
	{
		return (Calendar.getInstance().getTime());
	}

	public void setCancelTimestampJustAfter(Date timestamp)
	{
		long timeJustBefore = timestamp.getTime() + 1;
		Calendar calendar = Calendar.getInstance();
		calendar.setTimeInMillis(timeJustBefore);
		fCancelTimestamp = calendar.getTime();
	}

	public void setCancelTimestampToNow()
	{
		fCancelTimestamp = getTimestamp();
	}

	public boolean isMessageCancelled(Date timestamp, boolean reenable)
	{
		if (fCancelTimestamp == null)
			return (false);
		// Note, cancel can appear immediatly on the same timestamp of the speak message hence if timestamps the same assume cancel came before the message
		if ((timestamp.equals(fCancelTimestamp)) || (timestamp.after(fCancelTimestamp)))
		{
			if (reenable)
				fCancelTimestamp = null;
			return (false);
		}
		return (true);
	}

}
