/*
* Copyright (c) Isaac Porat 2011 - 2013 all rights reserved.
* This file is released under the GNU General Public License, version 2, or (at your option) any later version.
* See the file SpeechHub_License.txt for more details.
*/

package sh.server;

import log.common.CascadingLoggerInterface;
import sh.common.SHConfig;
import sh.connections.SHConstants;
import sh.server.transport.SHMainServerMultiSocketInet;
import sh.server.transport.SHMainServerMultiSocketUnix;
import sh.server.transport.SHMainServerSingleSocketInet;
import sh.server.transport.SHMainServerStdio;
import sh.services.SHTTSModulesControllerServiceInterface;
import sh.settings.SHEnhancedPropertiesList;
import sh.settings.SHPropertiesFileLocations;

public class SHServerLauncher
{
	private CascadingLoggerInterface log;

	public SHServerLauncher(String transportType, int singleClientServerSocketInetPort,
			SHTTSModulesControllerServiceInterface ttsModulesControllerService, CascadingLoggerInterface logger)
			throws Exception
	{
		log = logger.clone(SHServerLauncher.class);
		SHConfig.setTTSModulesControllerService(ttsModulesControllerService);
		SHPropertiesFileLocations propertiesFileLocations = new SHPropertiesFileLocations(log);
		String filePath = propertiesFileLocations.getConfigurationServerPropertiesFilesPath();
		SHEnhancedPropertiesList properties = new SHEnhancedPropertiesList(filePath);
		SHConfig.setServerProperties(properties);
		SHServerAutoShutdown serverAutoShutdown = SHServerAutoShutdown.getInstance(log);
		if (transportType.equals(SHConstants.SERVER_TRANSPORT_MULTI_SOCKET_INET))
		{
			new SHMainServerMultiSocketInet(log);
			serverAutoShutdown.startTimer();
		} else if (transportType.equals(SHConstants.SERVER_TRANSPORT_MULTI_SOCKET_UNIX))
		{
			new SHMainServerMultiSocketUnix(log);
			serverAutoShutdown.startTimer();
		} else if (transportType.equals(SHConstants.SERVER_TRANSPORT_SINGLE_SOCKET_INET))
			new SHMainServerSingleSocketInet(singleClientServerSocketInetPort, log);
		else if (transportType.equals(SHConstants.SERVER_TRANSPORT_STDIO))
			new SHMainServerStdio(log);
	}
}
