/*
* Copyright (c) Isaac Porat 2011 - 2013 all rights reserved.
* This file is released under the GNU General Public License, version 2, or (at your option) any later version.
* See the file SpeechHub_License.txt for more details.
*/

package sh.server.transport;

import hrpc.common.HRHandlerInterface;
import hrpc.common.HRHandlerFactoryInterface;
import hrpc.transport.HRServerMultiSocketInet;

import java.io.IOException;

import log.common.CascadingLoggerInterface;

import sh.common.SHConfig;
import sh.connections.SHConstants;
import sh.server.essip.SHExtendedSSIPHandler;

public class SHMainServerMultiSocketInet
{
	private CascadingLoggerInterface log;
	private int fServerPort = 6564;
	private HRHandlerFactoryInterface fHandlerFactory;
	private boolean fDebugMode = false;

	public SHMainServerMultiSocketInet(CascadingLoggerInterface logger) throws IOException
	{
		log = logger.clone(SHMainServerMultiSocketInet.class);
		String value = SHConfig.getServerProperty(SHConstants.PROP_SERVER_SOCKET_INET_PORT);
		try
		{
			fServerPort = Integer.parseInt(value);
		} catch (NumberFormatException ex)
		{
			throw new NumberFormatException("Server prot number not specify correctly in server.config.txt file");
		}
		value = SHConfig.getServerProperty(SHConstants.PROP_SERVER_DEBUG_MODE);
		fDebugMode = ((value != null) && (value.equalsIgnoreCase("true")));
		fHandlerFactory = new HandlerFactory();
		new HRServerMultiSocketInet(fServerPort, 0, fHandlerFactory, fDebugMode, log);
	}

	private class HandlerFactory implements HRHandlerFactoryInterface
	{

		public HRHandlerInterface getNewInstance()
		{
			return (new SHExtendedSSIPHandler(fDebugMode, log));
		}

	}

}
