/*
* Copyright (c) Isaac Porat 2011 - 2013 all rights reserved.
* This file is released under the GNU General Public License, version 2, or (at your option) any later version.
* See the file SpeechHub_License.txt for more details.
*/

package sh.server.transport;

import hrpc.common.HRHandlerInterface;
import hrpc.common.HRHandlerFactoryInterface;
import hrpc.transport.HRServerMultiSocketUnix;

import java.io.IOException;

import log.common.CascadingLoggerInterface;

import sh.common.SHConfig;
import sh.connections.SHConstants;
import sh.server.essip.SHExtendedSSIPHandler;

public class SHMainServerMultiSocketUnix
{
	private CascadingLoggerInterface log;
	private String fSocketPath;
	private HRHandlerFactoryInterface fHandlerFactory;
	private boolean fDebugMode = false;

	public SHMainServerMultiSocketUnix(CascadingLoggerInterface logger) throws IOException
	{
		log = logger.clone(SHMainServerMultiSocketUnix.class);
		fSocketPath = SHConfig.getServerProperty(SHConstants.PROP_SERVER_SOCKET_UNIX_PATH);
		if (fSocketPath == null)
			throw new IOException("Cannot obtain Unix socket path from configuration file");
		String value = SHConfig.getServerProperty(SHConstants.PROP_SERVER_DEBUG_MODE);
		fDebugMode = ((value != null) && (value.equalsIgnoreCase("true")));
		fHandlerFactory = new HandlerFactory();
		new HRServerMultiSocketUnix(fSocketPath, 0, fHandlerFactory, fDebugMode, log);
	}

	private class HandlerFactory implements HRHandlerFactoryInterface
	{

		public HRHandlerInterface getNewInstance()
		{
			return (new SHExtendedSSIPHandler(fDebugMode, log));
		}

	}

}
