/*
* Copyright (c) Isaac Porat 2011 - 2013 all rights reserved.
* This file is released under the GNU General Public License, version 2, or (at your option) any later version.
* See the file SpeechHub_License.txt for more details.
*/

package sh.services;

import sh.connections.SHEngineInterface;
import sh.connections.SHModuleInterface;

/**
 * TTS module service interface
 * Handles TTS modules which wrapp TTS engines
 * There are different types of modules depending on use
 * Server module for use in SpeechHub server
 * Full stack modules for applications that do not use a server
 * Module adaptor where a module is not required at all (connected directly to the TTS engine)
 *  
 *  @author Isaac Porat
 *
 */
public interface SHTTSModuleServiceInterface
{

	/**
	 * 
	 * @return Engine connector identifier
	 * Note it is the identifier of the connector itself
	 * and not of individual engines
	 * (some connectors support multiple engines)
	 */
	public String getTTSEngineConnectorIdentifier();

	/**
	 * 
	 * @return true if engines conforming to the required comm protocol are available
	 * Available means stored under the 'engines' directory and are not disabled (directory name prefixed by underline '_')
	 */
	public boolean isAvailable();

	/**
	 * 
	 * @return engine (without a module) 
	 * that is SHConnectorInterface wrapped in SHEngineInterface 
	 */
	public SHEngineInterface getTTSEngine();

	/**
	 * Closes the engine and destroy all modules
	 */
	public void closeTTS();

	/**
	 * Sets an engine wrapped in the appropriate module
	 * @param moduleType Module type to set
	 * @param module 
	 */
	public void setTTSModule(String moduleType, SHModuleInterface module);

	/**
	 * 
	 * @param moduleType The module type to  get
	 * @return module requested
	 */
	public SHModuleInterface getTTSModule(String moduleType);

	public String getProperty(String key);

	public String getProperty(String key, String defaultValue);

}
