/*
* Copyright (c) Isaac Porat 2011 - 2013 all rights reserved.
* This file is released under the GNU General Public License, version 2, or (at your option) any later version.
* See the file SpeechHub_License.txt for more details.
*/

package sh.settings;


import log.common.CascadingLoggerInterface;

/**
 * Reads engine common properties
 * These properties can be set by the user
 * @author Isaac Porat
 *
 */
public class SHEngineCommonProperties
{
	private CascadingLoggerInterface log;
	private SHEnhancedPropertiesList fPropertiesList;
	
	public SHEngineCommonProperties(CascadingLoggerInterface logger)
	{
		log = logger.clone(SHEngineCommonProperties.class);
	}

	private void loadEngineProperties()
	{
		try
		{
			SHModuleConfiguration configuration = new SHModuleConfiguration();
			SHPropertiesFileLocations propertiesFileLocations = new SHPropertiesFileLocations(log);
			fPropertiesList = new SHEnhancedPropertiesList();
			String[] propertiesFilePathArray = propertiesFileLocations
					.getConfigurationEngineCommonPropertiesFilesPath();
			for (int n = 0; n < propertiesFilePathArray.length; n++)
			{
				configuration.loadProperties(fPropertiesList, propertiesFilePathArray[n]);
			}
		} catch (Exception ex)
		{
			log.error("Error in loadEngineProperties method:\n" + core.utilities.StackTrace.stackToString(ex));
		}
	}

	public String getProperty(String key)
	{
		if (fPropertiesList == null)
			loadEngineProperties();
		return (fPropertiesList.get(key));
	}

}
