/*
* Copyright (c) Isaac Porat 2011 - 2013 all rights reserved.
* This file is released under the GNU General Public License, version 2, or (at your option) any later version.
* See the file SpeechHub_License.txt for more details.
*/

package sh.settings;

import java.io.IOException;

import log.common.CascadingLoggerInterface;
import sh.common.SHConfig;

/**
 * 
 *Read 'Factory set' engine properties not intended for user settings
* @author Isaac Porat
 */
public class SHEngineFactoryProperties
{
	private CascadingLoggerInterface log;
	private SHEngineParameters fEngineParameters;
	private SHEnhancedPropertiesList fPropertiesList;
	private String fPreviousPropertiesEngineIdentifier = "";

	public SHEngineFactoryProperties(CascadingLoggerInterface logger)
	{
		log = logger.clone(SHEngineFactoryProperties.class);
		fEngineParameters = SHEngineParameters.getInstance(logger);
	}

	private void loadEngineProperties(String engineIdentifier)
	{
		try
		{
			String enginePropertiesFilePath = SHConfig.getEnginesBaseDirectory() + "/engine/"
					+ fEngineParameters.getEngineDirectory(engineIdentifier) + "/sh/engine.properties";
			SHModuleConfiguration configuration = new SHModuleConfiguration();
			fPropertiesList = new SHEnhancedPropertiesList();
			configuration.loadProperties(fPropertiesList, enginePropertiesFilePath);
		} catch (Exception ex)
		{
			log.error("Error in loadEngineProperties method:\n" + core.utilities.StackTrace.stackToString(ex));
		}
	}

	public String getProperty(String engineIdentifier, String key) throws IOException
	{
		if (fPreviousPropertiesEngineIdentifier.equals(engineIdentifier) == false)
		{
			loadEngineProperties(engineIdentifier);
			fPreviousPropertiesEngineIdentifier = engineIdentifier;
		}
		return (fPropertiesList.get(key));
	}

}
