/*
* Copyright (c) Isaac Porat 2011 - 2013 all rights reserved.
* This file is released under the GNU General Public License, version 2, or (at your option) any later version.
* See the file SpeechHub_License.txt for more details.
*/

package sh.settings;

import java.io.IOException;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import sh.connections.SHItemReplacementInterface;

public class SHReplacement
{

	public SHReplacement()
	{
	}

	public void loadProperties(SHEnhancedPropertiesList masterPropertiesList, String filePath) throws IOException
	{
		SHEnhancedPropertiesList propertiesList = new SHEnhancedPropertiesList(filePath);
		String[] keysArray = propertiesList.toKeysArray();
		for (int n = 0; n < keysArray.length; n++)
		{
			String key = keysArray[n];
			String value = propertiesList.get(keysArray[n]);
			masterPropertiesList.put(key, value);
		}
	}

	public SHItemReplacementInterface[] makeReplacementArray(SHEnhancedPropertiesList masterPropertiesList)
	{
		String[] keysArray = masterPropertiesList.toKeysArray();
		SHItemReplacementInterface[] replacementArray = new Replacement[keysArray.length];
		for (int n = 0; n < keysArray.length; n++)
		{
			String patternString = keysArray[n];
			String replacement = masterPropertiesList.get(patternString);
			Pattern pattern = Pattern.compile(patternString);
			replacementArray[n] = new Replacement(patternString, pattern, replacement);
		}
		return (replacementArray);
	}

	public String replaceAll(SHItemReplacementInterface[] replacementArray, String message)
	{
		for (int n = 0; n < replacementArray.length; n++)
		{
			//message = 		message.replaceAll(replacementArray[n].getPatternString(), replacementArray[n].getReplacement());

			Matcher matcher = replacementArray[n].getPattern().matcher(message);
			message = matcher.replaceAll(replacementArray[n].getReplacement());

		}
		return (message);
	}

	private class Replacement implements SHItemReplacementInterface
	{
		private String fPatternString;
		private Pattern fPattern;
		private String fReplacement;

		public Replacement(String patternString, Pattern pattern, String replacement)
		{
			fPatternString = patternString;
			fPattern = pattern;
			fReplacement = replacement;
		}

		public String getPatternString()
		{
			return (fPatternString);
		}

		public Pattern getPattern()
		{
			return (fPattern);
		}

		public String getReplacement()
		{
			return (fReplacement);
		}

	}

}
