/*
* Copyright (c) Isaac Porat 2011 - 2013 all rights reserved.
* This file is released under the GNU General Public License, version 2, or (at your option) any later version.
* See the file SpeechHub_License.txt for more details.
*/

package sh.tts;

import log.common.CascadingLoggerInterface;
import sh.connections.SHCustomeHandlersInterface;
import sh.connections.SHEngineInterface;
import sh.connections.SHMessageEnvelopeInterface;
import sh.connections.SHModuleInterface;
import sh.connections.SHModuleTimestampInterface;
import sh.connections.SHAudioDataHandlerInterface;
import sh.connections.SHEventsListenerInterface;
import sh.connections.SHTTSInterface;
import sh.connections.SHUpdateParametersInterface;
import core.utilities.StackTrace;

/**
 * TTS adaptor template class
 * For development, modify as required
 * @author Isaac Porat
 *
 */
public class SHTTSAdaptorTemplate implements SHTTSInterface, SHModuleInterface, SHEngineInterface
{
	private CascadingLoggerInterface log;
	private SHTTSInterface fTTS;

	public SHTTSAdaptorTemplate(SHTTSInterface tts, CascadingLoggerInterface logger)
	{
		log = logger.clone(SHTTSAdaptorTemplate.class);
		fTTS = tts;
	}

	@Override
	public boolean configureTiming(SHModuleTimestampInterface moduleTimestamp, boolean masterCancel)
	{
		// Not applicable
		return (false);
	}

	@Override
	public boolean initialize()
	{
		return (fTTS.initialize());
	}

	@Override
	public boolean close()
	{
		return (fTTS.close());
	}

	@Override
	public boolean setPriority(int priorityLevel, boolean selfCancel)
	{
		// Not applicable
		return (false);
	}

	@Override
	public String[] getEngineIdentifiers()
	{
		try
		{
			return (fTTS.getEngineIdentifiers());
		} catch (Exception ex)
		{
			log.error("Error in getEngineIdentifiers method:\n" + StackTrace.stackToString(ex));
			return (null);
		}
	}

	@Override
	public boolean selectEngine(String engineIdentifier)
	{
		try
		{
			return (fTTS.selectEngine(engineIdentifier));
		} catch (Exception ex)
		{
			log.error("Error in selectEngine method:\n" + StackTrace.stackToString(ex));
			return (false);
		}
	}

	@Override
	public String getSelectedEngineIdentifier()
	{
		return (fTTS.getSelectedEngineIdentifier());
	}

	@Override
	public boolean selectLanguage(String languageCode)
	{
		return (fTTS.selectLanguage(languageCode));
	}

	@Override
	public String getSelectedLanguageCode()
	{
		return (fTTS.getSelectedLanguageCode());
	}

	@Override
	public String[] getVoiceIdentifiers()
	{
		return (fTTS.getVoiceIdentifiers());
	}

	@Override
	public boolean selectVoice(String voiceIdentifier)
	{
		return (fTTS.selectVoice(voiceIdentifier));
	}

	@Override
	public String getSelectedVoiceIdentifier()
	{
		return (fTTS.getSelectedVoiceIdentifier());
	}

	@Override
	public String[] getVariantIdentifiers()
	{
		return (fTTS.getVariantIdentifiers());
	}

	@Override
	public boolean selectVariant(String variantIdentifier)
	{
		return (fTTS.selectVariant(variantIdentifier));
	}

	@Override
	public String getSelectedVariantIdentifier()
	{
		return (fTTS.getSelectedVariantIdentifier());
	}

	@Override
	public boolean speak(SHMessageEnvelopeInterface messageEnvelope)
	{
		// Not applicable
		return (false);
	}

	@Override
	public boolean speak(int messageIdentifier, String message, boolean speakSync)
	{
		return (fTTS.speak(messageIdentifier, message, speakSync));
	}

	@Override
	public boolean speakChar(int messageIdentifier, String message, boolean speakSync)
	{
		return (fTTS.speakChar(messageIdentifier, message, speakSync));
	}

	@Override
	public boolean speakKey(int messageIdentifier, String message, boolean speakSync)
	{
		return (fTTS.speakKey(messageIdentifier, message, speakSync));
	}

	@Override
	public boolean speakSpell(int messageIdentifier, String message, boolean speakSync)
	{
		return (fTTS.speakSpell(messageIdentifier, message, speakSync));
	}

	@Override
	public boolean pause()
	{
		return (fTTS.pause());
	}

	@Override
	public boolean resume()
	{
		return (fTTS.resume());
	}

	@Override
	public boolean stop()
	{
		return (fTTS.stop());
	}

	@Override
	public boolean cancel()
	{
		return (fTTS.cancel());
	}

	@Override
	public boolean setVolume(float volume)
	{
		return (fTTS.setVolume(volume));
	}

	@Override
	public float getVolume()
	{
		return (fTTS.getVolume());
	}

	@Override
	public boolean setRate(float rate)
	{
		return (fTTS.setRate(rate));
	}

	@Override
	public float getRate()
	{
		return (fTTS.getRate());
	}

	@Override
	public boolean setRateBoostMode(boolean mode)
	{
		return (fTTS.setRateBoostMode(mode));
	}

	@Override
	public boolean setPitch(float pitch)
	{
		return (fTTS.setPitch(pitch));
	}

	@Override
	public float getPitch()
	{
		return (fTTS.getPitch());
	}

	@Override
	public boolean setPunctuationLevel(String level)
	{
		return (fTTS.setPunctuationLevel(level));
	}

	@Override
	public boolean setCharCapitalIndicator(String charCapitalIndicator, String arg)
	{
		return (fTTS.setCharCapitalIndicator(charCapitalIndicator, arg));
	}

	@Override
	public boolean setSilenceBeforeMessage(int silenceMilliseconds)
	{
		return (fTTS.setSilenceBeforeMessage(silenceMilliseconds));
	}

	@Override
	public boolean setProperty(String key, String value)
	{
		return (fTTS.setProperty(key, value));
	}

	@Override
	public String getProperty(String key)
	{
		return (fTTS.getProperty(key));
	}

	@Override
	public boolean setEventsListener(SHEventsListenerInterface eventsListener)
	{
		return (fTTS.setEventsListener(eventsListener));
	}

	@Override
	public boolean setAudioDataHandler(SHAudioDataHandlerInterface audioDataHandler)
	{
		return (fTTS.setAudioDataHandler(audioDataHandler));
	}

	@Override
	public boolean setCustomeHandlers(SHCustomeHandlersInterface customeHandlers)
	{
		// Not applicable
		return (false);
	}

	@Override
	public boolean updateParametersDirectly(SHUpdateParametersInterface updateParameters)
	{
		// Not applicable
		return (false);
	}

	@Override
	public boolean setUpdateIdentifier(int updateIdentifier)
	{
		// Not applicable
		return (false);
	}

	@Override
	public boolean isUpdateAccepted(int updateIdentifier)
	{
		// Not applicable
		return (false);
	}

}
