/***************************** License **********************************

    Copyright(c)2005, Communicative Machines
    All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are 
met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright 
  notice, this list of conditions and the following disclaimer in the 
  documentation and/or other materials provided with the distribution.

- Neither the name of the copyright holders nor the names of its 
  contributors may be used to endorse or promote products derived from 
  this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

************************************************************************/
// BaseObject.java


package com.cmlabs.air;

import java.util.*;
import org.w3c.dom.*;

public class BaseObject implements AIRBase {

  public BaseObject() {
  }

  public Object clone() {
    return null;
  }

  public String toXML() {
    return "";
  }

  public int getBinarySize(int chunk) {
    return 0;
  }

  public int getBinaryChunkCount() {
    return 0;
  }

  public int toBinaryBuffer(int chunk, byte[] data, int offset, int maxlen) {
    return 0;
  }

  public boolean fromBinaryBuffer(int chunk, byte[] data, int offset, int len) {
    return false;
  }









  public static boolean isBaseObject(Object obj) {
    if (obj == null)
      return false;
    return isClassBaseObject(obj.getClass());
  }

  public static boolean isClassBaseObject(Class cls) {
    if (cls == null)
      return false;
    Class[] interfaces = cls.getInterfaces();
    Class interf;
    for (int n=0; n<interfaces.length; n++) {
      interf = interfaces[n];
      if ( (interf != null) &&
	   ( (interf.getName().equalsIgnoreCase("AIRBase")) || (interf.getName().equalsIgnoreCase("com.cmlabs.air.AIRBase")) ) )
	return true;
    }
    return isClassBaseObject(cls.getSuperclass());
  }

  public static BaseObject createFromXML(String xml) {
    return createFromXML(Utils.xmlGetRootNodeFromXML(xml));
  }

  public static BaseObject createFromXML(XMLElement node) {

    if (node == null)
      return null;

    String nodename = node.getName();
    if (nodename == null)
      return null;
    else if (nodename.equalsIgnoreCase("message")) {
      return new Message(node);
    }
    else if (node.hasAttribute("sec")) {
      return new Time(node);
    }
    else if (nodename.equalsIgnoreCase("tcplocation")) {
      return new TCPLocation(node);
    }
    else if (nodename.equalsIgnoreCase("collection")) {
      TextCollection col = new TextCollection();
      col.fromXML(node);
      return col;
    }
    else if (nodename.equalsIgnoreCase("objectcollection")) {
      ObjectCollection col = new ObjectCollection();
      col.fromXML(node);
      return col;
    }
    else if (nodename.equalsIgnoreCase("dictionary")) {
      TextDictionary d = new TextDictionary();
      d.fromXML(node);
      return d;
    }
    else if (nodename.equalsIgnoreCase("objectdictionary")) {
      ObjectDictionary d = new ObjectDictionary();
      d.fromXML(node);
      return d;
    }
    else if (nodename.equalsIgnoreCase("datasample")) {
      return new DataSample();
    }
    else if (nodename.equalsIgnoreCase("bitmap")) {
      return new DataSample();
    }
    else if (nodename.equalsIgnoreCase("bitmapupdate")) {
      return new DataSample();
    }
    else if (nodename.equalsIgnoreCase("perfstat")) {
      return new PerfStat(node);
    }
    else if (nodename.equalsIgnoreCase("infoitem")) {
      return new InfoItem(node);
    }
    else if (nodename.equalsIgnoreCase("parameter")) {
      return new Parameter(node);
    }
    else if (nodename.equalsIgnoreCase("connectionprofile")) {
      return new ConnectionProfile(node);
    }
    else if (nodename.equalsIgnoreCase("reference")) {
      return new Reference(node);
    }

    else if (nodename.equalsIgnoreCase("phase")) {
      return new PhaseSpec(node);
    }
    else if (nodename.equalsIgnoreCase("trigger")) {
      return new TriggerSpec(node, "");
    }
    else if (nodename.equalsIgnoreCase("retrieve")) {
      return new RetrieveSpec(node, "");
    }
    else if (nodename.equalsIgnoreCase("crank")) {
      return new CrankSpec(node);
    }
    else if (nodename.equalsIgnoreCase("post")) {
      return new PostSpec(node, "");
    }




    return (new RawXML(node.toString()));
  }


}
