/***************************** License **********************************

    Copyright(c)2005, Communicative Machines
    All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are 
met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright 
  notice, this list of conditions and the following disclaimer in the 
  documentation and/or other materials provided with the distribution.

- Neither the name of the copyright holders nor the names of its 
  contributors may be used to endorse or promote products derived from 
  this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

************************************************************************/
// CrankSpec.java


package com.cmlabs.air;

import java.util.*;

public class CrankSpec extends BaseObject {

  public boolean continuous = false;
  public String name = "";
  public TextDictionary mediaStreams = new TextDictionary();

  public CrankSpec() {
  }

  public CrankSpec(String xml) {
    fromXML(Utils.xmlGetRootNodeFromXML(xml));
  }

  public CrankSpec(XMLElement node) {
    fromXML(node);
  }

  public boolean fromXML(String xml) {
    return fromXML(Utils.xmlGetRootNodeFromXML(xml));
  }

  public boolean fromXML(XMLElement node) {

    if (node == null)
      return false;

    String nodename = node.getName();
    if ( (nodename == null) || (!nodename.equalsIgnoreCase("crank")))
      return false;

    name = node.getStringAttribute("name");

    String streamName, source;
    String key;
    String value;
    XMLElement child;
    XMLElement sub;
    for (Enumeration e = node.enumerateChildren(); e.hasMoreElements(); ) {
      child = (XMLElement) e.nextElement();
      if (child != null) {
        key = child.getName();
        value = child.getContent();
        if (key.equalsIgnoreCase("stream")) {
          streamName = child.getStringAttribute("name");
          source = child.getStringAttribute("source");
          if ( (streamName != null) && (streamName.length() > 0) && (source != null) && (source.length() > 0) )
            mediaStreams.put(streamName, source);
        }
        else if (key.equalsIgnoreCase("mediaconnection")) {
          streamName = child.getStringAttribute("alias");
          source = child.getStringAttribute("source");
          if ( (streamName != null) && (streamName.length() > 0) && (source != null) && (source.length() > 0) )
            mediaStreams.put(streamName, source);
        }
        else if (key.equalsIgnoreCase("mediastream")) {
          streamName = child.getStringAttribute("name");
          source = child.getStringAttribute("source");
          if ( (streamName != null) && (streamName.length() > 0) && (source != null) && (source.length() > 0) )
            mediaStreams.put(streamName, source);
        }
      }
    }

    return true;
  }

  public String toXML() {

    String xml = "";

    String mediaName, source;
    for (int m=0; m<mediaStreams.getCount(); m++) {
      mediaName = mediaStreams.getKey(m);
      source = mediaStreams.get(mediaName);
      if ( (mediaName.length() > 0) && (source.length() > 0))
        xml += "<stream name=\"" + Utils.xmlStringEncode(mediaName) +
            "\" source=\"" + Utils.xmlStringEncode(source) + "\" />\n";
    }

    if (xml.length() > 0)
      return "<crank name=\"" + Utils.xmlStringEncode(name) + "\">\n" + Utils.xmlIndent(xml) + "</crank>";
    else
      return "<crank name=\"" + Utils.xmlStringEncode(name) + "\" />";
  }

  public String print() {
    return "crank name=\"" + name + "\"";
  }

  public String toHTML() {
    return print();
  }

}
