/***************************** License **********************************

    Copyright(c)2005, Communicative Machines
    All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are 
met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright 
  notice, this list of conditions and the following disclaimer in the 
  documentation and/or other materials provided with the distribution.

- Neither the name of the copyright holders nor the names of its 
  contributors may be used to endorse or promote products derived from 
  this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

************************************************************************/
// DataSample.java


package com.cmlabs.air;

import java.util.*;
import org.w3c.dom.*;

public class DataSample extends BaseObject {

  public TextDictionary params = new TextDictionary();
  public String id = "";
  public String name = "";
  public String type = "";
  public Time timestamp = new Time();
  public byte[] data = null;
  public int size = 0;
  public int duration = 0;
  public int width = 0;
  public int height = 0;
  public BaseObject object = null;
  public BaseObject aux = null;

  public static final int DATACTRL = 1;
  public static final int RLECTRL = 2;
  public static final int DIFCTRL = 3;

  public DataSample() {
    id = Utils.generateID();
  }

  public Object clone() {
    return null;
  }

  public String toString() {
    return "DataSample [" + timestamp.print24HourTimeString() + "] " + size + " bytes ";
  }

  public String toXML() {
    return "<datasample />";
  }

  public boolean fromXML(XMLElement node) {
    if (node == null)
      return false;
    if (!node.getName().equalsIgnoreCase("datasample"))
      return false;
    return true;
  }

  public int getBinarySize(int chunk) {
    if (chunk > 0)
      return 0;

    params.put("builtin:id", id);
    params.put("builtin:type", type);
    params.put("builtin:duration", ""+duration);
    params.put("builtin:timestamp", timestamp.asText());
    params.put("builtin:name", name);
    params.put("builtin:width", ""+width);
    params.put("builtin:height", ""+height);
    String extra = params.printListLine("%sep%", "%equals%");

    if (object == null)
      return (int)(4 + size + extra.length()+1);
    else
      return (4 + object.toXML().length()+1 + extra.length()+1);
  }

  public int getBinaryChunkCount() {
    return 1;
  }

  public int toBinaryBuffer(int chunk, byte[] buffer, int offset, int maxlen) {
    if (chunk > 0)
      return 0;

    if (object != null)
      type = "Object";

    params.put("builtin:id", id);
    params.put("builtin:type", type);
    params.put("builtin:duration", ""+duration);
    params.put("builtin:timestamp", timestamp.asText());
    params.put("builtin:name", name);
    params.put("builtin:width", ""+width);
    params.put("builtin:height", ""+height);

    String extra = params.printListLine("%sep%", "%equals%");

    String xml = "";
    if (object != null) {
      xml = object.toXML();
      size = xml.length() + 1;
    }
    else {
    }

    int len = (int) (4 + size + extra.length()+1);
    if (len > maxlen)
      return 0;

    Utils.writeIntToBytes(buffer, offset, size);

    if (xml.length() > 0) {
      Utils.writeStringToBytes(buffer, offset+4, xml);
    }
    else if ((data != null) && (size > 0)) {
      System.arraycopy(data, 0, buffer, offset+4, size);
    }
    Utils.writeStringToBytes(buffer, offset+4+size, extra);
    buffer[buffer.length-1] = 0;

    if (object != null)
      size = 0;

    return len;
  }

  public boolean fromBinaryBuffer(int chunk, byte[] buffer, int offset, int len) {
    if (chunk > 0)
      return false;

    if ((buffer == null) || (len <= 0))
      return false;

    int binsize = Utils.readIntFromBytes(buffer, 0);

    String extra = Utils.readStringFromBytes(buffer, 4+binsize, buffer.length-4-binsize-1);
    if (extra.length() == 0)
      return false;
    params = Utils.splitString(extra, "%sep%", "%equals%");
    if (params.getCount() < 3)
      return false;

    int dur = Utils.str2int(params.get("builtin:duration"));
    String t = params.get("builtin:timestamp");
    timestamp = new Time(Utils.str2long(t));
    id = params.get("builtin:id");
    type = params.get("builtin:type");
    name = params.get("builtin:name");
    width = Utils.str2int(params.get("builtin:width"));
    height = Utils.str2int(params.get("builtin:height"));
    params.remove("builtin:duration");
    params.remove("builtin:timestamp");
    params.remove("builtin:name");
    params.remove("builtin:type");
    params.remove("builtin:width");
    params.remove("builtin:height");

    if (binsize > 0) {
      if (type.equalsIgnoreCase("Object")) {
	String xml = Utils.readStringFromBytes(buffer, 4, binsize-1);
	duration = dur;
	object = createFromXML(xml);
	return (object != null);
      }
      else {
	size = binsize;
	data = new byte[size];
	System.arraycopy(buffer, 4, data, 0, size);
	return true;
      }
    }
    else {
      return true;
    }
  }

}
