/***************************** License **********************************

    Copyright(c)2005, Communicative Machines
    All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are 
met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright 
  notice, this list of conditions and the following disclaimer in the 
  documentation and/or other materials provided with the distribution.

- Neither the name of the copyright holders nor the names of its 
  contributors may be used to endorse or promote products derived from 
  this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

************************************************************************/
//  IncomingSocketConnector.java


package com.cmlabs.air;

import java.net.*;
import java.util.*;

/**
* This class runs on its own thread, and waits for a socket connection to come in.
* then, it creates NetworkConnection of type INCOMING, which allows one
* message to be sent back and forth, and then ceases to exist.
* This object then sits and waits, again.
*/

// basically, this is a little server.
public class IncomingSocketConnector extends Thread {

    int port;
    NetProtocol netProtocol;
    ServerSocket serverSocket;
    MessageReceiver messageReceiver;

    Vector connectionList = new Vector();

    boolean shouldBeRunning = true;

    public IncomingSocketConnector(int port, NetProtocol netProtocol, MessageReceiver messageReceiver) {
	this.port = port;
	this.netProtocol = netProtocol;
	this.messageReceiver = messageReceiver;
    }

    public void destroy() {
      NetworkConnection con;

      for (Iterator i = connectionList.iterator(); i.hasNext(); ) {
	con = (NetworkConnection) i.next();
	con.destroy();
      }
    }

    public boolean init() {
	try {
	    serverSocket = new ServerSocket(this.port);
	    serverSocket.setSoTimeout(50);
	} catch (Exception e) {
	    //System.out.println("IncomingSocketConnector: constructor: Error opening server socket - the port " + port + " might be in use. The exception was: "); e.printStackTrace();
	    System.out.println("Error code ISC-1");
	    return false;
	}
	try {
	    if (serverSocket.getLocalPort() != this.port) {
	      System.out.println("Error binding to port " + port);
	      serverSocket = null;
	      return false;
	    }
	} catch (Exception e) {
	    //System.out.println("IncomingSocketConnector: constructor: Error opening server socket - the port " + port + " might be in use. The exception was: "); e.printStackTrace();
	    System.out.println("Error code ISC-2");
	    return false;
	}
	return true;
    }

    public void stopAll() {
	shouldBeRunning = false;
    }

    public void run() {

      int count = 0;
      NetworkConnection networkConnection = null;

	while (shouldBeRunning) {
	    Socket socket = null;
	    try {
		socket = serverSocket.accept();
	    } catch (SocketTimeoutException ste) {
	    } catch(Exception e) {
		//System.out.println("IncomingSocketConnector: serverSocket.accept(): The exception was: "); e.printStackTrace(); break;
		System.out.println("Error code ISC-2");
	    }
	    // System.out.println("ServerSocket Run: " + System.currentTimeMillis());
	    if (socket != null) {
	      try {
		networkConnection = new NetworkConnection(socket, netProtocol.makeCopy(),
		    messageReceiver, this);
		networkConnection.name = "" + count;
		count++;
	      }
	      catch (Exception e) {
		//System.out.println("IncomingSocketConnector: run: Error opening networkConnection - the port " + port + " might be in use. The exception was: "); e.printStackTrace();
		System.out.println("Error code ISC-3");
		networkConnection = null;
	      }
	      if (networkConnection != null) {
		connectionList.addElement(networkConnection);
		// System.out.println("Got new connection " + networkConnection.name + " " + socket.toString());
		// networkConnection.start();
	      }
	    }
	    Thread.currentThread().yield();
	}
	// System.out.println("ISC: run: !shouldBeRunning");
	Iterator conIt = connectionList.iterator();
	while (conIt.hasNext()) {
	    NetworkConnection nc = (NetworkConnection)conIt.next();
	    nc.destroy();
	}

    }

    public void connectionIsDone(NetworkConnection networkConnection) {
	connectionList.removeElement(networkConnection);
	//System.out.println("IncomingSocketConnector: connectionIsDone: Just dereferenced connection.");
    }
}
