/***************************** License **********************************

    Copyright(c)2005, Communicative Machines
    All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are 
met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright 
  notice, this list of conditions and the following disclaimer in the 
  documentation and/or other materials provided with the distribution.

- Neither the name of the copyright holders nor the names of its 
  contributors may be used to endorse or promote products derived from 
  this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

************************************************************************/
// InfoItem.java


package com.cmlabs.air;

import java.util.*;
import org.w3c.dom.*;

public class InfoItem extends BaseObject {

  public String id = "";
  public Time createdTime = new Time();
  TextDictionary entries = new TextDictionary();
  ObjectDictionary timestamps = new ObjectDictionary();
  BaseObject object = null;

    public InfoItem() {
      this.id = Utils.generateID();
    }

    public InfoItem(String xml) {
      fromXML(xml);
    }

    public InfoItem(XMLElement node) {
      fromXML(node);
    }

    public String getEntry(String entry) {
      return entries.get(entry.toLowerCase());
    }

    public boolean setEntry(String entry, String value) {
      return entries.put(entry.toLowerCase(), value);
    }

    public Time getTime(String entry) {
      return (Time) timestamps.get(entry.toLowerCase());
    }

    public boolean setTime(String entry, Time time) {
      return timestamps.put(entry.toLowerCase(), time);
    }

    public String toXML() {
      String xml = createdTime.toXML("createdtime") + "\n";

      if (object != null) {
	xml += "<object>\n" + Utils.xmlIndent(object.toXML()) + "\n" + "</object>\n";
      }

      String key, value;
      int n;
      for (n = 0; n < this.entries.getCount(); n++) {
	key = entries.getKey(n);
	value = entries.get(n);
	if ( (key != null) && (value != null) && (key.length() > 0)) {
	  xml += "<" + Utils.xmlStringEncode(key) + ">" + Utils.xmlStringEncode(value) + "</" + Utils.xmlStringEncode(key) + ">\n";
	}
      }

      Time time;
      for (n = 0; n < this.timestamps.getCount(); n++) {
	key = timestamps.getKey(n).toString();
	time = (Time) timestamps.get(n);
	if ( (key != null) && (time != null) && (key.length() > 0)) {
	  xml += Utils.xmlIndent(time.toXML(Utils.xmlStringEncode(key))) + "\n";
	}
      }

      return "<infoitem id=\"" + Utils.xmlStringEncode(id) + "\">\n" + Utils.xmlIndent(xml) + "</infoitem>\n";
    }

    public boolean fromXML(String xml) {
      return fromXML(Utils.xmlGetRootNodeFromXML(xml));
    }

    public boolean fromXML(XMLElement node) {

      if (node == null)
	return false;

      String nodename = node.getName();
      if ( (nodename == null) || (!nodename.equalsIgnoreCase("infoitem")))
	return false;

      id = node.getStringAttribute("id");

      String key;
      String value;
      XMLElement child;
      XMLElement sub;
      for (Enumeration e=node.enumerateChildren(); e.hasMoreElements(); ) {
	child = (XMLElement) e.nextElement();
	if (child != null) {
	  key = child.getName();
	  value = child.getContent();
	  if (key.equalsIgnoreCase("createdtime"))
	    createdTime = new Time(child);
	  else if ((key.equalsIgnoreCase("object")) && (child.hasAttributes())) {
	    sub = (XMLElement) child.getChildren().elementAt(0);
	    object = createFromXML(sub);
	  }
	  else if (child.hasAttribute("sec")) {
	    timestamps.put(key, new Time(child));
	  }
	  else {
	    entries.put(key, value);
	  }
	}
      }

      return true;
    }

    public String toString() {

	String message = "InfoItem:\n" +
	    "   ID: " + id + "\n" +
	    "   Created: " + createdTime.printString() + "\n";

	String key, value;
	for (int n=0; n<this.entries.getCount(); n++) {
	  key = entries.getKey(n);
	  value = entries.get(n);
	  if ( (key != null) && (value != null) && (key.length() > 0) ) {
	    message += "   " + key + ": " + value + "\n";
	  }
	}

	Time time;
	for (int n=0; n<this.timestamps.getCount(); n++) {
	  key = timestamps.getKey(n).toString();
	  time = (Time) timestamps.get(n);
	  if ( (key != null) && (time != null) && (key.length() > 0) ) {
	    message += "   " + key + ": " + time.printString() + "\n";
	  }
	}

	return message;
    }

    public int getBinarySize(int chunk) {
      if (object == null)
	return 0;
      return object.getBinarySize(chunk);
    }

    public int getBinaryChunkCount() {
      if (object == null)
	return 0;
      return object.getBinaryChunkCount();
    }

    public int toBinaryBuffer(int chunk, byte[] buffer, int offset, int maxlen) {
      if (object == null)
	return 0;
      return object.toBinaryBuffer(chunk, buffer, offset, maxlen);
    }

    public boolean fromBinaryBuffer(int chunk, byte[] buffer, int offset, int len) {
      if (object == null)
	return false;
      return object.fromBinaryBuffer(chunk, buffer, offset, len);
    }

}

