/***************************** License **********************************

    Copyright(c)2005, Communicative Machines
    All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are 
met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright 
  notice, this list of conditions and the following disclaimer in the 
  documentation and/or other materials provided with the distribution.

- Neither the name of the copyright holders nor the names of its 
  contributors may be used to endorse or promote products derived from 
  this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

************************************************************************/
// PlugConnection.java


package com.cmlabs.air;

import java.util.*;

public class JavaAIRPlug extends Communicator {

  ObjectDictionary storedMediaServers = new ObjectDictionary();
  Messenger messenger = null;
  ObjectDictionary triggers = new ObjectDictionary();
  String lastWhiteboard = "";
  String lastFullRegistration = "";
  Time lastStatus = new Time();

    public JavaAIRPlug(String name, String cnsHost, int cnsPort) {
      super(name, cnsHost, cnsPort);
      messenger = new Messenger(name, this, "AIR.Instruct.Wakeup", "Psyclone.Module.Report.PhaseChange");
    }

    public JavaAIRPlug(String name, String cnsHost, int cnsPort, MessageAcceptor messageAcceptor) {
      super(name, cnsHost, cnsPort);
      super.messageAcceptor = messageAcceptor;
      messenger = new Messenger(name, this, "AIR.Instruct.Wakeup", "Psyclone.Module.Report.PhaseChange");
    }

    public boolean init() {
      return init("");
    }

    public boolean init(String registration) {
      if (!super.init())
        return false;
      if (registration.length() == 0)
        requestCurrentPhaseSpec();
      else {
        if (!sendRegistration(registration))
          requestCurrentPhaseSpec();
      }
      sendMsgTo("AIRCentral", new Message(compName, "AIRCentral", "Psyclone.Module.Connect." + compName));
      return true;
    }


    public void destroy() {
      super.destroy();
    }

    public void run() {

    }

    boolean setPlugPriority(double pri) {
      messenger.priority = pri;
      return super.setPlugPriority(pri);
    }

    public boolean isConnected() {
	return super.isConnected();
    }

    public boolean pingServer() {
      return ping("CNS");
    }

    public boolean isServerAvailable() {
      if (lastStatus.getAge() > 3000)
        sendStatus();
      return isConnected();
    }

    public String getServerID() {
      Message send = new Message(compName, "AIRServer", "GET_ID");
      Message recv = sendReceiveMessage("AIRServer", send);
      if ( (recv != null) && (recv.type.equals("ID")) )
        return recv.getContent();
      return "";
    }

    public String getServerName() {
      Message send = new Message(compName, "AIRServer", "GET_NAME");
      Message recv = sendReceiveMessage("AIRServer", send);
      if ( (recv != null) && (recv.type.equals("NAME")) )
        return recv.getContent();
      return "";
    }

    public boolean sendRegistration(String xml) {
      if (xml.length() > 0)
        lastFullRegistration = xml;
      return postMessage("AIRCentral", new Message(compName, "AIRCentral", "AIR.Subscribe", xml, "xml"), "");
    }

    public boolean overwriteRegistration(String xml) {
      if (xml.length() > 0)
        lastFullRegistration = xml;
      return postMessage("AIRCentral", new Message(compName, "AIRCentral", "AIR.Subscribe", xml, "xml"), "");
    }

    public boolean openTwoWayConnectionTo(String name) {
      return openReceivingConnection(this, name);
    }

    public boolean postMessage(String whiteboard, String messageType, String cc) {
      return postMessage(whiteboard, messageType, "", "", cc);
    }

    public boolean postMessage(String whiteboard, String messageType, String content, String language, String cc) {
      Message msg = new Message(compName, whiteboard, messageType, content, language);
      return postMessage(whiteboard, msg, cc);
    }

    public boolean postMessage(String whiteboard, Message msg, String cc) {
      if ( (msg.cc.length() == 0) || (cc.length() > 0))
        msg.cc = cc;
      msg.postedTime = new Time();
      if (msg.priority == 0)
        msg.priority = this.priority;

      long postd = Utils.str2long(statusDict.get("MsgPosted"));
      if (postd < 0)
        postd = 1;
      else
        postd++;
      statusDict.put("MsgPosted", "" + postd);
      perfStat.msgPosted++;
      perfStat.lastMessagePosted = msg.shallowClone();

      boolean res = sendMessage(whiteboard, msg);
      if (lastStatus.getAge() > 3000)
        sendStatus();
      return res;
    }


    public String addTrigger(String whiteboard, String type) {
      return addTrigger(whiteboard, type, false);
    }

    public String addTrigger(String whiteboard, String type, boolean allowSelfTriggering) {
      String id = Utils.generateID("trigger");
      Trigger trigger = new Trigger(whiteboard, type);
      trigger.allowSelfTriggering = allowSelfTriggering;
      triggers.put(id, trigger);
      if (!sendTriggers()) {
	removeTrigger(id);
	return "";
      }
      else
	return id;
    }

    public String addTrigger(String whiteboard, String type, String retrieveXML) {
      return addTrigger(whiteboard, type, retrieveXML, false);
    }

    public String addTrigger(String whiteboard, String type, String retrieveXML, boolean allowSelfTriggering) {
      String id = Utils.generateID("trigger");
      Trigger trigger = new Trigger(whiteboard, type, retrieveXML);
      trigger.allowSelfTriggering = allowSelfTriggering;
      triggers.put(id, trigger);
      if (!sendTriggers()) {
	removeTrigger(id);
	return "";
      }
      else
	return id;
    }

    public boolean removeTrigger(String id) {
      triggers.remove(id);
      return sendTriggers();
    }

    public boolean removeAllTriggers() {
      triggers.removeAll();
      return sendTriggers();
    }

    public ObjectDictionary getTriggers() {
      return triggers;
    }

    public boolean sendTriggers() {

      if (triggers.getCount() == 0)
	return true;

      String xml = "";
      String retrieveXML = "";
      Trigger trigger;

      int n;
      for (n=0; n<triggers.getCount(); n++) {
	trigger = (Trigger) triggers.get(n);
	if (trigger != null) {
	  xml += trigger.toXML();
	  if (trigger.retrieveXML.length() > 0)
	    retrieveXML += trigger.retrieveXML + "\n";
	}
      }
      xml = "<triggers>\n" + Utils.xmlIndent(xml) + "</triggers>\n";
      if (retrieveXML.length() > 0)
	xml += "<retrieves>\n" + Utils.xmlIndent(retrieveXML) + "</retrieves>\n";

      boolean res = this.overwriteRegistration(xml);
      if (lastStatus.getAge() > 3000)
        sendStatus();
      return res;
    }

/*    public ObjectCollection retrieveMessages(String xml) {
      Message msg = new Message(compName, "AIRCentral", "AIR.Retrieve", xml, "xml");
      Message rmsg = sendReceiveMessage("AIRCentral", msg);
      if (!rmsg.type.equalsIgnoreCase("AIR.Retrieve.Success"))
	return null;
      return (ObjectCollection) rmsg.object;
    }*/

    public boolean primaryReconnectNotify() {
      if (!connectToServer()) {
	System.out.println("AIR: Could not reconnect to the Server on '" +
			   cnsLocation.addr + "' port " + cnsLocation.port +
			   "...");
	return false;
      }
//      if (lastWhiteboard.length() == 0)
      // Give the server a chance to start up and be ready...
      Utils.pause(1000);
      if (lastFullRegistration.length() > 0) {
        sendRegistration(lastFullRegistration);
        sendMsgTo("AIRCentral", new Message(compName, "AIRCentral", "Psyclone.Module.Reconnect." + compName));
      }
      else if (triggers.getCount() == 0) {
        requestCurrentPhaseSpec();
        sendMsgTo("AIRCentral", new Message(compName, "AIRCentral", "Psyclone.Module.Reconnect." + compName));
        System.out.println("AIR: Reconnect successful");
      }
      else if (sendTriggers()) {
        sendMsgTo("AIRCentral", new Message(compName, "AIRCentral", "Psyclone.Module.Reconnect." + compName));
	System.out.println("AIR: Reconnect and reregistration successful");
      }
      else {
        requestCurrentPhaseSpec();
        sendMsgTo("AIRCentral", new Message(compName, "AIRCentral", "Psyclone.Module.Reconnect." + compName));
	System.out.println("AIR: Reconnected, but reregistration failed");
      }
      super.primaryReconnectNotify();
      return true;
    }

    public boolean primaryDisconnectNotify() {
      System.out.println("AIR: Primary connection to the Server on '" +
			 cnsLocation.addr + "' port " + cnsLocation.port +
			 " disconnected, will keep trying...");
      super.primaryDisconnectNotify();
      return true;
    }
































    // Get number of triggers up until now
    public int getTriggerCount() {
      return messenger.getTriggerCount();
    }

    public String getTriggerAlias() {
      return messenger.getTriggerAlias();
    }
    // Get first active crank name
    public String getCurrentCrankName() {
      PhaseSpec spec = getPhaseSpec();
      if (spec == null)
        return "";
      CrankSpec crankSpec = (CrankSpec) spec.cranks.getFirst();
      if (crankSpec == null)
        return "";
      return crankSpec.name;
    }

    // Get all active crank name
    public TextCollection getCurrentCrankNames() {
      TextCollection col = new TextCollection();
      PhaseSpec spec = getPhaseSpec();
      if (spec == null)
        return col;
      CrankSpec crankSpec;
      for (int n=0; n<spec.cranks.getCount(); n++) {
        crankSpec = (CrankSpec) spec.cranks.get(n);
        if (crankSpec != null)
          col.add(crankSpec.name);
      }
      return col;
  }

	// Test whether I am in Context
	public boolean inContext() {
		PhaseSpec spec = getPhaseSpec();
		if (spec == null)
			return false;
		return true;
	}

	// return current Context name
	public String getCurrentContextName() {
		PhaseSpec spec = getPhaseSpec();
		if (spec == null)
			return "";
		return spec.context;
	}

	// return current Phase name
	public String getCurrentPhaseName() {
		PhaseSpec spec = getPhaseSpec();
		if (spec == null)
			return "";
		return spec.name;
	}


    // Get list of post/output destinations
    public TextCollection getDestinations() {
      return messenger.getDestinations();
    }

    // Has any new trigger message arrived
    public Message waitForNewMessage(int timeout) {
      if (lastStatus.getAge() > 3000)
        sendStatus();
      return messenger.waitForNewMessage(timeout);
   /*   Message msg = messenger.waitForNewMessage(timeout);
      if (msg == null)
        return null;

      PhaseSpec phaseSpec = messenger.getPhaseSpec();
      if (phaseSpec == null)
        return null;

      StreamAccess accessSpec;
      MediaConnection mediaConnection;
      for (int n = 0; n < phaseSpec.streams.getCount(); n++) {
        accessSpec = (StreamAccess) phaseSpec.streams.get(n);
        if (accessSpec != null) {
          if (storedMediaServers.contains(accessSpec.source)) {
            messenger.addStream(accessSpec.alias,
                                (MediaConnection) storedMediaServers.get(accessSpec.source));
          }
          else {
            mediaConnection = connectToMediaServer(accessSpec.source);
            if (mediaConnection != null) {
              storedMediaServers.put(accessSpec.source, mediaConnection);
              messenger.addStream(accessSpec.alias, mediaConnection);
            }
          }
        }
      }*/

      // return getTriggerMessage();
  }

  // How many trigger messages are waiting
  public int getInputMessageCount() {
    return messenger.getInputMessageCount();
  }

  // How many output messages have not yet been sent
  public int getOutputMessageCount() {
    return messenger.getOutputMessageCount();
  }

  // Get the current Trigger Message
  public Message getWakeupMessage() {
    return messenger.getWakeupMessage();
  }

  // Get the current Trigger Message
  public Message getTriggerMessage() {
    return messenger.getTriggerMessage();
  }

  // Get the current number of retrieved messages
  public int getRetrievedMessageCount() {
    return messenger.getRetrievedMessageCount();
  }

  // Get the full collection of the retrieved messages
  public ObjectCollection getAllRetrievedMessages() {
    return messenger.getAllRetrievedMessages();
  }

  // Get a specific retrieved message
  public Message getRetrievedMessage(int pos) {
    return messenger.getRetrievedMessage(pos);
  }

// Get the current phase spec
  public PhaseSpec getPhaseSpec() {
    return messenger.getPhaseSpec();
  }

  // Retrieve additional messages from a source
  public ObjectCollection retrieveMessages(String retrieveSpecXML) {
    return messenger.retrieveMessages("AIRCentral", retrieveSpecXML);
  }

  // Retrieve additional messages from a source
  public ObjectCollection retrieveMessages(RetrieveSpec retrieveSpec) {
    return messenger.retrieveMessages("AIRCentral", retrieveSpec);
  }

  // Retrieve additional messages from a source
  public ObjectCollection retrieveMessages(ObjectCollection retrieveSpecs) {
    return messenger.retrieveMessages("AIRCentral", retrieveSpecs);
  }

  // Put an output message in the outbox
  public boolean addOutputMessage(Message msg) {
    return messenger.addOutputMessage(msg);
  }

  // Put an output message in the outbox
  public boolean addOutputMessage(Message msg, String destination) {
    return messenger.addOutputMessage(msg, destination);
  }

  // Send one message
  public boolean postOutputMessage(Message msg) {
    messenger.addOutputMessage(msg);
    return messenger.sendOutputMessages();
  }

  // Send one message
  public boolean postOutputMessage(Message msg, String destination) {
    messenger.addOutputMessage(msg, destination);
    return messenger.sendOutputMessages();
  }


  // Send all messages in the outbox
  public boolean sendOutputMessages() {
    boolean res = messenger.sendOutputMessages();
    if (lastStatus.getAge() > 3000)
      sendStatus();
    return res;
  }

  // Send message to all destinations
  public boolean postMessageToAllDestinations(Message msg) {
    boolean res = messenger.postMessageToAllDestinations(msg);
    if (lastStatus.getAge() > 3000)
      sendStatus();
    return res;
  }

  // Get a list of stream names
  public TextCollection getStreamNames() {
    return messenger.getStreamNames();
  }

  // Get a specific stream by name
  public MediaConnection getStreamConnection(String name) {
    return messenger.getStreamConnection(name);
  }

  // Wait for a specific stream by name
  public MediaConnection waitForStreamConnection(String name, int timeout) {
    MediaConnection con = messenger.getStreamConnection(name);
    if (con != null)
      return con;

    // Wait for streams to be available
    Time start = new Time();
    while (true) {
      if (getStreamNames().contains(name)) {
        if ( (con = messenger.getStreamConnection(name)) != null)
          return con;
      }
      Utils.pause(100);
      if (start.getAge() > timeout) {
        break;
      }
    }
    return null;
  }

  public MediaConnection connectToMediaServer(String mediaName) {

    TCPLocation loc = resolve(mediaName);
    if ((loc == null) || (!loc.isValid()))
      return null;

    MediaConnection mediacon = new MediaConnection(compName, loc);
    if (!mediacon.init()) {
      return null;
    }

    return mediacon;
  }

  public Message acceptMessage(Message message) {
    if ((messenger == null) || (message == null))
      return null;

    Message msg;
    if (message.type.equalsIgnoreCase("AIR.Instruct.Wakeup")) {
      if ( (msg = MessageProcessor.getTriggerMessage(message)) == null) {
        messenger.processPhaseSpec(MessageProcessor.getPhaseSpec(message), message.postedTime);
        processMediaInformation(messenger.getPhaseSpec());
        return new Message(compName, message.from, "RECEIVE_ACCEPT");
      }
    }

    messenger.addInputMessage(message);
    return new Message(compName, message.from, "RECEIVE_ACCEPT");
  }

  public boolean processMediaInformation(PhaseSpec phaseSpec) {

    if (phaseSpec == null) {
    //  System.out.println("Processing no PhaseSpec for Media Information...");
      return false;
    }

    //System.out.println("Processing PhaseSpec " + phaseSpec.name + " for Media Information...");
    messenger.clearStreams();

    TextCollection currentStreams = new TextCollection();

    StreamAccess accessSpec;
    MediaConnection mediaConnection;
    for (int n = 0; n < phaseSpec.streams.getCount(); n++) {
      accessSpec = (StreamAccess) phaseSpec.streams.get(n);
      if (accessSpec != null) {
        if (storedMediaServers.contains(accessSpec.source)) {
        //  System.out.println("  Found known stream: " + accessSpec.alias);
          messenger.addStream(accessSpec.alias,
                              (MediaConnection) storedMediaServers.get(accessSpec.source));
        }
        else {
          mediaConnection = connectToMediaServer(accessSpec.source);
          if (mediaConnection != null) {
          //  System.out.println("  Found new stream: " + accessSpec.alias);
            storedMediaServers.put(accessSpec.source, mediaConnection);
            messenger.addStream(accessSpec.alias, mediaConnection);
          }
        }
        currentStreams.add(accessSpec.source);
      }
    }

    // Now, any stream in storedMediaServers and NOT in currentStreams should be reset
    String key;
    for (int m = 0; m < storedMediaServers.getCount(); m++) {
      key = storedMediaServers.getKey(m);
      if (!currentStreams.contains(key)) {
        if ( (mediaConnection = (MediaConnection) storedMediaServers.get(key)) != null ) {
          mediaConnection.reset();
        }
      }
    }

    return true;
  }

  public boolean requestCurrentPhaseSpec() {
    Message msg = sendReceiveMsgTo("Psyclone", new Message(compName, "Psyclone", "Psyclone.Module.Request.CurrentPhase"));
    if ( (msg == null) || (!msg.type.equalsIgnoreCase("Psyclone.Module.CurrentPhase")) ) {
      return false;
    }
    PhaseSpec spec = (PhaseSpec) msg.object;
    messenger.processPhaseSpec(spec, msg.postedTime);
    processMediaInformation(messenger.getPhaseSpec());
    return true;
  }


  boolean sendStatus() {
    if (lastStatus == null)
      return true;
    lastStatus = new Time();
    perfStat.lastUpdated = new Time();
    return sendMsgTo("AIRServer", new Message(compName, "AIRServer", "System.PerfStat", perfStat));
  }

  boolean noStatus() {
    lastStatus = null;
    return true;
  }

  void doMaintenance() {
    super.doMaintenance();
    if (lastStatus.getAge() > 5000)
      sendStatus();
  }


}
