/***************************** License **********************************

    Copyright(c)2005, Communicative Machines
    All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are 
met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright 
  notice, this list of conditions and the following disclaimer in the 
  documentation and/or other materials provided with the distribution.

- Neither the name of the copyright holders nor the names of its 
  contributors may be used to endorse or promote products derived from 
  this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

************************************************************************/
// MediaConnection.java


package com.cmlabs.air;

import java.util.*;
import org.w3c.dom.*;

public class MediaConnection extends Thread implements MessageReceiver {

  public int maxBufferCount = 100;
  public boolean shouldContinue = true;
  TCPLocation mediaLocation = null;
  NetworkConnection mediaCon = null;
  NetworkConnection continuousCon = null;
  public String myName = "";

  ObjectCollection buffer = new ObjectCollection();

  public MediaConnection(String name, TCPLocation loc) {
    mediaLocation = loc;
    myName = name;
  }

  public void destroy() {
    shutdown();
  }

  public boolean isConnected() {
    return ( (mediaCon != null) && (mediaCon.isConnected()) );
  }

  public boolean init() {
    // Connect to Media Server
    if ( (mediaCon != null) && (mediaCon.isConnected()) )
      return true;

    try {
    //  mediaCon = new NetworkConnection(mediaLocation.addr, mediaLocation.port, new NetMessageProtocol());
      mediaCon = new NetworkConnection(mediaLocation, new NetMessageProtocol());
    }
    catch (Exception e) {
      System.out.println("MediaConnection: Could not connect to MediaServer on '" +
			 mediaLocation.addr + "' port " + mediaLocation.port + "...");
      //e.printStackTrace();
      //System.out.println("Error code JAP-1");
      return false;
    }

    // Send a message to start the protocol on the server
    this.getServerName();
    return true;
  }

  public boolean shutdown() {
    if (continuousCon != null)
      continuousCon.destroy();
    continuousCon = null;
    if (mediaCon != null)
      mediaCon.destroy();
    mediaCon = null;
    return true;
  }

  public void run() {

    while (shouldContinue) {
      Utils.pause(100);
    }

  }


  public boolean reset() {
    stopContinuousBackgroundReceive();
    buffer.removeAll();
    return true;
  }

  public String getServerName() {
    return askRemoteServerForString("GET_NAME", 3000);
  }

  public ConnectionProfile getServerConnectionProfile() {
    return (ConnectionProfile) askRemoteServerForObject("GET_CONNECTION_PROFILE", 3000);
  }

  public Time getOldestTimestamp() {
    return (Time) askRemoteServerForObject("GET_OLDEST_TIMESTAMP", 3000);
  }

  public Time getNewestTimestamp() {
    return (Time) askRemoteServerForObject("GET_NEWEST_TIMESTAMP", 3000);
  }

  public long getCount() {
    return Utils.str2long(askRemoteServerForString("GET_COUNT", 3000));
  }

  public long getSize() {
    return Utils.str2long(askRemoteServerForString("GET_SIZE", 3000));
  }

  public ObjectCollection getDataSampleList() {
    return (ObjectCollection) askRemoteServerForObject("GET_SAMPLE_LIST", 3000);
  }


  public DataSample getDataSample(String id) {
    return askRemoteServerForData("GET_SAMPLE_BY_ID", id, 3000);
  }

  public DataSample getDataSample(Time time) {
    return askRemoteServerForData("GET_SAMPLE_BY_TIME", time, 3000);
  }

  public ObjectCollection getDataSamples(Time t1, Time t2) {
    ObjectCollection col = new ObjectCollection();
    col.add(t1);
    col.add(t2);
    return askRemoteServerForDataCollection("GET_SAMPLES_BY_TIME", col, 3000);
  }


  public boolean addDataSample(DataSample sample) {
    return sendSampleToRemoteServer("ADD_SAMPLE", sample);
  }

  public boolean addDataSamples(ObjectCollection samples) {
    return sendSamplesToRemoteServer("ADD_SAMPLES", samples);
  }


  public DataSample waitForFirstSampleAfter(Time t, int timeout) {
    return askRemoteServerForData("WAIT_FOR_FIRST_SAMPLE " + timeout, t, timeout+3000);
  }

  public DataSample waitForLastSampleAfter(Time t, int timeout) {
    return askRemoteServerForData("WAIT_FOR_LAST_SAMPLE " + timeout, t, timeout+3000);
  }

  public DataSample waitForFirstSampleAfter(String id, int timeout) {
    return askRemoteServerForData("WAIT_FOR_FIRST_AFTERID " + timeout, id, timeout+3000);
  }

  public DataSample waitForLastSampleAfter(String id, int timeout) {
    return askRemoteServerForData("WAIT_FOR_LAST_AFTERID " + timeout, id, timeout+3000);
  }





















  public ObjectCollection askRemoteServerForDataCollection(String question, BaseObject obj, int timeout) {
    Message msg = new Message(myName, mediaLocation.name, question);
    msg.object = obj;
    Message rmsg = askRemoteServer(msg, 2000);

    if (rmsg == null)
      return null;

    ObjectCollection answer = (ObjectCollection) rmsg.object;

    if (answer != null) {
    }

    rmsg = null;
    return answer;
  }

  public DataSample askRemoteServerForData(String question, BaseObject obj, int timeout) {
    Message msg = new Message(myName, mediaLocation.name, question);
    msg.object = obj;
    Message rmsg = askRemoteServer(msg, 2000);

    if (rmsg == null)
      return null;

    DataSample answer = (DataSample) rmsg.object;

    if (answer != null) {
    }

    rmsg = null;
    return answer;
  }

  public DataSample askRemoteServerForData(String question, String str, int timeout) {
    Message msg = new Message(myName, mediaLocation.name, question);
    msg.content = str;
    Message rmsg = askRemoteServer(msg, 2000);

    if (rmsg == null)
      return null;

    DataSample answer = (DataSample) rmsg.object;

    if (answer != null) {
    }

    rmsg = null;
    return answer;
  }

  public boolean sendSampleToRemoteServer(String question, DataSample sample) {
    Message msg = new Message(myName, mediaLocation.name, question);
    msg.object = sample;
    Message rmsg = askRemoteServer(msg, 2000);

    if (rmsg == null)
      return false;

    String answer = rmsg.type;
    rmsg = null;
    return(answer.equalsIgnoreCase("DATA_ACCEPTED"));
  }

  public boolean sendSamplesToRemoteServer(String question, ObjectCollection samples) {
    Message msg = new Message(myName, mediaLocation.name, question);
    msg.object = samples;
    Message rmsg = askRemoteServer(msg, 2000);

    if (rmsg == null)
      return false;

    String answer = rmsg.type;
    rmsg = null;
    return(answer.equalsIgnoreCase("DATA_ACCEPTED"));
  }





  public BaseObject askRemoteServerForObject(String question, int timeout) {
	Message msg = new Message(myName, mediaLocation.name, question);
	Message rmsg = askRemoteServer(msg, timeout);

	if (rmsg == null)
	  return null;

	BaseObject answer = (BaseObject) rmsg.object;
	rmsg = null;
	return answer;
  }

  public String askRemoteServerForString(String question, int timeout) {

    Message msg = new Message(myName, mediaLocation.name, question);
    Message rmsg = askRemoteServer(msg, timeout);

    if (rmsg == null)
      return null;

    String answer = rmsg.content;
    rmsg = null;
    return answer;
  }

  public Message askRemoteServer(Message msg, int timeout) {

//    boolean isContinuousStartRequest = false;
//    if (msg.type.equalsIgnoreCase("START_CONTINUOUS_RECEIVE"))
//      isContinuousStartRequest = true;

    if (msg == null)return null;

    if (timeout < 0)
      timeout = 3000;
    else
      timeout += 3000;

    Message rmsg = mediaCon.sendReceiveMessage(msg, timeout);

    if (rmsg == null) {
      mediaCon.destroy();
      mediaCon = null;
//      continuousCon = null;
      if (!init()) {
	System.out.println(
	    "MediaConnection lost connection to Media Server and could not reconnect...");
	return null;
      }
      rmsg = mediaCon.sendReceiveMessage(msg, timeout);
      if (rmsg == null) {
	System.out.println(
	    "MediaConnection lost connection to Media Server and could not reconnect...");
	return null;
      }
    }

//    if ( (isContinuousStartRequest) &&
//	(rmsg.type.equalsIgnoreCase("STARTED_CONTINUOUS_RECEIVE"))) {
//      continuousCon = mediaCon;
//    }

    return rmsg;
  }















  public Message receiveMessage(Message msg) {
//    System.out.println("Receiving continuous sample...");
    if (msg.object == null)
      return null;
    synchronized(buffer) {
      buffer.add(msg.object);
      //System.out.println("Now we have " + buffer.getCount() + " in the media buffer...");
      if (buffer.getCount() > maxBufferCount)
        buffer.remove(0);
      buffer.notify();
    }
    return new Message("", "", "RECEIVE_ACCEPT");
  }

  public boolean isInContinuousReceive() {
    return ((continuousCon != null) && (continuousCon.isConnected()));
  }

  public DataSample receiveContinuousSample(int timeout) {

    if (continuousCon == null) {
      Utils.pause(5);
      return null;
    }
    DataSample sample = null;
    synchronized(buffer) {
      if ( (sample = (DataSample) buffer.getFirst()) != null) {
        buffer.remove(0);
        return sample;
      }
      try {
        buffer.wait(timeout);
      }
      catch(Exception e) {}
      sample = (DataSample) buffer.getFirst();
    }
    return sample;
  }

  public boolean startContinuousBackgroundReceive() {
    return startContinuousBackgroundReceive(null);
  }

  public boolean startContinuousBackgroundReceive(MessageReceiver receiver) {

    if (isInContinuousReceive())
      return true;

    if (receiver == null)
        receiver = this;

    if (continuousCon != null) {
      continuousCon.destroy();
    }

    try {
      continuousCon = new NetworkConnection(mediaLocation, new NetMessageProtocol());
    }
    catch (Exception e) {
      System.out.println("MediaConnection: Could not connect to MediaServer on '" +
                         mediaLocation.addr + "' port " + mediaLocation.port + "...");
      //e.printStackTrace();
      //System.out.println("Error code JAP-1");
      return false;
    }

    if (!continuousCon.initializeAsReceiver(receiver, myName)) {
      continuousCon.destroy();
      continuousCon = null;
      return false;
    }

    if (!continuousCon.isConnected()) {
      continuousCon.destroy();
      continuousCon = null;
      return false;
    }

    return true;

  }

  public boolean stopContinuousBackgroundReceive() {

    if (!isInContinuousReceive()) {
      if (continuousCon != null) {
        continuousCon.destroy();
        continuousCon = null;
      }
      return true;
    }

    if (continuousCon != null) {
      continuousCon.destroy();
      continuousCon = null;
    }

    return true;
  }

}


