/***************************** License **********************************

    Copyright(c)2005, Communicative Machines
    All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are 
met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright 
  notice, this list of conditions and the following disclaimer in the 
  documentation and/or other materials provided with the distribution.

- Neither the name of the copyright holders nor the names of its 
  contributors may be used to endorse or promote products derived from 
  this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

************************************************************************/
// Message.java


package com.cmlabs.air;

import java.util.*;
import org.w3c.dom.*;

public class Message extends BaseObject {

    public String to="", from="", content="", id="", type="", language="", cc="", stored="", isresponse="";
    Reference inReplyTo;
    double timetolive=0, priority=0;
    public Time postedTime = new Time();
    public Time receivedTime;
    TextDictionary entryMap = new TextDictionary();
    ObjectDictionary timeMap = new ObjectDictionary();
    public XMLElement xmlContentNode = null;
    public Vector attachedMsgs = null;
    public String phaseText = "";
    public BaseObject object = null;
    public ObjectDictionary history = new ObjectDictionary();

    public Message() {
	    this.id = Utils.generateID();
    }

    public Message(String xml) {
      //long beginTime = System.currentTimeMillis();
      fromXML(xml);
      //long totalTime = System.currentTimeMillis() - beginTime;
      //System.out.println("Total time to parse message: " + totalTime);
    }

    public Message(XMLElement node) {
      fromXML(node);
    }

/*    public Message(String xml) {
	to = get("to", xml);
	from = get("from", xml);
	content = get("content", xml);
	id = get("id", xml);
	type = get("type", xml);
	createdTime = getTime("createdTime", xml);
	postedTime = getTime("postedTime", xml);
	receivedTime = getTime("receivedTime", xml); // this should be set by whatever creates this message
    }*/

  // ONLY testcases use this
  public Message(String from, String to, String type, String content, String language, String id) {
    this.to = to;
    this.from = from;
    this.content = content;
    this.language = language;
    this.id = id;
    this.type = type;
    // this.postedTime = new Time();
    // this.receivedTime = new Time();
  }

  // this one is generaly used
  // same as above, but generates ID
  public Message(String from, String to, String type, String content,
		 String language) {
    this.to = to;
    this.from = from;
    this.content = content;
    this.language = language;
    this.id = Utils.generateID();
    this.type = type;
    // this.postedTime = new Time();
    // this.receivedTime = new Time();
  }

  // this one is generaly used
  // same as above, but generates ID
  public Message(String from, String to, String type) {
    this.to = to;
    this.from = from;
    this.content = "";
    this.language = "";
    this.id = Utils.generateID();
    this.type = type;
    // this.postedTime = new Time();
    // this.receivedTime = new Time();
  }

  // this one is generaly used
  // same as above, but generates ID
  public Message(String from, String to, String type, BaseObject obj) {
    this.to = to;
    this.from = from;
    this.content = "";
    this.language = "";
    this.id = Utils.generateID();
    this.type = type;
    this.object = obj;
    // this.postedTime = new Time();
    // this.receivedTime = new Time();
  }

  public Message shallowClone() {
    Message msg = new Message(from, to, type, content, language, id);
    msg.stored = stored;
    msg.postedTime = postedTime;
    msg.receivedTime = receivedTime;
    msg.priority = priority;
    msg.timetolive = timetolive;
    return msg;
  }

  public String get(String key) {
    return entryMap.get(key.toLowerCase());
  }

  public String getOther(String key) {
    return entryMap.get(key.toLowerCase());
  }

  public boolean set(String key, String value) {
    return entryMap.put(key.toLowerCase(), value);
  }

  public boolean setOther(String key, String value) {
    return entryMap.put(key.toLowerCase(), value);
  }

  public Time getTime(String key) {
    return (Time) timeMap.get(key.toLowerCase());
  }

  public boolean setTime(String key, Time time) {
    return timeMap.put(key.toLowerCase(), time);
  }


    public String getContent() {
      if (object != null) {
	return object.toXML();
      }
      else
	return this.content;
    }

    public String toXML() {
      String inReplyToXML = "";
      if (inReplyTo != null)
	inReplyToXML = "<inreplyto>\n" + Utils.xmlIndent(this.inReplyTo.toXML()) + "</inreplyto>\n";
      String xml = "<type>" + Utils.xmlStringEncode(this.type) + "</type>\n" +
	  "<id>" + Utils.xmlStringEncode(this.id) + "</id>\n" +
	  "<from>" + Utils.xmlStringEncode(this.from) + "</from>\n" +
	  "<to>" + Utils.xmlStringEncode(this.to) + "</to>\n" +
	  "<cc>" + Utils.xmlStringEncode(this.cc) + "</cc>\n" +
	  inReplyToXML +
	  "<stored>" + Utils.xmlStringEncode(this.stored) + "</stored>\n" +
	  "<isresponse>" + Utils.xmlStringEncode(this.isresponse) + "</isresponse>\n" +
	  this.postedTime.toXML("postedtime") + "\n";
      if (receivedTime != null) {
	xml += this.receivedTime.toXML("receivedtime") + "\n";
      }

      if (object != null) {
	xml += "<content language=\"xml\">\n" + Utils.xmlIndent(object.toXML()) + "\n" + "</content>" + "\n";
      }
      else if (language.equalsIgnoreCase("xml"))
	xml += "<content language=\"xml\">\n" + Utils.xmlIndent(this.content) + "\n" + "</content>" + "\n";
      else
	xml += "<content language=\"" + Utils.xmlStringEncode(this.language) + "\">" + Utils.xmlStringEncode(this.content) + "</content>" + "\n";

      String key, value;
      for (int n = 0; n < this.entryMap.getCount(); n++) {
	key = entryMap.getKey(n);
	value = entryMap.get(n);
	if ( (key != null) && (value != null) && (key.length() > 0)) {
	  xml += "<" + Utils.xmlStringEncode(key) + ">" + Utils.xmlStringEncode(value) + "</" + Utils.xmlStringEncode(key) + ">\n";
	}
      }

      Time t;
      for (int n = 0; n < this.entryMap.getCount(); n++) {
        key = timeMap.getKey(n);
        t = (Time) timeMap.get(n);
        if ( (key != null) && (t != null) && (key.length() > 0)) {
          xml += t.toXML(key) + "\n";
        }
      }

      ObjectCollection refCol;
      String author;
      Reference ref;
      String histXML = "", refXML;
      for (int m = 0; m < history.getCount(); m++) {
	author = (String) history.getKey(m);
	refCol = (ObjectCollection) history.get(m);
	if (refCol != null) {
	  refXML = "";
	  for (int k = 0; k < refCol.getCount(); k++) {
	    ref = (Reference) refCol.get(k);
	    if (ref != null) {
	      refXML += ref.toXML();
	    }
	  }
	  histXML += "<group author=\"" + Utils.xmlStringEncode(author) + "\">\n" + Utils.xmlIndent(refXML) + "</group>";
	}
      }
      if (histXML.length() > 0)
	xml += "<history>\n" + Utils.xmlIndent(histXML) + "</history>";

      return "<message priority=\"" + priority + "\" timetolive=\"" + timetolive + "\">\n" + Utils.xmlIndent(xml) + "</message>\n";
    }

    public boolean fromXML(String xml) {
      return fromXML(Utils.xmlGetRootNodeFromXML(xml));
    }

    public boolean fromXML(XMLElement node) {

      if (node == null)
	return false;

      xmlContentNode = null;

      String nodename = node.getName();
      if ( (nodename == null) || (!nodename.equalsIgnoreCase("message")))
	return false;

      priority = node.getDoubleAttribute("priority");
      timetolive = node.getDoubleAttribute("timetolive");

      String key;
      String value;
      XMLElement child;
      XMLElement sub;
      for (Enumeration e=node.enumerateChildren(); e.hasMoreElements(); ) {
	child = (XMLElement) e.nextElement();
	if (child != null) {
	  key = child.getName();
	  value = child.getContent();
	  if (key.equalsIgnoreCase("from"))
	    from = value;
	  else if (key.equalsIgnoreCase("to"))
	    to = value;
	  else if (key.equalsIgnoreCase("cc"))
	    cc = value;
	  else if (key.equalsIgnoreCase("id"))
	    id = value;
	  else if (key.equalsIgnoreCase("type"))
	    type = value;
	  else if (key.equalsIgnoreCase("stored"))
	    stored = value;
	  else if (key.equalsIgnoreCase("isresponse"))
	    isresponse = value;
	  else if ((key.equalsIgnoreCase("content")) && (child.hasAttributes())) {
	    language = child.getStringAttribute("language");
	    if (language.equalsIgnoreCase("xml")) {
	      Vector v = child.getChildren();
	      object = null;
	      if ((v != null) &&  (v.size() > 0)) {
		sub = (XMLElement) child.getChildren().elementAt(0);
		object = createFromXML(sub);
		if (object == null) {
		  xmlContentNode = sub;
		  content = xmlContentNode.toString();
		}
	      }
	      else
		content = value;
	    }
	    else
	      content = value;
	  }
	  else if ((key.equalsIgnoreCase("inreplyto")) && (child.hasChildren())) {
	    sub = (XMLElement) child.getChildren().elementAt(0);
	    if (sub != null)
		    inReplyTo = new Reference(sub);
	  }
	  else if ((key.equalsIgnoreCase("history")) && (child.hasChildren())) {
	    XMLElement groupNode, refNode;
	    ObjectCollection refCol;
	    String author;
	    for (Enumeration groups=child.enumerateChildren(); groups.hasMoreElements(); ) {
	      groupNode = (XMLElement) groups.nextElement();
	      if (groupNode != null) {
		author = groupNode.getStringAttribute("author");
		refCol = new ObjectCollection();
		history.put(author, refCol);
		for (Enumeration refs = groupNode.enumerateChildren();
		     refs.hasMoreElements(); ) {
		  refNode = (XMLElement) refs.nextElement();
		  if (refNode != null) {
		    refCol.add(new Reference(refNode));
		  }
		}
	      }
	    }
	  }
	  else if (key.equalsIgnoreCase("postedtime"))
	    postedTime = new Time(child);
	  else if (key.equalsIgnoreCase("receivedtime"))
	    receivedTime = new Time(child);
	  else {
            if (child.hasAttribute("sec"))
              setTime(key, new Time(child));
            else
              set(key, value);
          }
	}
      }

      // if (receivedTime == null)
	// receivedTime = postedTime;

      return true;
    }

    /*
    public String toXml_Legacy() {
	String xml = "<message><content>" +
			"<collection type=\"dictionary\">" +
			    "<entry type=\"String\" key=\"timestamp\">" + this.timestamp  +
			    "</entry>" +
			    "<entry type=\"String\" key=\"id\">" + this.id +
			    "</entry>" +
			    "<entry type=\"String\" key=\"from\">" + this.from +
			    "</entry>" +
			    "<entry type=\"String\" key=\"to\">" + this.to +
			    "</entry>" +
			    "<entry type=\"String\" key=\"type\">" + this.type +
			    "</entry>" +
			    "<entry type=\"String\" key=\"content\">" + this.content +
			    "</entry>" +
		    "</collection>" +
	    "</content></message>";
	return xml;
    }
*/
    public String toString() {

 //     public String to="", from="", content="", id="", type="", language="", cc="", stored="", extensions="";

	String message = "Message:\n" +
	    "   ID: " + id + "\n" +
	    "   From: " + from + "\n" +
	    "   To: " + to + "\n" +
	    "   Type: " + type + "\n" +
	    "   Language: " + language + "\n" +
	    "   Content: " + content + "\n" +
	    "   CC: " + cc + "\n" +
	    "   Stored: " + stored + "\n";

	if (inReplyTo != null)
	    message += "   InReplyTo: " + inReplyTo.toString() + "\n";

	String key, value;
	for (int n=0; n<this.entryMap.getCount(); n++) {
	  key = entryMap.getKey(n);
	  value = entryMap.get(n);
	  if ( (key != null) && (value != null) && (key.length() > 0) ) {
	    message += "   " + key + ": " + value + "\n";
	  }
	}

	return message;
    }

    public boolean addReferenceToHistory(Message msg, String author) {
      if (msg == null)
	return false;
      return addReferenceToHistory(new Reference(msg), author);
    }

    public boolean addReferenceToHistory(Reference ref, String author) {
      if (ref == null)
	return false;
      if (history == null)
	history = new ObjectDictionary();
      ObjectCollection col = (ObjectCollection) history.get(author);
      if (col == null) {
	col = new ObjectCollection();
	history.put(author, col);
      }
      col.add(ref);
      return true;
    }

    public int getBinarySize(int chunk) {
      if (object == null)
	return 0;
      return object.getBinarySize(chunk);
    }

    public int getBinaryChunkCount() {
      if (object == null)
	return 0;
      return object.getBinaryChunkCount();
    }

    public int toBinaryBuffer(int chunk, byte[] buffer, int offset, int maxlen) {
      if (object == null)
	return 0;
      return object.toBinaryBuffer(chunk, buffer, offset, maxlen);
    }

    public boolean fromBinaryBuffer(int chunk, byte[] buffer, int offset, int len) {
      if (object == null)
	return false;
      return object.fromBinaryBuffer(chunk, buffer, offset, len);
    }


}

