/***************************** License **********************************

    Copyright(c)2005, Communicative Machines
    All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are 
met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright 
  notice, this list of conditions and the following disclaimer in the 
  documentation and/or other materials provided with the distribution.

- Neither the name of the copyright holders nor the names of its 
  contributors may be used to endorse or promote products derived from 
  this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

************************************************************************/
// MessageProcessor.java


package com.cmlabs.air;

public class MessageProcessor {

//! Get the trigger message from msg
  public static boolean isWakeupMessage(Message msg) {
    if (msg == null)
      return false;
    if (!msg.type.equalsIgnoreCase("AIR.Instruct.Wakeup"))
      return false;
    try {
      ObjectDictionary dict = (ObjectDictionary) msg.object;
      if (dict == null)
        return false;
    }
    catch(Exception e) {return false;}
    return true;
  }

//! Get the trigger message from msg
  public static Message getTriggerMessage(Message msg) {
    try {
      ObjectDictionary dict = (ObjectDictionary) msg.object;
      if (dict == null)
        return null;
      Message hmsg = (Message) dict.get("TriggerMessage");
      return hmsg;
    }
    catch(Exception e) {return null;}
  }

//! Get the retrieved message collection from msg
  public static ObjectCollection getRetrievedMessages(Message msg) {
    try {
      ObjectDictionary dict = (ObjectDictionary) msg.object;
      if (dict == null)
        return null;
      ObjectCollection msgs = (ObjectCollection) dict.get("RetrievedMessages");
      return msgs;
    }
    catch(Exception e) {return null;}
  }

//! Get the PhaseSpec from msg
  public static PhaseSpec getPhaseSpec(Message msg) {
    try {
      ObjectDictionary dict = (ObjectDictionary) msg.object;
      if (dict == null)
        return null;
      PhaseSpec phasespec = (PhaseSpec) dict.get("PhaseSpec");
      return phasespec;
    }
    catch(Exception e) {return null;}
  }


//! Get an attached message from msg
  public static Message getMessage(Message msg, int pos) {

     if (pos == 0)
	return getTriggerMessage(msg);

     ObjectCollection coll = getRetrievedMessages(msg);
     if (coll != null) {
	return (Message) coll.get(pos);
     }

     return null;
  }
}
