/***************************** License **********************************

    Copyright(c)2005, Communicative Machines
    All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are 
met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright 
  notice, this list of conditions and the following disclaimer in the 
  documentation and/or other materials provided with the distribution.

- Neither the name of the copyright holders nor the names of its 
  contributors may be used to endorse or promote products derived from 
  this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

************************************************************************/
// NetMessageProtocol.java


package com.cmlabs.air;

import java.net.*;
import java.util.*;
import java.io.*;

public class NetMessageProtocol implements NetProtocol{

    Socket mySocket = null;
    OutputStream myOutputStream = null;
    InputStream myInputStream = null;
    String lastError = "";

    public NetMessageProtocol() {
    }

    public NetProtocol makeCopy() {
      return new NetMessageProtocol();
    }

    public boolean initializeConversation(Socket socket) {
      return true;
    }

    public boolean initializeAsReceiver(Socket socket, String myname) {

		if (mySocket != socket) {
		    try {
				myOutputStream = socket.getOutputStream();
				myInputStream = socket.getInputStream();
		    } catch(Exception e1) {
				myOutputStream = null;
				myInputStream = null;
				mySocket = socket;
		    }
		}

	//  System.out.println("InitializeAsReceiver...!!!");

      Message msg = new Message(myname, "", "RECEIVE_NAME_NOTIFICATION");
      String messageXml = msg.toXML();
      byte[] messageAndHeaderBytes = getMessageAndHeaderBytes(messageXml, "Receive", 0);

      // a series of quick checks that we have all we need to write to the socket
      if (socket == null) {
	  //System.out.println("NetMessageProtocol: sendMessage: (socket == null): returning false");
	  System.out.println("Error code NMP-2");
	  return false;
      }
      if (messageAndHeaderBytes == null) {
	  //System.out.println("NetMessageProtocol: sendMessage: (messageBytes == null): returning false");
	  System.out.println("Error code NMP-3");
	  return false;
      }
      if (messageAndHeaderBytes.length == 0) {
	  //System.out.println("NetMessageProtocol: sendMessage: (messageBytes.length == 0): returning false");
	  System.out.println("Error code NMP-4");
	  return false;
      }

      // the first parameter, "1", should be messageXml.length(), but that parameter is not used
      if ( ! ( writeNetwork(1, messageAndHeaderBytes) ) ) {
	  //System.out.println("NetMessageProtocol: sendMessage: ( ! ( writeNetwork(socket, 1, messageXml.getBytes()) ) ): returning false");
	  System.out.println("Error code NMP-5");
	  return false;
      }

      return true;
    }

    public boolean isSocketConnected(Socket socket) {
      if (mySocket != socket) {
	try {
	  myOutputStream = socket.getOutputStream();
	  myInputStream = socket.getInputStream();
	  mySocket = socket;
	}
	catch (Exception e1) {
	  closeSocket(socket);
	  myOutputStream = null;
	  myInputStream = null;
	  mySocket = null;
	}
      }
      return isConnected();
    }

    public boolean sendMessage(Socket socket, Message message) {

	if (mySocket != socket) {
	    try {
		myOutputStream = socket.getOutputStream();
		myInputStream = socket.getInputStream();
		mySocket = socket;
	    } catch(Exception e1) {
		closeSocket(socket);
		myOutputStream = null;
		myInputStream = null;
		mySocket = null;
	    }
	}

	if (!isConnected())
	  return false;

	int binChunkCount = message.getBinaryChunkCount();
	int binSize = 0;
	int[] bufSizes = new int[binChunkCount];

	if ( (binChunkCount == 1) && (message.type.equalsIgnoreCase("BinaryOnly")) )
	  return sendBinaryOnlyMessage(socket, message);
	else if (binChunkCount > 0) {
	  int subBinSize = 0;
	  for (int i = 0; i < binChunkCount; i++) {
	    bufSizes[i] = message.getBinarySize(i);
	    binSize += bufSizes[i];
	  }
	  if (bufSizes.length > 0) {
	    message.setOther("BinarySizes", Utils.printListLine(bufSizes, ","));
	    message.setOther("TotalBinarySize", "" + binSize);
	  }
	  else {
	    binChunkCount = 0;
	    binSize = 0;
	  }
	}

	String messageXml = message.toXML();
	byte[] buffer = getMessageAndHeaderBytes(messageXml, "Message", binSize);

	fillInAnyBinaryData(message, buffer, 12+messageXml.length(), bufSizes);

	// a series of quick checks that we have all we need to write to the socket
	if (socket == null) {
	    System.out.println("NetMessageProtocol: sendMessage: (socket == null): returning false");
	    System.out.println("Error code NMP-2");
	    return false;
	}
	if (buffer == null) {
	    System.out.println("NetMessageProtocol: sendMessage: (messageBytes == null): returning false");
	    System.out.println("Error code NMP-3");
	    return false;
	}
	if (buffer.length == 0) {
	    System.out.println("NetMessageProtocol: sendMessage: (messageBytes.length == 0): returning false");
	    System.out.println("Error code NMP-4");
	    return false;
	}

	// the first parameter, "1", should be messageXml.length(), but that parameter is not used
	if ( ! ( writeNetwork(1, buffer) ) ) {
	    System.out.println("NetMessageProtocol: sendMessage: ( ! ( writeNetwork(socket, 1, messageXml.getBytes()) ) ): returning false");
	    System.out.println("Error code NMP-5");
	    return false;
	}

	// System.out.println("NetMessageProtocol: sendMessage: Success");
	return true;
    }









    public byte[] getMessageAndHeaderBytes(String messageXml, String headline, int extraspace) {
	byte[] header = new byte[12+messageXml.length()+extraspace];
	int len = headline.length();
	if (len > 7) len = 7;
	headline.getBytes(0, len, header, 0);

	byte zeroByte = 0;
	header[7] = zeroByte;

	int XmlLength = messageXml.length();

	byte[] fourByteInt = int2bytes(XmlLength);
	header[8] = fourByteInt[0];
	header[9] = fourByteInt[1];
	header[10] = fourByteInt[2];
	header[11] = fourByteInt[3];

	System.arraycopy(messageXml.getBytes(), 0, header, 12, messageXml.length());

	return header;
    }

    // wait for up to timeout ms for the socket to produce a valid message string, then return a Message object
    public Message receiveMessage(Socket socket, int timeout) {

      lastError = "";
	if (mySocket != socket) {
	    try {
//	      System.out.println("Change Socket: " + System.currentTimeMillis());
		myOutputStream = socket.getOutputStream();
		myInputStream = socket.getInputStream();
		mySocket = socket;
	    } catch(Exception e1) {
	      closeSocket(socket);
		myOutputStream = null;
		myInputStream = null;
		mySocket = null;
		socket = null;
	    }
	}

	if (!isConnected())
	  return null;

	byte[] socketHeader = readNetwork(12, timeout);

	if (socketHeader == null) {
	  closeSocket(socket);
	  myOutputStream = null;
	  myInputStream = null;
	  mySocket = null;
	  return null;
	}

	// checks that the header is valid
	if (socketHeader.length == 0) {
	    // TODO - this message should be shown in a state of error, but there are cases where this is acceptible behavior - what to do?
	    //System.out.println("NetMessageProtocol: recieveMessage: socketHeader.length == 0");
	    return null;
	}
	if (socketHeader.length < 12) {
	    System.out.println("NetMessageProtocol: recieveMessage: socketHeader.length < 12");
	    this.closeSocket(socket);
	    System.out.println("Error code NMP-7");
	    return null;
	}

	int numBytesToRead = readIntFromBytes(socketHeader, 8);

	String str = readStringFromBytes(socketHeader, 0, 8);
	if (str.equalsIgnoreCase("BinaryM")) {
	  return receiveBinaryOnly(numBytesToRead);
	}
	else if (!str.equalsIgnoreCase("Message")) {
	  lastError = "Received something other than message: '" + str + "'";
	  System.out.println(lastError);
	  this.closeSocket(socket);
	  myOutputStream = null;
	  myInputStream = null;
	  mySocket = null;
	  return null;
	}

	// System.out.println("Protocol: Receiving " + numBytesToRead + " bytes...");
	// byte[] socketData = readNetwork(numBytesToRead, timeout);
	byte[] socketData = readNetwork(numBytesToRead, 3000);

	Message message = null;
	if (socketData == null) {
	  this.closeSocket(socket);
	  myOutputStream = null;
	  myInputStream = null;
	  mySocket = null;
	  return null;
	}
	else if (socketData.length < numBytesToRead) {
	  Time now = new Time();
	  lastError = "Only read partial message: " + socketData.length + " out of " +  numBytesToRead + " (" + now.printTime() + ")";
	  System.out.println(lastError);
	  this.closeSocket(socket);
	  myOutputStream = null;
	  myInputStream = null;
	  mySocket = null;
	  return null;
	}
	else if (socketData.length > numBytesToRead) {
	  System.out.println("Received more than the full message: " + socketData.length + " out of " +  numBytesToRead + "");
	  String dataString = new String(socketData).trim();
	  message = new Message(dataString);
	}
	else {
	    String dataString = new String(socketData).trim();
	    message = new Message(dataString);
	}
	// System.out.println("Protocol: Receiving Message [" + message.type + "]...");

	String binString = message.getOther("BinarySizes");
	if (binString.length() == 0) {
	  return message;
	}

	TextCollection binSizes = Utils.splitString(binString, ",");

	// It would seem we have binary data to read...
	long totalBinSize = Utils.str2long(message.getOther("TotalBinarySize"));
	// printf("BinaryRec: %d chunks %ld bytes\n", binSizes.getCount(), totalBinSize);

	byte[] buffer;
	int subBinSize;
	for (int n = 0; n < binSizes.getCount(); n++) {
	  subBinSize = Utils.str2int((String)binSizes.get(n));
	  if (subBinSize <= 0) {
	    // Error...
	  }
	  else {
	    buffer = readNetwork(subBinSize, 10000);
	    if (buffer.length != subBinSize) {
	      buffer = null;
	      // Error...
	    }
	    else {
	      //	printf("BinRec [%d]: [%c%c%c%c%c%c] [%d]\n", n, buffer[0], buffer[1], buffer[2], buffer[3], buffer[4], buffer[5], subBinSize);
	      if (!message.fromBinaryBuffer(n, buffer, 0, subBinSize)) {
		// Error...
	      }
	      buffer = null;
	    }
	  }
	}

	return message;
    }


    boolean sendBinaryOnlyMessage(Socket socket, Message message) {

      DataSample sample = (DataSample) message.object;
      if (sample == null) {
	      return false;
      }

      int binSize = sample.getBinarySize(0);
      byte[] buffer = getMessageAndHeaderBytes("", "BinaryM", binSize);

      if (sample.toBinaryBuffer(0, buffer, 12, binSize) != binSize) {
	// Error...
	return false;
      }

      // a series of quick checks that we have all we need to write to the socket
      if (socket == null) {
	  System.out.println("NetMessageProtocol: sendMessage: (socket == null): returning false");
	  System.out.println("Error code NMP-2");
	  return false;
      }
      if (buffer == null) {
	  System.out.println("NetMessageProtocol: sendMessage: (messageBytes == null): returning false");
	  System.out.println("Error code NMP-3");
	  return false;
      }
      if (buffer.length == 0) {
	  System.out.println("NetMessageProtocol: sendMessage: (messageBytes.length == 0): returning false");
	  System.out.println("Error code NMP-4");
	  return false;
      }

      // the first parameter, "1", should be messageXml.length(), but that parameter is not used
      if ( ! ( writeNetwork(1, buffer) ) ) {
	  System.out.println("NetMessageProtocol: sendMessage: ( ! ( writeNetwork(socket, 1, messageXml.getBytes()) ) ): returning false");
	  System.out.println("Error code NMP-5");
	  return false;
      }

      // System.out.println("NetMessageProtocol: sendMessage: Success");
      return true;
    }

    Message receiveBinaryOnly(int len) {
      Message msg = null;
      byte[] buffer = readNetwork(len, 10000);
      if ( (buffer == null) || (buffer.length != len) ) {
	buffer = null;
	return new Message("" ,"", "BINARY_RECEIVE_FAILED");
      }
      else {
	DataSample sample = new DataSample();
	if (!sample.fromBinaryBuffer(0, buffer, 0, len)) {
	  buffer = null;
	  sample = null;
	  return new Message("" ,"", "BINARY_RECEIVE_FAILED");
	}
	buffer = null;
	msg = new Message("" ,"", "BinaryOnly");
	msg.object = sample;
      }
      return msg;
    }

    boolean fillInAnyBinaryData(Message message, byte[] buffer, int offset, int[] bufSizes) {
      int bytesSoFar = offset;
      for (int n=0; n<bufSizes.length; n++) {
	message.toBinaryBuffer(n, buffer, bytesSoFar, bufSizes[n]);
	bytesSoFar += bufSizes[n];
      }
      return true;
    }












































    // ###################################################################
    //   Network Functions
    // ###################################################################

   public byte[] readNetwork(int len, int timeout) {
	long beginReadNetTime = System.currentTimeMillis();

	if ((myInputStream == null) || (len <= 0)) {
	    System.out.println("Socket NULL");
	    return null;
	}

	int lengthSoFar = 0;
	int ret = 0;

	byte arr[] = new byte[len];
	byte nonread[] = new byte[0];

	long timeNow = System.currentTimeMillis();

	while (lengthSoFar < len) {

	    if ((System.currentTimeMillis() - timeNow > timeout) && (ret == 0)) {
	      if (lengthSoFar == 0)
		return nonread;
	      byte arr2[] = new byte[lengthSoFar];
	      System.arraycopy(arr, 0, arr2, 0, lengthSoFar);
	      arr = null;
	      return arr2;
	    }

	    int tmout = -1;
	    try {
	      mySocket.setSoTimeout(timeout);
	      ret = myInputStream.read(arr, lengthSoFar, len-lengthSoFar);
	    }
//	    catch (SocketTimeoutException ste) {
//	      System.out.println("Socket Timeout: " + System.currentTimeMillis() + " " + timeout);
//	      ret = 0;
//	    }
	    catch (InterruptedIOException ioe) {
	    //  System.out.println("Socket Read Interrupted: " + ioe.toString());
	      ret = 0;
	      /*
	       System.out.println("Socket Read Interrupted: " + ioe.toString());
			       System.out.println("Error code NMP-8");
			       if (lengthSoFar == 0)
		return nonread;*/
	      /*  byte arr2[] = new byte[lengthSoFar];
		for (int n = 0; n < lengthSoFar; n++)
		  arr2[n] = arr[n];
		return arr2;*/
	    }
	    catch (SocketException se) {
	      // System.out.println("Socket Read Exception: " + se.toString());
	      closeSocket(mySocket);
	      return null;
	    }
	    catch (Exception e) {
	      System.out.println("Socket Read Exception: " + e.toString());
	      closeSocket(mySocket);
	      // System.out.println("Error code NMP-9");
	      return null;
	    }

	    if ((ret+lengthSoFar) == len)
	      return arr;

	    /*                lengthSoFar = ret;
	      if(avail > 0)
	      {
		  if(avail > (len - lengthSoFar))
	       avail = len - lengthSoFar;

		  ret = myInputStream.read(arr, lengthSoFar, avail);
	      }

	     */
	    if (ret > 0) {
	      lengthSoFar += ret;
	    }
	    else if (ret < 0) {
	      // System.out.println("Error code NMP-9");
	      return null;
	    }
	    else {
	      // pause(10);
	    }
	}
	return arr;
   }

   public boolean writeNetwork(int len, byte[] arr) {
	if ((myOutputStream == null) || (len <= 0) || (arr == null) || (arr.length == 0)) {
	    return false;
	}

	try {
	    myOutputStream.write(arr);
	}
	catch(Exception e) {
	  System.out.println("writeNetwork except: " + e.toString());
	  closeSocket(mySocket);
	  return false;
	}

	return true;
   }

   public boolean isConnected() {
     if (mySocket == null)
       return false;
     try {
       //if (mySocket.isClosed())
	 //return false;
       int n = myInputStream.available();
       n = (int) myInputStream.skip(0);
       n = mySocket.getPort();
       InetAddress ia = mySocket.getInetAddress();
       if (ia == null) {
	 System.out.println("No longer connected...");
	 return false;
       }
       if (ia.getHostAddress().length() == 0) {
	 System.out.println("No longer connected...");
	 return false;
       }
     }
     catch (Exception ee) {
       System.out.println("No longer connected...");
       return false;
     }
     return true;
   }

   public boolean closeSocket(Socket socket) {
     if (socket == null)
       return true;
     try {
       socket.close();
     }
     catch (Exception ee) {
       return false;
     }
     return true;
   }

    // Util functions

    public static void pause(int msec) {
	Thread thisThread;
	thisThread = Thread.currentThread();
	try { thisThread.sleep(msec); } catch (InterruptedException e){}
    }

    public static byte[] int2bytes(int val) {

	byte arr[] = new byte[4];
	arr[0] = (byte) val;
	arr[1] = (byte) (val >>> 8);
	arr[2] = (byte) (val >>> 16);
	arr[3] = (byte) (val >>> 24);
	return arr;
    }

    public static int readIntFromBytes(byte[] arr, int pos) {
	int i = ((arr[pos] & 0xFF) << 0)
	    +   ((arr[pos+1] & 0xFF) << 8)
	    +   ((arr[pos+2] & 0xFF) << 16)
	    +   ((arr[pos+3] & 0xFF) << 24);
	return i;
    }

    public static String readStringFromBytes(byte[] arr, int pos, int len) {
      if (arr.length < pos+len)
	return "";
      byte arr2[] = new byte[len];
      System.arraycopy(arr, pos, arr2, 0, len);
      String str = new String(arr2).trim();
      return str;
    }

    public String getLastError() {
      return lastError;
    }






}
