/***************************** License **********************************

    Copyright(c)2005, Communicative Machines
    All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are 
met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright 
  notice, this list of conditions and the following disclaimer in the 
  documentation and/or other materials provided with the distribution.

- Neither the name of the copyright holders nor the names of its 
  contributors may be used to endorse or promote products derived from 
  this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

************************************************************************/
// Component.java


package com.cmlabs.air;

import java.util.*;
import org.w3c.dom.*;

public class ObjectCollection extends BaseObject {

  String globaltype = "orderedobjectcollection";
  String type = "Object";
  private Vector vector = new Vector();

  public ObjectCollection() {
  }

  public ObjectCollection(String xml) {
    fromXML(Utils.xmlGetRootNodeFromXML(xml));
  }

  public ObjectCollection(XMLElement node) {
    fromXML(node);
  }

  public Object clone() {
    return null;
  }

  public int getPos(Object obj) {
    return vector.indexOf(obj);
  }

  public int getCount() {
    return vector.size();
  }

  public Object get(int pos) {
    try {
      if (getCount() > pos)
	return vector.elementAt(pos);
    }
    catch(Exception e) {}
    return null;
  }

  public Object getFirst() {
    return get(0);
  }

  public Object getLast() {
    return get(getCount()-1);
  }

  public boolean removeAll() {
    vector.removeAllElements();
    return true;
  }

  public boolean remove(Object obj) {
    try {
      return vector.removeElement(obj);
    }
    catch (Exception e) {}
    return false;
  }

  public boolean remove(int pos) {
    try {
      vector.removeElementAt(pos);
      return true;
    }
    catch (Exception e) {}
    return false;
  }

  public boolean addAll(ObjectCollection col) {
    for (int n=0; n<col.getCount(); n++) {
      add(col.get(n));
    }
    return true;
  }

  public boolean add(Object obj) {
    vector.addElement(obj);
    return true;
  }

  public boolean addFirst(Object obj) {
    vector.insertElementAt(obj, 0);
    return true;
  }

  public boolean addLast(Object obj) {
    vector.addElement(obj);
    return true;
  }

  public boolean add(int pos, Object obj) {
    if (getCount() < pos)
      vector.addElement(obj);
    vector.insertElementAt(obj, pos);
    return true;
  }

  public boolean replace(Object obj1, Object obj2) {
    int pos = getPos(obj1);
    if (pos < 0)
      return false;
    try {
      vector.setElementAt(obj2, pos);
      return true;
    }
    catch (Exception e) {}
    return false;
  }

  public boolean contains(Object obj) {
    return (getPos(obj) >= 0);
  }









  public String toXML() {

    String xml = "";

    Object key;
    Object obj;
    AIRBase baseobj;
    String value;
    Iterator i;

    for (int n=0; n<getCount(); n++) {
      obj = get(n);
      if (BaseObject.isBaseObject(obj)) {
	baseobj = (AIRBase) obj;
	value = baseobj.toXML();
	xml += "<object>\n" + Utils.xmlIndent(value) + "</object>\n";
      }
    }

    return "<objectcollection>\n" + Utils.xmlIndent(xml) + "</objectcollection>\n";
  }


  public boolean fromXML(XMLElement node) {

    if (node == null) return false;

    Node subnode;
    String entry;
    String key;
    Object obj;
    Hashtable attr;

    String nodename = node.getName();
    if ( (nodename == null) || (!nodename.equalsIgnoreCase("objectcollection")))
      return false;

    XMLElement child, sub;
    for (Enumeration e=node.enumerateChildren(); e.hasMoreElements(); ) {
      child = (XMLElement) e.nextElement();
      if (child != null) {
	entry = child.getName();
	if ( (entry.equalsIgnoreCase("object")) && (child.hasChildren())) {
	  obj = null;
	  sub = (XMLElement) child.getChildren().elementAt(0);
	  obj = BaseObject.createFromXML(sub);
	  if (obj == null)
	    obj = new RawXML(sub.toString());
	  if (obj != null)
	    vector.addElement(obj);
	}
      }
    }

    return true;
  }

  public int getBinarySize(int chunk) {

    int size = 0;
    AIRBase baseobj;

    ObjectCollection binObjects = getBinaryDataObjects();

    if (chunk < 0) {
      for (int n=0; n<binObjects.getCount(); n++) {
	baseobj = (AIRBase) binObjects.get(n);
	if (baseobj != null)
	  size += baseobj.getBinarySize(0);
      }
    }
    else {
      baseobj = (AIRBase) binObjects.get(chunk);
      if (baseobj != null)
	size = baseobj.getBinarySize(0);
    }

    return size;
  }

  public int getBinaryChunkCount() {
    return getBinaryDataObjects().getCount();
  }

  public int toBinaryBuffer(int chunk, byte[] data, int offset, int maxlen) {
    ObjectCollection binObjects = getBinaryDataObjects();
    AIRBase baseobj = (AIRBase) binObjects.get(chunk);
    if (baseobj != null)
      return baseobj.toBinaryBuffer(0, data, offset, maxlen);
    else
      return 0;
  }

  public boolean fromBinaryBuffer(int chunk, byte[] data, int offset, int len) {
    ObjectCollection binObjects = getBinaryDataObjects();
    AIRBase baseobj = (AIRBase) binObjects.get(chunk);
    if (baseobj != null)
      return baseobj.fromBinaryBuffer(0, data, offset, len);
    else
      return false;
  }

  public ObjectCollection getBinaryDataObjects() {

    ObjectCollection binObjects = new ObjectCollection();
    Object obj;
    AIRBase baseobj;

    for (int n=0; n<getCount(); n++) {
      obj = get(n);
      if (BaseObject.isBaseObject(obj)) {
	baseobj = (AIRBase) obj;
	if ( baseobj.getBinaryChunkCount() > 0 )
	  binObjects.add(baseobj);
      }
    }

    return binObjects;
  }



}
