/***************************** License **********************************

    Copyright(c)2005, Communicative Machines
    All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are 
met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright 
  notice, this list of conditions and the following disclaimer in the 
  documentation and/or other materials provided with the distribution.

- Neither the name of the copyright holders nor the names of its 
  contributors may be used to endorse or promote products derived from 
  this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

************************************************************************/
// Component.java


package com.cmlabs.air;

import java.util.*;
import org.w3c.dom.*;

public class ObjectDictionary extends BaseObject {

  String globaltype = "objectdictionary";
  String type = "Object";
  Vector elems = new Vector();
  Hashtable table = new Hashtable();

  public ObjectDictionary() {
  }

  public ObjectDictionary(String xml) {
    fromXML(Utils.xmlGetRootNodeFromXML(xml));
  }

  public ObjectDictionary(XMLElement node) {
    fromXML(node);
  }

  public Object clone() {
    return null;
  }

  public boolean addAll(ObjectDictionary col) {
    for (int n=0; n<col.getCount(); n++) {
      put(col.getKey(n), col.get(n));
    }
    return true;
  }

  public boolean put(Object key, Object val) {
    if (!elems.contains(key)) {
      elems.addElement(key);
    }
    return (table.put(key, val) != null);
  }

  public Object get(Object obj) {
    return table.get(obj);
  }

  public Object get(int pos) {
    try {
      Object val = elems.elementAt(pos);
      if (val == null)
        return null;
      return table.get(val);
    }
    catch(Exception e) { return null; }
  }

  public String getKey(int pos) {
    try {
      Object val = elems.elementAt(pos);
      if (val == null)
        return "";
      return val.toString();
    }
    catch(Exception e) {
      return "";
    }
  }

  public Object getKeyObject(int pos) {
    try {
      Object val = elems.elementAt(pos);
      if (val == null)
        return null;
      return val;
    }
    catch(Exception e) {
      return null;
    }
  }

  public TextCollection getAllKeys() {
    TextCollection col = new TextCollection();
    for (Enumeration e = elems.elements(); e.hasMoreElements(); ) {
      col.add(e.nextElement().toString());
    }
    return col;
  }

  public ObjectCollection getAllObjects() {
    ObjectCollection col = new ObjectCollection();
    for (int n=0; n<this.getCount(); n++) {
      col.add(get(n));
    }
    return col;
  }

  public Object getFirst() {
    return get(0);
  }

  public Object getLast() {
    return get(getCount()-1);
  }

  public int getPos(Object obj) {
    return elems.indexOf(obj);
  }

  public int getCount() {
    return elems.size();
  }

  public String getFirstKey() {
    return getKey(0);
  }

  public String getLastKey() {
    return getKey(getCount()-1);
  }

  public Object getFirstKeyObject() {
    return getKeyObject(0);
  }

  public Object getLastKeyObject() {
    return getKeyObject(getCount()-1);
  }

  public boolean removeAll() {
    elems.removeAllElements();
    table.clear();
    return true;
  }

  public boolean remove(Object key) {
    int pos = elems.indexOf(key);
    if (pos < 0)
      return false;
    elems.removeElementAt(pos);
    return (table.remove(key) != null);
  }

  public boolean remove(int pos) {
    return remove(getKey(pos));
  }

  public boolean containsKey(Object key) {
    return elems.contains(key);
  }

  public boolean contains(Object obj) {
    return table.containsValue(obj);
  }













  public String toXML() {

    String xml = "";

    Object key;
    Object obj;
    AIRBase baseobj;
    String value;

    for (int n=0; n<this.getCount(); n++) {
      key = getKey(n);
      obj = get(n);
      if ( (key != null) && (BaseObject.isBaseObject(obj)) ) {
	baseobj = (AIRBase) obj;
	value = baseobj.toXML();
	xml += "<object name=\"" + Utils.xmlStringEncode(key.toString()) + "\">\n" + Utils.xmlIndent(value) + "</object>\n";
      }
    }

    return "<objectdictionary>\n" + Utils.xmlIndent(xml) + "</objectdictionary>\n";
  }





  public boolean fromXML(XMLElement node) {

    if (node == null) return false;

    Node subnode;
    String entry;
    String key;
    Object obj;
    Hashtable attr;

    String nodename = node.getName();
    if ( (nodename == null) || (!nodename.equalsIgnoreCase("objectdictionary")))
      return false;

    XMLElement child, sub;
    for (Enumeration e=node.enumerateChildren(); e.hasMoreElements(); ) {
      child = (XMLElement) e.nextElement();
      if (child != null) {
	entry = child.getName();
	if ( (entry.equalsIgnoreCase("object")) && (child.hasChildren())) {
	  if (child.hasAttribute("name")) {
	    key = child.getStringAttribute("name");
	    obj = null;
	    sub = (XMLElement) child.getChildren().elementAt(0);
	    obj = BaseObject.createFromXML(sub);
	    if (obj == null)
	      obj = new RawXML(sub.toString());
	    if (obj != null)
	      this.put(key, obj);
	  }
	}
      }
    }

    return true;
  }


  public String printListLine(String sep, String eq) {

    String str = "";

    if (getCount() == 0)
      return "";
    else if (getCount() == 1)
      return getKey(0).toString() + eq + get(0).toString();

    for (int n=0; n<getCount()-1; n++) {
      str += getKey(n).toString() + eq + get(n).toString() + sep;
    }
    int nn = getCount()-1;
    str += getKey(nn).toString() + eq + get(nn).toString();

    return str;
  }











  public int getBinarySize(int chunk) {

    int size = 0;
    AIRBase baseobj;

    ObjectCollection binObjects = getBinaryDataObjects();

    if (chunk < 0) {
      for (int n=0; n<binObjects.getCount(); n++) {
	baseobj = (AIRBase) binObjects.get(n);
	if (baseobj != null)
	  size += baseobj.getBinarySize(0);
      }
    }
    else {
      baseobj = (AIRBase) binObjects.get(chunk);
      if (baseobj != null)
	size = baseobj.getBinarySize(0);
    }

    return size;
  }

  public int getBinaryChunkCount() {
    return getBinaryDataObjects().getCount();
  }

  public int toBinaryBuffer(int chunk, byte[] data, int offset, int maxlen) {
    ObjectCollection binObjects = getBinaryDataObjects();
    AIRBase baseobj = (AIRBase) binObjects.get(chunk);
    if (baseobj != null)
      return baseobj.toBinaryBuffer(0, data, offset, maxlen);
    else
      return 0;
  }

  public boolean fromBinaryBuffer(int chunk, byte[] data, int offset, int len) {
    ObjectCollection binObjects = getBinaryDataObjects();
    AIRBase baseobj = (AIRBase) binObjects.get(chunk);
    if (baseobj != null)
      return baseobj.fromBinaryBuffer(0, data, offset, len);
    else
      return false;
  }




  public ObjectCollection getBinaryDataObjects() {

    ObjectCollection binObjects = new ObjectCollection();
    Object obj;
    Object key;
    AIRBase baseobj;

    for (int n=0; n<this.getCount(); n++) {
      key = getKey(n);
      obj = get(n);
      if (BaseObject.isBaseObject(obj)) {
	baseobj = (AIRBase) obj;
	if ( baseobj.getBinaryChunkCount() > 0 )
	  binObjects.add(baseobj);
      }
    }
    return binObjects;
  }

}












