/***************************** License **********************************

    Copyright(c)2005, Communicative Machines
    All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are 
met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright 
  notice, this list of conditions and the following disclaimer in the 
  documentation and/or other materials provided with the distribution.

- Neither the name of the copyright holders nor the names of its 
  contributors may be used to endorse or promote products derived from 
  this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

************************************************************************/
// Component.java


package com.cmlabs.air;

import java.util.*;
import org.w3c.dom.*;

public class PerfStat extends BaseObject {

  public String name = "";
  public Time created = new Time();
  public Time lastUpdated = new Time();

  public long runCount = 0;
  public long msgMax = 0;
  public long msgStored = 0;
  public long msgReceived = 0;
  public long msgPosted = 0;
  public long msgSent = 0;
  public long msgWoken = 0;
  public long msgProcessed = 0;

  public double totalRunSeconds = 0;
  public double totalCPUSeconds = 0;
  public double deltaSeconds = 0;
  public double deltaRunSeconds = 0;
  public double deltaCPUSeconds = 0;
  public double currentCPUUsage = 0;
  public double currentSystemUsage = 0;
  public double averageSystemUsage = 0;
  public double totalSystemUsage = 0;

  public long streamsCount = 0;
  public long streamsTotalSize = 0;
  public double streamTotalInRate = 0;
  public double streamTotalOutRate = 0;
  public double streamsTotalSizeEver = 0;
  public Message lastMessageTrigger = null;
  public Message lastMessagePosted = null;


  public PerfStat() {
  }

  public PerfStat(XMLElement node) {
    fromXML(node);
  }

  public PerfStat(String xml) {
    fromXML(xml);
  }

  public PerfStat(byte[] data) {
    fromBinary(data);
  }

  public boolean fromXML(String xml) {
    return fromXML(Utils.xmlGetRootNodeFromXML(xml));
  }

  public boolean fromXML(XMLElement node) {

    if (node == null)
      return false;

    String nodename = node.getName();
    if ( (nodename == null) || (!nodename.equalsIgnoreCase("perfstat")))
      return false;

    name = node.getStringAttribute("name");
    runCount = node.getIntAttribute("count");
    msgMax = node.getIntAttribute("max");
    msgStored = node.getIntAttribute("stored");
    msgSent = node.getIntAttribute("sent");
    msgWoken = node.getIntAttribute("woken");
    msgProcessed = node.getIntAttribute("processed");
    msgReceived = node.getIntAttribute("received");
    msgPosted = node.getIntAttribute("posted");

    streamsCount = node.getIntAttribute("streamscount");
    streamsTotalSize = node.getIntAttribute("streamssize");
    streamTotalInRate = node.getDoubleAttribute("streamsratein");
    streamTotalOutRate = node.getDoubleAttribute("streamsrateout");
    streamsTotalSizeEver = node.getDoubleAttribute("streamsever");

    currentCPUUsage = node.getDoubleAttribute("currentcpu");
    totalRunSeconds = node.getDoubleAttribute("totalrun");
    totalCPUSeconds = node.getDoubleAttribute("totalcpu");
    deltaSeconds = node.getDoubleAttribute("delta");
    deltaRunSeconds = node.getDoubleAttribute("deltarun");
    deltaCPUSeconds = node.getDoubleAttribute("deltacpu");
    currentSystemUsage = node.getDoubleAttribute("currentsystem");
    averageSystemUsage = node.getDoubleAttribute("averagesystem");
    totalSystemUsage = node.getDoubleAttribute("totalsystem");

    XMLElement sub = (XMLElement) node.getChildren().elementAt(0);
    if (sub != null)
      created = new Time(sub);
    sub = (XMLElement) node.getChildren().elementAt(1);
    if (sub != null)
      lastUpdated = new Time(sub);
    if (node.getChildren().size() > 2) {
      sub = (XMLElement) node.getChildren().elementAt(2);
      if (sub != null)
        lastMessageTrigger = new Message(sub);
    }
    if (node.getChildren().size() > 3) {
      sub = (XMLElement) node.getChildren().elementAt(3);
      if (sub != null)
        lastMessagePosted = new Message(sub);
    }

    return true;
  }

  public String toXML() {

    String subxml = "";
    if (lastMessageTrigger != null)
      subxml = Utils.xmlIndent(lastMessageTrigger.toXML());
    if (lastMessagePosted != null) {
      if (subxml.length() == 0) {
        Message msg = new Message();
        subxml = Utils.xmlIndent(msg.toXML());
      }
      subxml += Utils.xmlIndent(lastMessagePosted.toXML());
    }

    String xml = "<perfstat name=\"" + Utils.xmlStringEncode(name) + "\" count=\"" + runCount + "\" received=\"" + msgReceived + "\" posted=\"" + msgPosted;
    xml += "\" max=\"" + msgMax + "\" stored=\"" + msgStored + "\" sent=\"" + msgSent + "\" woken=\"" + msgWoken + "\" processed=\"" + msgProcessed;
    xml += "\" streamscount=\"" + streamsCount + "\" streamssize=\"" + streamsTotalSize + "\" streamsratein=\"" + streamTotalInRate + "\" streamsrateout=\"" + streamTotalOutRate + "\" streamsever=\"" + streamsTotalSizeEver;
    xml += "\" currentcpu=\"" + currentCPUUsage + "\" totalrun=\"" + totalRunSeconds + "\" totalcpu=\"" + totalCPUSeconds;
    xml += "\" delta=\"" + deltaSeconds + "\" deltarun=\"" + deltaRunSeconds + "\" deltacpu=\"" + deltaCPUSeconds;
    xml += "\" currentsystem=\"" + currentSystemUsage + "\" averagesystem=\"" + averageSystemUsage + "\" totalsystem=\"" + totalSystemUsage + "\">\n";
    xml += Utils.xmlIndent(created.toXML()) + "\n" + Utils.xmlIndent(lastUpdated.toXML()) + "\n" + subxml + "\n</perfstat>";
    return xml;
  }

  public byte[] toBinary() {
    return toXML().getBytes();
  }

  public boolean fromBinary(byte[] data) {
    String xml = new String(data).trim();
    return fromXML(xml);
  }

  public String print() {
    Time now = new Time();
    String str = "perfstat[" + name  + "] count: " + runCount + " received: " + msgReceived +  " posted: " + msgPosted;
    str += " currentcpu: " + currentCPUUsage + " current: " + currentSystemUsage + " average: " + averageSystemUsage + " total: " + totalSystemUsage;
    return str;
  }

  public boolean isOlderThan(long ms) {
    Time now = new Time();
    return (now.fullMS - lastUpdated.fullMS > ms);
  }


}
