/***************************** License **********************************

    Copyright(c)2005, Communicative Machines
    All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are 
met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright 
  notice, this list of conditions and the following disclaimer in the 
  documentation and/or other materials provided with the distribution.

- Neither the name of the copyright holders nor the names of its 
  contributors may be used to endorse or promote products derived from 
  this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

************************************************************************/
// PostSpec.java


package com.cmlabs.air;

import java.util.*;

public class PostSpec extends BaseObject {

  public String alias = "";
  public String from = "";
  public String to = "";
  public String cc = "";
  public String phasechange = "";
  public String type = "";
  public String content = "";
  public String language = "";
  public double priority = 1;
  public boolean useInReplyTo = true;

  public PostSpec() {
  }

  public PostSpec(String xml, String globalFrom) {
    if (fromXML(Utils.xmlGetRootNodeFromXML(xml)))
      if (globalFrom.length() > 0)
        from = globalFrom;
  }

  public PostSpec(XMLElement node, String globalFrom) {
    if (fromXML(node))
      if (globalFrom.length() > 0)
        from = globalFrom;
  }

  public boolean fromXML(String xml) {
    return fromXML(Utils.xmlGetRootNodeFromXML(xml));
  }

  public boolean fromXML(XMLElement node) {

    if (node == null)
      return false;

    String nodename = node.getName();
    if ( (nodename == null) || (!nodename.equalsIgnoreCase("post")))
      return false;

    from = node.getStringAttribute("from");
    type = node.getStringAttribute("type");
    to = node.getStringAttribute("to");
    cc = node.getStringAttribute("cc");
    phasechange = node.getStringAttribute("phasechange");
    language = node.getStringAttribute("language");
    if (node.hasAttribute("alias"))
      alias = node.getStringAttribute("name");
    else if (node.hasAttribute("name"))
      alias = node.getStringAttribute("name");
    else
      alias = node.getStringAttribute("type");

    if (node.hasAttribute("priority"))
      priority = node.getDoubleAttribute("priority");

    if (node.hasAttribute("content"))
      content = node.getStringAttribute("content");
    else {
      Vector vect = node.getChildren();
      if ((vect != null) && (vect.size() > 0)) {
        XMLElement subnode = (XMLElement) vect.get(0);
        if (subnode != null) {
          language = "xml";
          content = subnode.toString();
        }
      }
    }

    if (node.getStringAttribute("useinreplyto").equalsIgnoreCase("no")) {
      useInReplyTo = false;
    }

    return true;
  }

  public String toXML() {

    String xml = "";
    if (language.equalsIgnoreCase("xml"))
      xml = content;

    String inreplytext = "";
    if (!useInReplyTo)
      inreplytext = " useinreplyto=\"no\"";

    String nameparm = "";
    if (!alias.equalsIgnoreCase(type))
      nameparm = " alias=\"" + alias + "\"";

    if (xml.length() > 0) {
      return "<post "+nameparm+"from=\"" + Utils.xmlStringEncode(from) + "\" to=\"" + Utils.xmlStringEncode(to) + "\" cc=\"" + Utils.xmlStringEncode(cc) + "\" type=\"" + Utils.xmlStringEncode(type) + "\" phasechange=\"" + Utils.xmlStringEncode(phasechange) + "\" " + inreplytext + " language=\"" + Utils.xmlStringEncode(language) + " priority=\"" + priority + "\">\n" + xml + "</post>";
    }
    else
      return "<post "+nameparm+"from=\"" + Utils.xmlStringEncode(from) + "\" to=\"" + Utils.xmlStringEncode(to) + "\" cc=\"" + Utils.xmlStringEncode(cc) + "\" type=\"" + Utils.xmlStringEncode(type) + "\" phasechange=\"" + Utils.xmlStringEncode(phasechange) + "\" " + inreplytext + " language=\"" + Utils.xmlStringEncode(language) + " priority=\"" + priority + "\" />";
  }

  public String print() {
    String inreplytext = "";
    if (!useInReplyTo)
      inreplytext = " useinreplyto=\"no\"";
    return "post from=\"" + Utils.xmlStringEncode(from) + "\" to=\"" + Utils.xmlStringEncode(to) + "\" cc=\"" + Utils.xmlStringEncode(cc) + "\" type=\"" + Utils.xmlStringEncode(type) + "\" phasechange=\"" + Utils.xmlStringEncode(phasechange) + "\" " + inreplytext + " language=\"" + Utils.xmlStringEncode(language) + "\"";
  }

  public String toHTML() {
    return print();
  }

}
