/***************************** License **********************************

    Copyright(c)2005, Communicative Machines
    All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are 
met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright 
  notice, this list of conditions and the following disclaimer in the 
  documentation and/or other materials provided with the distribution.

- Neither the name of the copyright holders nor the names of its 
  contributors may be used to endorse or promote products derived from 
  this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

************************************************************************/
// Reference.java


package com.cmlabs.air;

import java.util.*;
import org.w3c.dom.*;

public class Reference extends BaseObject {

	public String from="", id="", type="", stored="";
	public Time postedTime = null;

	//! to create empty Reference
	public Reference() {
	}

	//! to create from XML node
	public Reference(XMLElement node) {
		if (!fromXML(node)) {}
	}

	//! to create from Message
	public Reference(Message msg) {
		if (msg == null)
			return;
		id = msg.id;
		stored = msg.stored;
		type = msg.type;
		from = msg.from;
		postedTime = msg.postedTime;
	}

	public boolean fromXML(XMLElement node) {
		if (node == null)
			return false;

		String nodename = node.getName();
		if ( (nodename == null) || (!nodename.equalsIgnoreCase("reference")))
			return false;

		id = node.getStringAttribute("id");
		stored = node.getStringAttribute("stored");
		type = node.getStringAttribute("type");
		from = node.getStringAttribute("from");
                postedTime = new Time(node);
		return true;
	}

	public String toXML() {
		if (postedTime == null)
			return "<reference />";
		String str = "id=\"" + Utils.xmlStringEncode(id) + "\" stored=\"" + Utils.xmlStringEncode(stored) + "\" type=\"" + Utils.xmlStringEncode(type) + "\" from=\"" + Utils.xmlStringEncode(from) + "\"";
		return postedTime.toXML("reference", str) + "\n";
	}

	public String toString() {
	  if (postedTime == null)
			return "No Reference";
		return "Reference id=\"" + id + "\" stored=\"" + stored + "\" type=\"" + type + "\" from=\"" + from + "\" (" + postedTime.toString() + ")";
	}

}



