/***************************** License **********************************

    Copyright(c)2005, Communicative Machines
    All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are 
met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright 
  notice, this list of conditions and the following disclaimer in the 
  documentation and/or other materials provided with the distribution.

- Neither the name of the copyright holders nor the names of its 
  contributors may be used to endorse or promote products derived from 
  this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

************************************************************************/
// RetrieveSpec.java


package com.cmlabs.air;

import java.util.*;

public class RetrieveSpec extends BaseObject {

  public String from = "";
  public String sender = "";
  public String id = "";
  public String type = "";
  public String content = "";
  public String contentmatch = "";

  public Time afterTime = null;
  public Time untilTime = null;

  public int maxCount = -1;
  public int minCount = -1;
  public int latest = -1;
  public int lastMS = -1;

  public RetrieveSpec() {
  }

  public RetrieveSpec(String xml, String globalFrom) {
    if (fromXML(Utils.xmlGetRootNodeFromXML(xml)))
      if (globalFrom.length() > 0)
        from = globalFrom;
  }

  public RetrieveSpec(XMLElement node, String globalFrom) {
    if (fromXML(node))
      if (globalFrom.length() > 0)
        from = globalFrom;
  }

  public boolean fromXML(String xml) {
    return fromXML(Utils.xmlGetRootNodeFromXML(xml));
  }

  public boolean fromXML(XMLElement node) {

    if (node == null)
      return false;

    String nodename = node.getName();
    if ( (nodename == null) || (!nodename.equalsIgnoreCase("retrieve")))
      return false;

    afterTime = null;
    untilTime = null;

    from = node.getStringAttribute("from");
    sender = node.getStringAttribute("sender");
    id = node.getStringAttribute("id");
    type = node.getStringAttribute("type");
    content = node.getStringAttribute("content");
    contentmatch = node.getStringAttribute("contentmatch");

    boolean didSpecifyQuantity = false;

    if (node.hasAttribute("latest")) {
      latest = node.getIntAttribute("latest");
      didSpecifyQuantity = true;
    }
    if (node.hasAttribute("lastmsec")) {
      lastMS = node.getIntAttribute("lastmsec");
      didSpecifyQuantity = true;
    }

    String key;
    String value;
    XMLElement child;
    XMLElement sub;
    for (Enumeration e = node.enumerateChildren(); e.hasMoreElements(); ) {
      child = (XMLElement) e.nextElement();
      if (child != null) {
        key = child.getName();
        value = child.getContent();
        if (key.equalsIgnoreCase("latest")) {
          latest = Utils.str2int(value);
          didSpecifyQuantity = true;
        }
        else if (key.equalsIgnoreCase("aftertime")) {
          afterTime = new Time(value);
          didSpecifyQuantity = true;
        }
        else if (key.equalsIgnoreCase("untilTime")) {
          untilTime = new Time(value);
          didSpecifyQuantity = true;
        }
        else if (key.equalsIgnoreCase("lastmsec")) {
          lastMS = Utils.str2int(value);
          didSpecifyQuantity = true;
        }
      }
    }

    if (!didSpecifyQuantity) {
      latest = 1;
    }
    return true;
  }

  public String toXML() {
    String xml = "";

    if (latest > 0)
      xml += "<latest>" + latest + "</latest>\n";
    if (afterTime != null)
      xml += "<afterTime>" + afterTime + "</afterTime>\n";
    if (untilTime != null)
      xml += "<untilTime>" + untilTime + "</untilTime>\n";
    if (lastMS > 0)
      xml += "<lastmsec>" + lastMS + "</lastmsec>\n";

    if (xml.length() > 0)
      return "<retrieve from=\"" + Utils.xmlStringEncode(from) + "\" sender=\"" + Utils.xmlStringEncode(sender) + "\" type=\"" + Utils.xmlStringEncode(type) + "\" id=\"" + Utils.xmlStringEncode(id) + "\" content=\"" + Utils.xmlStringEncode(content) + "\" contentmatch=\"" + Utils.xmlStringEncode(contentmatch) + "\">\n" + Utils.xmlIndent(xml) + "</retrieve>";
    else
      return "<retrieve from=\"" + Utils.xmlStringEncode(from) + "\" sender=\"" + Utils.xmlStringEncode(sender) + "\" type=\"" + Utils.xmlStringEncode(type) + "\" id=\"" + Utils.xmlStringEncode(id) + "\" content=\"" + Utils.xmlStringEncode(content) + "\" contentmatch=\"" + Utils.xmlStringEncode(contentmatch) + "\" />";
  }

  public String print() {
    return "retrieve from=\"" + Utils.xmlStringEncode(from) + "\" type=\"" + Utils.xmlStringEncode(type) + "\"";
  }

  public String toHTML() {
    return print();
  }

}
