/***************************** License **********************************

    Copyright(c)2005, Communicative Machines
    All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are 
met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright 
  notice, this list of conditions and the following disclaimer in the 
  documentation and/or other materials provided with the distribution.

- Neither the name of the copyright holders nor the names of its 
  contributors may be used to endorse or promote products derived from 
  this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

************************************************************************/
// TCPLocation.java


package com.cmlabs.air;

import java.util.*;
import org.w3c.dom.*;

public class TCPLocation extends BaseObject {

  static int NO_LISTEN = -1;
  public String addr = "";
  public int port = 0;
  public String name = "";
  public String additional = "";

  public TCPLocation() {
  }

  public TCPLocation(String address, int portnum, String alias) {
    addr = address;
    port = portnum;
    name = alias;
  }

  public TCPLocation(String xml) {
    fromXML(xml);
  }

  public TCPLocation(XMLElement node) {
    fromXML(node);
  }

  public String toString() {
    return addr + ":" + port + " " + name;
  }

  public String toXML() {
    return "<tcplocation address=\"" + Utils.xmlStringEncode(addr) + "\" port=\"" + port + "\" name=\"" + Utils.xmlStringEncode(name) + "\" additional=\"" + Utils.xmlStringEncode(additional) + "\" />";
  }

  TextCollection getAllAddresses() {
    TextCollection col = new TextCollection();
    col.add(addr);
    if (additional.length() > 0)
      col.addAll(Utils.splitString(additional, ","));
    return col;
  }

  String printList() {
    if (additional.length() > 0)
      return addr + ":" + port + " (" + additional + ")";
    else
      return addr + ":" + port;
  }

  public boolean fromXML(String xml) {
    return fromXML(Utils.xmlGetRootNodeFromXML(xml));
  }

  public boolean fromXML(XMLElement node) {

    if (node == null)
      return false;

    String nodename = node.getName();
    if ( (nodename == null) || (nodename.length() == 0) || (!node.hasAttribute("address")))
      return false;

    addr = node.getStringAttribute("address");
    port = node.getIntAttribute("port");
    name = node.getStringAttribute("name");
    additional = node.getStringAttribute("additional");

    return isValid();
  }

  public boolean isValid() {
    if ( (addr.length() > 0) && ((port > 0) || (port == NO_LISTEN)) )
      return true;
    else
      return false;
  }

  public boolean isNoListen() {
    return (port == NO_LISTEN);
  }

  public boolean equals(TCPLocation loc) {
	  return ((loc.addr.equalsIgnoreCase(addr)) && (loc.port == port) && (loc.name.equalsIgnoreCase(name)));
  }
}
