/***************************** License **********************************

    Copyright(c)2005, Communicative Machines
    All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are 
met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright 
  notice, this list of conditions and the following disclaimer in the 
  documentation and/or other materials provided with the distribution.

- Neither the name of the copyright holders nor the names of its 
  contributors may be used to endorse or promote products derived from 
  this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

************************************************************************/
// Component.java


package com.cmlabs.air;

import java.util.*;
import org.w3c.dom.*;

public class TextCollection extends BaseObject {

  String globaltype = "orderedcollection";
  String type = "String";
  private Vector vector = new Vector();

  public TextCollection() {
  }

  public TextCollection(String xml) {
    fromXML(Utils.xmlGetRootNodeFromXML(xml));
  }

  public TextCollection(XMLElement node) {
    fromXML(node);
  }

  public Object clone() {
    return null;
  }

  public int getPos(String val) {
    return vector.indexOf(val);
  }

  public int getCount() {
    return vector.size();
  }

  public String get(int pos) {
    Object obj;
    try {
      obj = vector.elementAt(pos);
    }
    catch(Exception e) { return ""; }
    if (obj == null)
      return "";
    else
      return obj.toString();
  }

  public String getFirst() {
    return get(0);
  }

  public String getLast() {
    return get(getCount()-1);
  }

  public boolean removeAll() {
    vector.removeAllElements();
    return true;
  }

  public boolean remove(String val) {
    if (!contains(val))
      return false;
    try {
      return vector.removeElement(val);
    }
    catch(Exception e) {
      return false;
    }
  }

  public boolean remove(int pos) {
    if (getCount() < pos)
      return false;
    try {
      vector.removeElementAt(pos);
    }
    catch(Exception e) {
      return false;
    }
    return true;
  }

  public boolean add(String val) {
    vector.addElement(val);
    return true;
  }

  public boolean addAll(TextCollection col) {
    for (int n=0; n<col.getCount(); n++) {
      add(col.get(n));
    }
    return true;
  }

  public boolean addFirst(String val) {
    try {
      vector.insertElementAt(val, 0);
    }
    catch(Exception e) {
      return false;
    }
    return true;
  }

  public boolean addLast(String val) {
    vector.addElement(val);
    return true;
  }

  public boolean add(int pos, String val) {
    if (getCount() < pos)
      return false;
    vector.insertElementAt(val, pos);
    return true;
  }

  public boolean replace(String val1, String val2) {
    int pos = getPos(val1);
    if (pos < 0)
      return false;
    vector.setElementAt(val2, pos);
    return true;
  }

  public boolean contains(String val) {
    return (getPos(val) >= 0);
  }






  public String toXML() {

    String xml = "";

    Object key;
    Object obj;
    AIRBase baseobj;
    String value;
    Iterator i;

    for (int n=0; n<getCount(); n++) {
      obj = get(n);
      if (obj != null) {
	value = obj.toString();
	if (Utils.startsOrEndsWithWhiteSpace(value))
	  xml += "<entry value=\"" + Utils.xmlStringEncode(value) + "\" />\n";
	else
	  xml += "<entry>" + Utils.xmlStringEncode(value) + "</entry>\n";
      }
    }

    return "<collection>\n" + Utils.xmlIndent(xml) + "</collection>\n";
  }

  public boolean fromXML(XMLElement node) {

    if (node == null) return false;

    Node subnode;
    String entry;
    String key;
    String value;
    Object obj;
    Hashtable attr;

    String nodename = node.getName();
    if ( (nodename == null) || (!nodename.equalsIgnoreCase("collection")))
      return false;

    XMLElement child, sub;
    for (Enumeration e=node.enumerateChildren(); e.hasMoreElements(); ) {
      child = (XMLElement) e.nextElement();
      if (child != null) {
	entry = child.getName();
	//if ( (entry.equalsIgnoreCase("entry")) && (child.hasChildren())) {
	if (entry.equalsIgnoreCase("entry")) {
	  value = child.getContent();
	  vector.addElement(value);
	}
      }
    }

    return true;
  }




}
