/***************************** License **********************************

    Copyright(c)2005, Communicative Machines
    All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are 
met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright 
  notice, this list of conditions and the following disclaimer in the 
  documentation and/or other materials provided with the distribution.

- Neither the name of the copyright holders nor the names of its 
  contributors may be used to endorse or promote products derived from 
  this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

************************************************************************/
// Component.java


package com.cmlabs.air;

import java.util.*;
import org.w3c.dom.*;

public class TextDictionary extends BaseObject {

  String globaltype = "dictionary";
  String type = "String";
  Vector elems = new Vector();
  Hashtable table = new Hashtable();

  public TextDictionary() {
  }

  public TextDictionary(String xml) {
    fromXML(Utils.xmlGetRootNodeFromXML(xml));
  }

  public TextDictionary(XMLElement node) {
    fromXML(node);
  }

  public Object clone() {
    return null;
  }


  public boolean put(String key, String val) {
    if (!elems.contains(key)) {
      elems.addElement(key);
    }
    return (table.put(key, val) != null);
  }

  public String get(String key) {
    if (!elems.contains(key))
      return "";
    return table.get(key).toString();
  }

  public String get(int pos) {
    Object val = elems.elementAt(pos);
    if (val == null)
      return null;
    return table.get(val).toString();
  }

  public String getKey(int pos) {
    try {
      Object val = elems.elementAt(pos);
      if (val == null)
        return "";
      return val.toString();
    }
    catch(Exception e) {
      return "";
    }
  }

  public TextCollection getAllKeys() {
    TextCollection col = new TextCollection();
    for (Enumeration e = elems.elements(); e.hasMoreElements(); ) {
      col.add(e.nextElement().toString());
    }
    return col;
  }

  public String getFirst() {
    return get(0);
  }

  public String getLast() {
    return get(getCount()-1);
  }

  public int getPos(String val) {
    return elems.indexOf(val);
  }

  public int getCount() {
    return elems.size();
  }

  public String getFirstKey() {
    return getKey(0);
  }

  public String getLastKey() {
    return getKey(getCount()-1);
  }

  public boolean removeAll() {
    elems.removeAllElements();
    table.clear();
    return true;
  }

  public boolean remove(String key) {
    int pos = elems.indexOf(key);
    if (pos < 0)
      return false;
    elems.removeElementAt(pos);
    return (table.remove(key) != null);
  }

  public boolean remove(int pos) {
    return remove(getKey(pos));
  }

  public boolean containsKey(String key) {
    return elems.contains(key);
  }

  public boolean contains(String val) {
    return table.containsValue(val);
  }

  public boolean addAll(TextDictionary col) {
    for (int n=0; n<col.getCount(); n++) {
      put(col.getKey(n), col.get(n));
    }
    return true;
  }


  public String printListLine(String sep, String eq) {

    String str = "";

    if (getCount() == 0)
      return "";
    else if (getCount() == 1)
      return getKey(0) + eq + get(0);

    for (int n=0; n<getCount()-1; n++) {
      str += getKey(n) + eq + get(n) + sep;
    }
    int nn = getCount()-1;
    str += getKey(nn) + eq + get(nn);

    return str;
  }


















  public String toXML() {

    String xml = "";

    Object key;
    Object obj;
    AIRBase baseobj;
    String value;

    for (int n=0; n<this.getCount(); n++) {
      key = getKey(n);
      obj = get(n);
      if ( (key != null) && (obj != null) ) {
	value = obj.toString();
	if (Utils.startsOrEndsWithWhiteSpace(value))
	  xml += "<entry name=\"" + Utils.xmlStringEncode(key.toString()) + "\" value=\"" + Utils.xmlStringEncode(value) + "\" />\n";
	else
	  xml += "<entry name=\"" + Utils.xmlStringEncode(key.toString()) + "\">" + Utils.xmlStringEncode(value) + "</entry>\n";
	// xml += "<entry name=\"" + key.toString() + "\">" + value + "</entry>\n";
      }
    }

    return "<dictionary>\n" + Utils.xmlIndent(xml) + "</dictionary>\n";
  }





  public boolean fromXML(XMLElement node) {

    if (node == null) return false;

    Node subnode;
    String entry;
    String key;
    String value;
    Hashtable attr;

    String nodename = node.getName();
    if ( (nodename == null) || (!nodename.equalsIgnoreCase("dictionary")))
      return false;

    XMLElement child, sub;
    for (Enumeration e=node.enumerateChildren(); e.hasMoreElements(); ) {
      child = (XMLElement) e.nextElement();
      if (child != null) {
	entry = child.getName();
	// if ( (entry.equalsIgnoreCase("entry")) && (child.hasChildren())) {
	if (entry.equalsIgnoreCase("entry")) {
	  if (child.hasAttribute("name")) {
	    key = child.getStringAttribute("name");
	    value = child.getContent();
	    if (child.hasChildren())
	      value = child.toString();
	    this.put(key, value);
	  }
	}
      }
    }

    return true;
  }







}












