/***************************** License **********************************

    Copyright(c)2005, Communicative Machines
    All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are 
met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright 
  notice, this list of conditions and the following disclaimer in the 
  documentation and/or other materials provided with the distribution.

- Neither the name of the copyright holders nor the names of its 
  contributors may be used to endorse or promote products derived from 
  this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

************************************************************************/
// Time.java


package com.cmlabs.air;

import java.util.*;
import org.w3c.dom.*;

public class Time extends BaseObject {
  public long fullMS = 0;
  public Date date = null;

  public Time() {
    fullMS = System.currentTimeMillis();
    date = new Date(fullMS);
  }

  public Time(long ms) {
    fullMS = ms;
    date = new Date(fullMS);
  }

  public Time(Date dat) {
    fullMS = dat.getTime();
    date = new Date(fullMS);
  }

  public Time(String xml) {
    fromXML(xml);
  }

  public Time(XMLElement node) {
    fromXML(node);
  }

  public long difference(Time othertime) {
    return fullMS - othertime.fullMS;
  }

  public long getAge() {
    return System.currentTimeMillis() - fullMS;
  }

  public int compareTo(Object obj) {

    if (obj.getClass().getName().indexOf(".Time") <= 0)
      return 0;

    Time t = (Time) obj;

    return (int) (fullMS - t.fullMS);
  }

  public int day() {
    return date.getDate();
  }

  public boolean equals(Object obj) {

    if (obj.getClass().getName().indexOf(".Time") <= 0)
      return false;
    Time t = (Time) obj;
    return ( (fullMS - t.fullMS) == 0);
  }

  public int hour() {
    return date.getHours();
  }

  public String intpad(int val, int dig) {
    return Utils.int2strpad(val, dig);
  }

  public int millisecond() {
    long sec = fullMS/1000;
    return (int)(fullMS - (sec*1000));
  }

  public int minute() {
    return date.getMinutes();
  }

  public int month() {
    return date.getMonth() + 1;
  }

  public String print24HourTimeString() {
    return intpad(hour(), 2) + ":" + intpad(minute(), 2) + ":" +
	intpad(second(), 2);
  }

  public String printTime() {
    return intpad(hour(), 2) + ":" + intpad(minute(), 2) + ":" +
	intpad(second(), 2) + "." + intpad(millisecond(), 3);
  }

  public String printDate() {
    return intpad(day(), 2) + "/" + intpad(month(), 2) + "/" + year();
  }

  public String printString() {

//		String str = java.text.DateFormat.getDateInstance(java.text.DateFormat.LONG).format(date) + " ";

//		str += intpad(hour(), 2) + ":" + intpad(minute(), 2) + ":" + intpad(second(), 2) + ":" + intpad(millisecond(), 3);
//		str += intpad(hour(), 2) + ":" + intpad(minute(), 2) + ":" + intpad(second(), 2) + ":" + intpad(millisecond(), 3);

    return printDate() + " " + print24HourTimeString();
  }

  public int second() {
    return date.getSeconds();
  }

  public int year() {
    return date.getYear() + 1900;
  }

  public void setByTime_t(long sec) {
    fullMS = sec * 1000;
    date = new Date(fullMS);
  }

  public String toString() {
    return printString();
  }

  public String asText() {
    return ""+fullMS;
  }

  public String toXML() {
    return toXML("time");
  }

  public String toXML(String tag) {
    return "<" + Utils.xmlStringEncode(tag) + " sec=\"" + (int)(this.fullMS/1000) + "\" msec=\"" + this.millisecond() + "\" />";
  }

  public static String toXMLInvalid(String tag) {
    return "<" + Utils.xmlStringEncode(tag) + " />";
  }

  public String toXML(String tag, String args) {
    return "<" + tag + " sec=\"" + (int)(this.fullMS/1000) + "\" msec=\"" + this.millisecond() + "\" " + args + " />";
  }

  public boolean fromXML(String xml) {
    if (Utils.looksLikeXML(xml))
      return fromXML(Utils.xmlGetRootNodeFromXML(xml));
    else if (xml.length() > 0) {
      fullMS = Utils.str2long(xml);
      date = new Date(fullMS);
      return true;
    }
    else
      return false;
  }

  public boolean fromXML(XMLElement node) {

    if (node == null)
      return false;

    String nodename = node.getName();
    if ( (nodename == null) || (nodename.length() == 0) || (!node.hasAttribute("sec")))
      return false;

    if (!node.hasAttribute("sec"))
      return false;

    long sec = node.getIntAttribute("sec");
    int ms = 0;

    if (node.hasAttribute("msec"))
	ms = node.getIntAttribute("msec");
    else if (node.hasAttribute("ms"))
	ms = node.getIntAttribute("ms");

    fullMS = sec * 1000 + ms;
    date = new Date(fullMS);
    return true;
  }

}
