/***************************** License **********************************

    Copyright(c)2005, Communicative Machines
    All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are 
met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright 
  notice, this list of conditions and the following disclaimer in the 
  documentation and/or other materials provided with the distribution.

- Neither the name of the copyright holders nor the names of its 
  contributors may be used to endorse or promote products derived from 
  this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

************************************************************************/
// TriggerSpec.java


package com.cmlabs.air;

import java.util.*;

public class TriggerSpec extends BaseObject {

  public String alias = "";
  public String from = "";
  public String type = "";
  public int delayMS = 0;

  public TriggerSpec() {
  }

  public TriggerSpec(String xml, String globalFrom) {
    if (fromXML(Utils.xmlGetRootNodeFromXML(xml)))
      if (globalFrom.length() > 0)
        from = globalFrom;
  }

  public TriggerSpec(XMLElement node, String globalFrom) {
    if (fromXML(node))
      if (globalFrom.length() > 0)
        from = globalFrom;
  }

  public boolean fromXML(String xml) {
    return fromXML(Utils.xmlGetRootNodeFromXML(xml));
  }

  public boolean fromXML(XMLElement node) {

    if (node == null)
      return false;

    String nodename = node.getName();
    if ( (nodename == null) || (!nodename.equalsIgnoreCase("trigger")))
      return false;

    from = node.getStringAttribute("from");
    type = node.getStringAttribute("type");
    delayMS = node.getIntAttribute("delay");
    if (node.hasAttribute("alias"))
      alias = node.getStringAttribute("name");
    else if (node.hasAttribute("name"))
      alias = node.getStringAttribute("name");
    else
      alias = node.getStringAttribute("type");
    return true;
  }

  public String toXML() {
    if (alias.equalsIgnoreCase(type))
      return "<trigger after=\"" + delayMS + "\" from=\"" + Utils.xmlStringEncode(from) + "\" type=\"" + Utils.xmlStringEncode(type) + "\" />";
    else
      return "<trigger alias=\"" + Utils.xmlStringEncode(alias) + "\" after=\"" + delayMS + "\" from=\"" + Utils.xmlStringEncode(from) + "\" type=\"" + Utils.xmlStringEncode(type) + "\" />";
  }

  public String print() {
    return "trigger after=\"" + delayMS + "\" from=\"" + Utils.xmlStringEncode(from) + "\" type=\"" + Utils.xmlStringEncode(type) + "\"";
  }

  public String toHTML() {
    return print();
  }

}
