/***************************** License **********************************

    Copyright(c)2005, Communicative Machines
    All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are 
met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright 
  notice, this list of conditions and the following disclaimer in the 
  documentation and/or other materials provided with the distribution.

- Neither the name of the copyright holders nor the names of its 
  contributors may be used to endorse or promote products derived from 
  this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

************************************************************************/
package com.cmlabs.air;

import java.awt.*;
import java.util.*;
import java.net.*;
import java.io.*;
import java.lang.Math;
import javax.xml.parsers.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;
import org.w3c.dom.*;
import java.io.Writer;

public class Utils {

  public Utils() {}

  // ###################################################################
  //   Utils
  // ###################################################################

  public static void pause(int msec) {
    Thread thisThread;
    thisThread = Thread.currentThread();
    try {
      thisThread.sleep(msec);
    }
    catch (InterruptedException e) {}
  }

  // ###################################################################
  //   Converters
  // ###################################################################

  public static short readShortFromBytes(byte[] arr, int pos) {
    short s = (short) ( ( (arr[pos] & 0xFF) << 0)
		       + ( (arr[pos + 1] & 0xFF) << 8));
    return s;
  }

  public static int readIntFromBytes(byte[] arr, int pos) {
    int i = ( (arr[pos] & 0xFF) << 0)
	+ ( (arr[pos + 1] & 0xFF) << 8)
	+ ( (arr[pos + 2] & 0xFF) << 16)
	+ ( (arr[pos + 3] & 0xFF) << 24);
    return i;
  }

  public static Time readTimeFromBytes(byte[] arr, int pos) {
    Time t = new Time();
    t.setByTime_t(readIntFromBytes(arr, pos));
    return t;
  }

  public static Time readTimeBFromBytes(byte[] arr, int pos) {
    long l = ( (long) readIntFromBytes(arr, pos)) * 1000;
    l += readIntFromBytes(arr, pos + 4);
    Time t = new Time(l);
    return t;
  }

  public static int readStringLengthFromBytes(byte[] arr, int pos) {
    int n = 0;
    while (arr[pos + n] != 0) n++;
    n++;
    return n;
  }

  public static String readStringFromBytes(byte[] arr, int pos, int len) {
    if (arr.length < pos+len)
      return "";
    byte arr2[] = new byte[len];
    System.arraycopy(arr, pos, arr2, 0, len);
    String str = new String(arr2).trim();
    return str;
  }

  public static byte[] short2bytes(short val) {
    byte arr[] = new byte[2];
    arr[0] = (byte) val;
    arr[1] = (byte) (val >>> 8);
    return arr;
  }

  public static int bytes2short(byte[] arr) {
    short s = (short) ( ( (arr[0] & 0xFF) << 0)
		       + ( (arr[1] & 0xFF) << 8));
    return s;
  }

  public static byte[] int2bytes(int val) {

    byte arr[] = new byte[4];
    arr[0] = (byte) val;
    arr[1] = (byte) (val >>> 8);
    arr[2] = (byte) (val >>> 16);
    arr[3] = (byte) (val >>> 24);
    return arr;
  }

  public static int bytes2int(byte[] arr) {
    int i = ( (arr[0] & 0xFF) << 0)
	+ ( (arr[1] & 0xFF) << 8)
	+ ( (arr[2] & 0xFF) << 16)
	+ ( (arr[3] & 0xFF) << 24);
    return i;
  }

  public static byte[] timebbytes(long sec, int ms) {
    byte arr[] = new byte[10];
    arr[0] = (byte) sec;
    arr[1] = (byte) (sec >>> 8);
    arr[2] = (byte) (sec >>> 16);
    arr[3] = (byte) (sec >>> 24);
    arr[4] = (byte) ms;
    arr[5] = (byte) (ms >>> 8);
    arr[6] = arr[7] = arr[8] = arr[9] = 0;
    return arr;
  }

  public static Time bytes2time(byte[] arr) {
    int i = ( (arr[0] & 0xFF) << 0)
	+ ( (arr[1] & 0xFF) << 8)
	+ ( (arr[2] & 0xFF) << 16)
	+ ( (arr[3] & 0xFF) << 24);
    int j = ( (arr[4] & 0xFF) << 0)
	+ ( (arr[5] & 0xFF) << 8);
    Time t = new Time( (long) i * 1000 + j);
    return t;
  }

  public static String int2strpad(int val, int dig) {
    String str = String.valueOf(val);
    int missing = dig - str.length();
    if (missing <= 0)return str;

    for (int n = 0; n < missing; n++)
      str = "0" + str;

    return str;
  }

  public static int str2int(String str) {
    return (int) obj2long(str);
  }

  public static double str2double(String str) {
    return obj2double(str);
  }

  public static int obj2int(Object obj) {
    return (int) obj2long(obj);
  }

  public static long str2long(String str) {
    return obj2long(str);
  }

  public static long obj2long(Object obj) {
    return (long) obj2double(obj);
  }

  public static double obj2double(Object obj) {
    if (obj == null)
      return -1;

    double m;
    try {
      m = Double.valueOf(obj.toString()).doubleValue();
    }
    catch (Exception ex) {
      m = -1;
    }
    return m;
  }

  public static Point decodeStringPoint(String str) {
    int x = -1, y = -1, n;

    if (str == "")return null;

    if ( (n = str.indexOf(",")) != -1) {
      x = str2int(str.substring(0, n).trim());
      y = str2int(str.substring(n + 1).trim());
      return new Point(x, y);
    }
    return null;
  }

  public static Dimension decodeStringDimension(String str) {
    Dimension d = new Dimension();

    Point p = decodeStringPoint(str);
    if (p == null)return null;

    d.setSize(p.x, p.y);
    return d;
  }

  public static Color decodeStringColor(String str) {
    int r = -1, g = -1, b = -1, n, m;

    if (str == "")return null;
    else if (str.equalsIgnoreCase("red"))return Color.red;
    else if (str.equalsIgnoreCase("blue"))return Color.blue;
    else if (str.equalsIgnoreCase("green"))return Color.green;
    else if (str.equalsIgnoreCase("gray"))return Color.gray;
    else if (str.equalsIgnoreCase("white"))return Color.white;
    else if (str.equalsIgnoreCase("black"))return Color.black;
    else if (str.equalsIgnoreCase("yellow"))return Color.yellow;
    else if (str.equalsIgnoreCase("cyan"))return Color.cyan;
    else {
      if ( (n = str.indexOf(",")) != -1) {
	r = str2int(str.substring(0, n).trim());
	if ( (m = str.indexOf(",", n + 1)) != -1) {
	  g = str2int(str.substring(n + 1, m).trim());
	  b = str2int(str.substring(m + 1).trim());
	}
	if ( (r != -1) && (g != -1) && (b != -1)) {
	  return (new Color(r, g, b));
	}
      }
    }

    return null;
  }

  public static Rectangle decodeStringRectangle(String str) {
    int x = -1, y = -1, w = -1, h = -1, n, m, k;

    if (str == "")return null;

    if ( (n = str.indexOf(",")) != -1) {
      x = str2int(str.substring(0, n).trim());
      if ( (m = str.indexOf(",", n + 1)) != -1) {
	y = str2int(str.substring(n + 1, m).trim());
	if ( (k = str.indexOf(",", m + 1)) != -1) {
	  w = str2int(str.substring(m + 1, k).trim());
	  h = str2int(str.substring(k + 1).trim());
	}
	if ( (x != -1) && (y != -1) && (w != -1) && (h != -1)) {
	  return (new Rectangle(x, y, w, h));
	}
      }
    }
    return null;
  }

  public static String replaceStrings(String text, Hashtable replacements) {

    String str = text;

    for (Enumeration e = replacements.keys(); e.hasMoreElements(); ) {
      Object key = e.nextElement();
      Object val = replacements.get(key);

      if ( (key.getClass().getName().equals("java.lang.String")) &&
	  (val.getClass().getName().equals("java.lang.String"))) {
	str = replaceStrings(text, (String) key, (String) val);
      }
      else {
	System.out.println("Table contains non-String values: Key: " +
			   key.getClass().getName() + " Val: " +
			   val.getClass().getName());
      }
    }
    return str;
  }

  public static String replaceStrings(String text, String oldstr, String newstr) {

    if ( (text == null) || (oldstr == null) || (newstr == null) ||
	(text.length() == 0) || (oldstr.length() == 0))
      return text;

    System.out.print("Replacing '" + oldstr + "' with '" + newstr + "'... ");

    String str = text;
    int n;
    int pos = 0;
    int oldlen = oldstr.length();
    int count = 0;

    while ( (n = str.indexOf(oldstr, pos)) >= 0) {
      pos = n + oldlen;
      str = str.substring(0, n) + newstr + str.substring(pos);
      count++;
    }
    System.out.println("   Replaced " + count + " occurences...");
    return str;
  }












  public static String bytifySize(long val) {
    return bytifySize((double)val);
  }

  public static String bytifySize(double val) {

	  double kb = 1024;
	  double mb = kb*1024;
	  double gb = mb*1024;
	  double tb = gb*1024;

	  if (val < 512)
	    return "" + val + " b";
	  else if (val < 512*kb) {
	    val = val / kb;
	    return "" + val + " kb";
	  }
	  else if (val < 512*mb) {
	    val = val / mb;
	    return "" + val + " mb";
	  }
	  else if (val < 512*mb) {
	    val = val / gb;
	    return "" + val + " gb";
	  }
	  else {
	    val = val / tb;
	    return "" + val + " tb";
	  }
  }

  public static String bytifySizes(long val1, long val2) {
	  return bytifySizes((double)val1, (double) val2);
  }

  public static String bytifySizes(double val1, double val2) {

	  double kb = 1024;
	  double mb = kb*1024;
	  double gb = mb*1024;
	  double tb = gb*1024;

	  String end;
	  double val;
	  double maximum = ((val1 > val2) ? val1 : val2);

	  if (maximum < 512) {
	    val = 1;
	    end = "b";
	  }
	  else if (maximum < 512*kb) {
	    val = kb;
	    end = "kb";
	  }
	  else if (maximum < 512*mb) {
	    val = mb;
	    end = "mb";
	  }
	  else if (maximum < 512*gb) {
	    val = gb;
	    end = "gb";
	  }
	  else {
	    val = gb;
	    end = "tb";
	  }
	  return "" + ((double)val1)/val + " / " + ((double)val2)/val + " " + end;
  }

  public static String bytifyRate(long val) {
	  return bytifyRate((double)val);
  }

  public static String bytifyRate(double val) {
    return "" + bytifySize(val) + "/s";
  }

  public static String bytifyRates(long val1, long val2) {
    return bytifyRates((double)val1, (double)val2);
  }

  public static String bytifyRates(double val1, double val2) {
    return "" + bytifySizes(val1, val2) + "/s";
  }






  // ###################################################################
  //   Web Access
  // ###################################################################

  public static URL webMakeURL(String addr) {
    String newaddr = addr;
    URL url;

    if ( (newaddr.startsWith("http://")) || (newaddr.startsWith("https://")) ||
	(newaddr.startsWith("ftp://"))) {
      // Do nothing
    }
    else
      return null;

    try {
      url = new URL(newaddr);
    }
    catch (MalformedURLException e) {
//			this.showStatus("Web address " + newaddr + " is not a valid web address!");
      return null;
    }
    return (url);
  }

  public static Image webGetImage(String addr, Component comp) {
    Image img;
    Toolkit tk;
    MediaTracker tracker;

    try {
      tk = Toolkit.getDefaultToolkit();
      tracker = new java.awt.MediaTracker(comp);
    }
    catch (Exception e) {
      return null;
    }

    URL myURL = webMakeURL(addr);
    if (myURL == null)return null;

    // java.awt.Toolkit
    try {
      // Create the link to the image
      img = tk.getImage(myURL);
      // Actually load it from the server
      tracker.addImage(img, 0); // this is the real McCoy
    }
    catch (Exception e) {
      //	showStatus("Loading of picture " + newaddr + " failed!");
      return null;
    }

    try {
      tracker.waitForID(0);
    }
    catch (InterruptedException e) {
      // showStatus("Waiting for mediatracker for picture " + newaddr + " failed!");
      return null;
    }

//		showStatus("GotWebImage: " + addr, 6);
    return img;
  }

  public static double webDownloadSpeed(String addr) {

    char buffer[] = new char[100];
    int dl;
    long length = 0;
    double speed = -1;

    try {
      URL myURL = webMakeURL(addr);
      if (myURL == null)return -1;

      URLConnection myConnection = myURL.openConnection();
      myConnection.setUseCaches(false);

      BufferedReader inStream = new BufferedReader(new InputStreamReader(
	  myConnection.getInputStream()));
      long t = System.currentTimeMillis();
      while ( (dl = inStream.read(buffer, 0, 100)) != -1) {
	length += dl;
      }
      t = System.currentTimeMillis() - t;

      if (length <= 0)
	return -1;

      if (t <= 0)
	return 999999999;

      // Calc bytes per second
      speed = (length / t) * 1000;

    }
    catch (Exception e) {
      return -1;
    }

    return speed;
  }

  public static boolean webCacheFile(String addr) {
    return (webDownloadSpeed(addr) > 0);
  }

  public static String webReadFile(String addr) {
    return webReadFile(addr, true);
  }

  public static String webReadFile(String addr, boolean useCache) {

    URL myURL;
    URLConnection myConnection;
    BufferedReader inStream;
    String inputBuffer = "";
    String inputLine = "";
//		long t = 0;
    boolean ignoreCommentLines = false;
    int nLines = 0;
    long bytesRead = 0;

    if (addr.endsWith(".xml")) ignoreCommentLines = true;

    try {
//			myURL = new URL(addr);
      myURL = webMakeURL(addr);
      myConnection = myURL.openConnection();
      myConnection.setUseCaches(useCache);

//			t = System.currentTimeMillis();
      inStream = new BufferedReader(new InputStreamReader(myConnection.
	  getInputStream()));
      while ( (inputLine = inStream.readLine()) != null) {
	if (! ( (ignoreCommentLines) && (inputLine.trim().startsWith("//"))))
	  inputBuffer += inputLine + " ";
	bytesRead += inputLine.length() + 2; // + <CR><LF>
      }
//			t = System.currentTimeMillis() - t;
      inStream.close();
//			if (t > 0) netSpeed = (int) ((bytesRead / t)*1000);
//			showStatus("Read " + bytesRead + " bytes from SML file " + addr + " in " + t + " ms (" + netSpeed + " bps)");
    }
    catch (Exception e) {
//			showStatus("Could not read file: " + addr + " - Exception: " + e + "\n");
    }
    return inputBuffer;
  }

  // ###################################################################
  //   File Access
  // ###################################################################

  public static Image fileGetImage(String path) {
    Image img = null;

    try {

      byte buffer[] = fileReadBinary(path);

      if (buffer.length == 0)
	return null;

      Toolkit tk = Toolkit.getDefaultToolkit();

      img = tk.createImage(buffer);

    }
    catch (Exception e) {
      return null;
    }

    return img;
  }

  public static String fileReadASCII(String path) {

    String inputBuffer = "";

    try {

      File file = new File(path);
      if ( (!file.exists()) || (!file.canRead()))
	return "";

      long length = file.length();

      BufferedReader reader = new BufferedReader(new FileReader(file));

      long bytesRead = 0;
      String inputLine;

      while ( (inputLine = reader.readLine()) != null) {
	// if (!((ignoreCommentLines) && (inputLine.trim().startsWith("//"))))
	inputBuffer += inputLine + " ";
	bytesRead += inputLine.length() + 2; // + <CR><LF>
      }
      reader.close();

    }
    catch (Exception e) {
      return "";
    }

    return inputBuffer;
  }

  public static byte[] fileReadBinary(String path) {

    byte error[] = new byte[0];
    FileInputStream is;
    int length;
    File file;

    try {

      file = new File(path);
      if ( (!file.exists()) || (!file.canRead()))
	return error;

      long llength = file.length();
      length = (int) llength;

      if (llength != (long) length)
	return error;

    }
    catch (Exception e) {
      return error;
    }

    byte buffer[] = new byte[length];

    int dl;
    int bytesRead = 0;

    try {
      is = new FileInputStream(file);
    }
    catch (Exception e) {
      return error;
    }

    try {
      bytesRead = is.read(buffer);
      is.close();

      if (bytesRead != length)
	return error;

    }
    catch (Exception e) {
      try {
	is.close();
      }
      catch (Exception ee) {}
      return error;
    }

    return buffer;

  }

  // ###################################################################
  //   Network Functions
  // ###################################################################

  public static boolean flushNetworkSocket(Socket mySocket) {

    if (mySocket == null) {
      return false;
    }

    byte arr[] = new byte[1000];
    int count = 0;

    try {
      InputStream myInputStream = mySocket.getInputStream();
      while (myInputStream.read(arr, 0, 1000) > 0) {
	count++;
      }
    }
    catch (Exception e) {
      return false;
    }
    if (count > 0)
      System.out.println("Network flushed " + count + " bytes...");
    return true;
  }

  public static byte[] readNetworkSocket(Socket mySocket, int len, int timeout) {

    if ( (mySocket == null) || (len <= 0)) {
      System.out.println("Socket NULL");
      return null;
    }

    InputStream myInputStream;
    BufferedInputStream bin;

    int lengthSoFar = 0;
    int ret;

    byte arr[] = new byte[len];
    byte nonread[] = new byte[0];

    long timeNow = System.currentTimeMillis();

    try {
      myInputStream = mySocket.getInputStream();
      bin = new BufferedInputStream(myInputStream);
      while (lengthSoFar < len) {
	if (System.currentTimeMillis() - timeNow > timeout) {
	  if (lengthSoFar == 0)
	    return nonread;
	  byte arr2[] = new byte[lengthSoFar];
	  for (int n = 0; n < lengthSoFar; n++)
	    arr2[n] = arr[n];
	  return arr2;
	}

	ret = myInputStream.read(arr, lengthSoFar, len - lengthSoFar);
	// ret = bin.read(arr, lengthSoFar, len-lengthSoFar);
	if (ret > 0) {
	  lengthSoFar += ret;
	}
	else {
	  Utils.pause(10);
	}
      }
    }
    catch (InterruptedIOException ioe) {
      // System.out.println("Socket Read Interrupted: " + ioe.toString());
      if (lengthSoFar == 0)
	return nonread;
      byte arr2[] = new byte[lengthSoFar];
      for (int n = 0; n < lengthSoFar; n++)
	arr2[n] = arr[n];
      return arr2;
    }
    catch (Exception e) {
      System.out.println("Socket Read Exception: " + e.toString());
      return null;
    }

    return arr;
  }

  public static boolean writeNetworkSocket(Socket mySocket, int len, byte[] arr) {

    if ( (mySocket == null) || (len <= 0) || (arr == null) || (arr.length == 0)) {
      return false;
    }

    OutputStream myOutputStream;
    // BufferedOutputStream bout;

    try {
      myOutputStream = mySocket.getOutputStream();
      // bout = BufferedOutputStream(myOutputStream, 2048);
      myOutputStream.write(arr);
    }
    catch (Exception e) {
      return false;
    }

    return true;
  }

  public static boolean isSocketConnected(Socket mySocket) {
    if (mySocket == null)return false;
    try {
      InputStream myInputStream = mySocket.getInputStream();
      int n = myInputStream.available();
      n = (int) myInputStream.skip(0);
      n = mySocket.getPort();
      InetAddress ia = mySocket.getInetAddress();
      if (ia.getHostAddress().length() == 0)
	return false;
    }
    catch (Exception ee) {
      return false;
    }
    return true;
  }

  public static Rectangle decodeStringRectangleAbs(String str) {
    int x = -1, y = -1, w = -1, h = -1, n, m, k;

    if (str == "")return null;

    if ( (n = str.indexOf(",")) != -1) {
      x = str2int(str.substring(0, n).trim());
      if ( (m = str.indexOf(",", n + 1)) != -1) {
	y = str2int(str.substring(n + 1, m).trim());
	if ( (k = str.indexOf(",", m + 1)) != -1) {
	  w = str2int(str.substring(m + 1, k).trim());
	  h = str2int(str.substring(k + 1).trim());
	  w -= x;
	  h -= y;
	}
	if ( (x != -1) && (y != -1) && (w != -1) && (h != -1)) {
	  return (new Rectangle(x, y, w, h));
	}
      }
    }
    return null;
  }

  public static boolean fileAppendASCII(String path, String text) {

    if ( (path == null) && (path.length() == 0))return false;

    try {

      File file = new File(path);

      FileWriter writer = new FileWriter(file);

      writer.write(text + "\n");

      writer.close();

    }
    catch (Exception e) {
      return false;
    }

    return true;
  }

  public static Vector fileGetDirsInDir(String dir) {

    Vector vect = new Vector();

    if (dir.length() == 0)
      return vect;

    File dirlist = new File(dir);

    if (!dirlist.exists())
      return vect;

    if (!dirlist.isDirectory())
      return vect;

    File list[] = dirlist.listFiles();

    File file;
    for (int n = 0; n < list.length; n++) {
      file = (File) list[n];
      if (file != null) {
	if (file.isDirectory())
	  vect.addElement(file.getName());
      }
    }

    return vect;
  }

  public static Vector fileGetDirsInDirAsFiles(String dir) {

    Vector vect = new Vector();

    if (dir.length() == 0)
      return vect;

    File dirlist = new File(dir);

    if (!dirlist.exists())
      return vect;

    if (!dirlist.isDirectory())
      return vect;

    File list[] = dirlist.listFiles();

    File file;
    for (int n = 0; n < list.length; n++) {
      file = (File) list[n];
      if (file != null) {
	if (file.isDirectory())
	  vect.addElement(file);
      }
    }

    return vect;
  }

  public static Vector fileGetFilesInDir(String dir) {

    Vector vect = new Vector();

    if (dir.length() == 0)
      return vect;

    File dirlist = new File(dir);

    if (!dirlist.exists())
      return vect;

    if (!dirlist.isDirectory())
      return vect;

    File list[] = dirlist.listFiles();

    File file;
    for (int n = 0; n < list.length; n++) {
      file = (File) list[n];
      if (file != null) {
	if (file.isFile())
	  vect.addElement(file.getName());
      }
    }

    return vect;
  }

  public static Vector fileGetFilesInDir(String dir, String extension) {

    Vector vect = new Vector();

    if (dir.length() == 0)
      return vect;

    File dirlist = new File(dir);

    if (!dirlist.exists())
      return vect;

    if (!dirlist.isDirectory())
      return vect;

    File list[] = dirlist.listFiles();

    File file;
    for (int n = 0; n < list.length; n++) {
      file = (File) list[n];
      if (file != null) {
	if ( (file.isFile()) && (file.getName().endsWith(extension)))
	  vect.addElement(file.getName());
      }
    }

    return vect;
  }

  public static Vector fileGetFilesInDirAsFiles(String dir) {

    Vector vect = new Vector();

    if (dir.length() == 0)
      return vect;

    File dirlist = new File(dir);

    if (!dirlist.exists())
      return vect;

    if (!dirlist.isDirectory())
      return vect;

    File list[] = dirlist.listFiles();

    File file;
    for (int n = 0; n < list.length; n++) {
      file = (File) list[n];
      if (file != null) {
	if (file.isFile())
	  vect.addElement(file);
      }
    }

    return vect;
  }

  public static Vector fileGetFilesInDirAsFiles(String dir, String extension) {

    Vector vect = new Vector();

    if (dir.length() == 0)
      return vect;

    File dirlist = new File(dir);

    if (!dirlist.exists())
      return vect;

    if (!dirlist.isDirectory())
      return vect;

    File list[] = dirlist.listFiles();

    File file;
    for (int n = 0; n < list.length; n++) {
      file = (File) list[n];
      if (file != null) {
	if ( (file.isFile()) && (file.getName().endsWith(extension)))
	  vect.addElement(file);
      }
    }

    return vect;
  }

  public static boolean fileWriteASCII(String path, String text) {

    if ( (path == null) && (path.length() == 0))return false;

    try {

      File file = new File(path);
      if (file.exists())
	file.delete();

      long length = file.length();

      FileWriter writer = new FileWriter(file);

      writer.write(text);

      writer.close();

    }
    catch (Exception e) {
      return false;
    }

    return true;
  }

  public static boolean fileWriteBinary(String path, byte[] data) {

    if ( (path == null) && (path.length() == 0))return false;

    try {

      File file = new File(path);
      if (file.exists())
	file.delete();

      long length = file.length();

      FileOutputStream writer = new FileOutputStream(file);

      writer.write(data);

      writer.close();

    }
    catch (Exception e) {
      return false;
    }

    return true;
  }

  public static TextCollection splitString(String text, String splittext) {

    TextCollection vect = new TextCollection();

    if ( (text == "") || (splittext == ""))return vect;

    int n = text.indexOf(splittext, 0);
    if (n < 0) {
      vect.add(text);
      return vect;
    }

    int pre = 0;
    String str;
    while (n > 0) {
      str = text.substring(pre, n);
      if (str.length() > 0)
	vect.add(str);
      pre = n + splittext.length();
      n = text.indexOf(splittext, pre);
    }

    str = text.substring(pre);
    if (str.length() > 0)
      vect.add(str);

    return vect;
  }

  public static TextDictionary splitString(String text, String splittext, String subsplit) {

    TextDictionary dict = new TextDictionary();
    TextCollection strings = splitString(text, splittext);

    TextCollection subs;
    for (int n=0; n<strings.getCount(); n++) {
      subs = splitString(strings.get(n), (subsplit));
      if (subs.getCount() > 1) {
	dict.put((String)subs.get(0), (String)subs.get(1));
      }
    }
    return dict;
  }

  public static XMLElement xmlGetRootNodeFromFile(String filename) {

    if ( (filename == null) || (filename.length() == 0))
      return null;

    String xml = Utils.fileReadASCII(filename);

    return Utils.xmlGetRootNodeFromXML(xml);
  }

  public static XMLElement xmlGetRootNodeFromXML(String xml) {

    if ( (xml == null) || (xml.length() == 0))
      return null;

    XMLElement node = new XMLElement();
    try {
      node.parseString(xml);
      return node;
    }
    catch(Exception e) {
      return node;
    }

/*
    DocumentBuilderFactory dbf = new org.apache.crimson.jaxp.DocumentBuilderFactoryImpl();

    // Optional: set various configuration options
    dbf.setValidating(false);
    dbf.setIgnoringComments(true);
    dbf.setIgnoringElementContentWhitespace(true);

    DocumentBuilder db = null;
    try {
      db = dbf.newDocumentBuilder();
    }
    catch (ParserConfigurationException pce) {
      System.err.println(pce);
      return null;
    }

    StringBufferInputStream is = new StringBufferInputStream(xml);

    Document doc = null;
    try {
      doc = db.parse(is);
    }
    catch (SAXException se) {
      System.err.println(se.getMessage());
      return null;
    }
    catch (IOException ioe) {
      System.err.println(ioe);
      return null;
    }

    // Now we have the root document node, now get the root node...

    return (Node) doc.getDocumentElement();
*/
  }
/*
  public static Hashtable xmlGetAtributes(Node node) {
    Hashtable table = new Hashtable();

    String key, val;
    NamedNodeMap atts = node.getAttributes();
    for (int i = 0; i < atts.getLength(); i++) {
      Node att = atts.item(i);
      if (att != null) {
	key = att.getNodeName();
	val = att.getNodeValue();
	if ( (key != null) && (key.length() > 0) && (val != null) )
	  table.put(key, val);
      }
    }

    return table;
  }
*/

  public static boolean looksLikeXML(String str) {
    String str2 = str.trim();
    return (str2.startsWith("<") && str2.endsWith(">"));
  }

  public static String xmlStringEncode(String str) {

/*    if (!( (str.indexOf('&') >= 0) ||
	 (str.indexOf('<') >= 0) ||
	 (str.indexOf('>') >= 0) ||
	 (str.indexOf('"') >= 0) ||
	 (str.indexOf('\'') >= 0) ||
	 (str.indexOf('\t') >= 0) ||
	 (str.indexOf('\r') >= 0) ||
	 (str.indexOf('\n') >= 0) ) )
      return str;*/

    StringWriter writer = new StringWriter();
    for (int i = 0; i < str.length(); i += 1) {
      char ch = str.charAt(i);
      switch (ch) {
	case '<':
	  writer.write('&');
	  writer.write('l');
	  writer.write('t');
	  writer.write(';');
	  break;
	case '>':
	  writer.write('&');
	  writer.write('g');
	  writer.write('t');
	  writer.write(';');
	  break;
	case '&':
	  writer.write('&');
	  writer.write('a');
	  writer.write('m');
	  writer.write('p');
	  writer.write(';');
	  break;
	case '"':
	  writer.write('&');
	  writer.write('q');
	  writer.write('u');
	  writer.write('o');
	  writer.write('t');
	  writer.write(';');
	  break;
	case '\'':
	  writer.write('&');
	  writer.write('a');
	  writer.write('p');
	  writer.write('o');
	  writer.write('s');
	  writer.write(';');
	  break;
	default:
	  int unicode = (int) ch;
	  if ( (unicode < 32) || (unicode > 126)) {
	    writer.write('&');
	    writer.write('#');
	    writer.write('x');
	    writer.write(Integer.toString(unicode & 0xFF, 16));
	    writer.write(';');
	  }
	  else {
	    writer.write(ch);
	  }
      }
    }
    return writer.toString();
  }

  public static boolean startsOrEndsWithWhiteSpace(String str) {
    return ( (str.length() > 0) && ( (str.charAt(0) < 33) || (str.charAt(str.length()-1) < 33) ) );
  }

  public static boolean startsWithWhiteSpace(String str) {
    return ( (str.length() > 0) && (str.charAt(0) < 33) );
  }

  public static boolean endsWithWhiteSpace(String str) {
    return ( (str.length() > 0) && (str.charAt(str.length()-1) < 33) );
  }

  public static String xmlIndent(String xml) {

    String outXML = "";
    TextCollection lines = splitString(xml, "\n");

    String line;
    for (int n = 0; n < lines.getCount(); n++) {
      line = (String) lines.get(n);
      if (line != null)
	outXML += "\t" + line + "\n";
    }

    return outXML;
  }

  public static String generateID() {
    return generateID("");
  }

  public static String generateID(String title) {
    UUIDGen uuidgen = new UUIDGen();
    if (title.length() > 0)
      return title + ":" + uuidgen.generateUUID();
    else
      return uuidgen.generateUUID();
   /* Random random = new Random();
    long timeNow = System.currentTimeMillis();
    int randomInt = random.nextInt(10000000) + 1;
    return "" + timeNow + randomInt;*/
  }





  public static String printListLine(int[] arr, String sep) {
    String str = "";

    if (arr.length == 0)
      return "";
    else if (arr.length == 1)
      return "" + arr[0];

    for (int n=0; n<arr.length-1; n++) {
      str += arr[n] + sep;
    }
    str += arr[arr.length-1];

    return str;
  }

  public static String printListLine(long[] arr, String sep) {
    String str = "";

    if (arr.length == 0)
      return "";
    else if (arr.length == 1)
      return "" + arr[0];

    for (int n=0; n<arr.length-1; n++) {
      str += arr[n] + sep;
    }
    str += arr[arr.length-1];

    return str;
  }

  public static String printListLine(Object[] arr, String sep) {
    String str = "";

    if (arr.length == 0)
      return "";
    else if (arr.length == 1)
      return "" + arr[0].toString();

    for (int n=0; n<arr.length-1; n++) {
      str += arr[n].toString() + sep;
    }
    str += arr[arr.length-1].toString();

    return str;
  }

  public static boolean writeIntToBytes(byte[] buffer, int offset, int size) {
    byte[] fourByteInt = int2bytes(size);
    buffer[offset] = fourByteInt[0];
    buffer[offset+1] = fourByteInt[1];
    buffer[offset+2] = fourByteInt[2];
    buffer[offset+3] = fourByteInt[3];
    return true;
  }

  public static boolean writeStringToBytes(byte[] buffer, int offset, String str) {
    System.arraycopy(str.getBytes(), 0, buffer, offset, str.length());
    return true;
  }

  public static boolean isPartOf(String a, String b) {

    TextCollection aa = Utils.splitString(a, ".");
    TextCollection bb = Utils.splitString(b, ".");

    int c = aa.getCount();
    if (c > bb.getCount())
      return false;
    for (int n=0; n<c; n++) {
      if (!aa.get(n).equalsIgnoreCase(bb.get(n)))
        return false;
    }
    return true;
  }

}
